@@ -118,7 +118,7 @@ struct PolygonContourWriter
     CPL_DISALLOW_COPY_ASSIGN(PolygonContourWriter)
 
     explicit PolygonContourWriter(OGRContourWriterInfo *poInfo, double minLevel)
-        : poInfo_(poInfo), previousLevel_(minLevel)
+        : poInfo_(poInfo), currentLevel_(minLevel)
     {
     }
 
@@ -128,6 +128,7 @@ struct PolygonContourWriter
         currentGeometry_.reset(new OGRMultiPolygon());
         currentLevel_ = level;
     }
+
     void endPolygon()
     {
         if (currentPart_)
@@ -160,6 +161,7 @@ struct PolygonContourWriter
         currentPart_ = new OGRPolygon();
         currentPart_->addRingDirectly(poNewRing);
     }
+
     void addInteriorRing(const marching_squares::LineString &ring)
     {
         OGRLinearRing *poNewRing = new OGRLinearRing();
@@ -173,8 +175,8 @@ struct PolygonContourWriter
     std::unique_ptr<OGRMultiPolygon> currentGeometry_ = {};
     OGRPolygon *currentPart_ = nullptr;
     OGRContourWriterInfo *poInfo_ = nullptr;
-    double currentLevel_ = 0;
-    double previousLevel_;
+    double currentLevel_;
+    double previousLevel_ = 0;
 };
 
 struct GDALRingAppender
@@ -560,6 +562,14 @@ CPLErr GDALContourGenerateEx(GDALRasterBandH hBand, void *hLayer,
     if (opt)
     {
         contourInterval = CPLAtof(opt);
+        // Written this way to catch NaN as well.
+        if (!(contourInterval > 0))
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Invalid value for LEVEL_INTERVAL. Should be strictly "
+                     "positive.");
+            return CE_Failure;
+        }
     }
 
     double contourBase = 0.0;
@@ -580,13 +590,19 @@ CPLErr GDALContourGenerateEx(GDALRasterBandH hBand, void *hLayer,
     opt = CSLFetchNameValue(options, "FIXED_LEVELS");
     if (opt)
     {
-        char **values = CSLTokenizeStringComplex(opt, ",", FALSE, FALSE);
-        fixedLevels.resize(CSLCount(values));
+        const CPLStringList aosLevels(
+            CSLTokenizeStringComplex(opt, ",", FALSE, FALSE));
+        fixedLevels.resize(aosLevels.size());
         for (size_t i = 0; i < fixedLevels.size(); i++)
         {
-            fixedLevels[i] = CPLAtof(values[i]);
+            fixedLevels[i] = CPLAtof(aosLevels[i]);
+            if (i > 0 && !(fixedLevels[i] >= fixedLevels[i - 1]))
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "FIXED_LEVELS should be strictly increasing");
+                return CE_Failure;
+            }
         }
-        CSLDestroy(values);
     }
 
     bool useNoData = false;
@@ -659,16 +675,44 @@ CPLErr GDALContourGenerateEx(GDALRasterBandH hBand, void *hLayer,
     {
         if (polygonize)
         {
-            int bSuccess;
-            PolygonContourWriter w(&oCWI,
-                                   GDALGetRasterMinimum(hBand, &bSuccess));
+            int bSuccessMin = FALSE;
+            double dfMinimum = GDALGetRasterMinimum(hBand, &bSuccessMin);
+            int bSuccessMax = FALSE;
+            double dfMaximum = GDALGetRasterMaximum(hBand, &bSuccessMax);
+            if ((!bSuccessMin || !bSuccessMax) && !fixedLevels.empty())
+            {
+                double adfMinMax[2];
+                if (GDALComputeRasterMinMax(hBand, false, adfMinMax) == CE_None)
+                {
+                    dfMinimum = adfMinMax[0];
+                    dfMaximum = adfMinMax[1];
+                }
+            }
+            if (!fixedLevels.empty())
+            {
+                // If the minimum raster value is larger than the first requested
+                // level, select the requested level that is just below the
+                // minimum raster value
+                if (fixedLevels[0] < dfMinimum)
+                {
+                    for (size_t i = 1; i < fixedLevels.size(); ++i)
+                    {
+                        if (fixedLevels[i] >= dfMinimum)
+                        {
+                            dfMinimum = fixedLevels[i - 1];
+                            break;
+                        }
+                    }
+                }
+            }
+
+            PolygonContourWriter w(&oCWI, dfMinimum);
             typedef PolygonRingAppender<PolygonContourWriter> RingAppender;
             RingAppender appender(w);
             if (!fixedLevels.empty())
             {
-                FixedLevelRangeIterator levels(
-                    &fixedLevels[0], fixedLevels.size(),
-                    GDALGetRasterMaximum(hBand, &bSuccess));
+                FixedLevelRangeIterator levels(&fixedLevels[0],
+                                               fixedLevels.size(), dfMaximum);
                 SegmentMerger<RingAppender, FixedLevelRangeIterator> writer(
                     appender, levels, /* polygonize */ true);
                 ContourGeneratorFromRaster<decltype(writer),
