@@ -105,7 +105,7 @@ hid_t GDAL_HDF5Open(const std::string &osFilename);
 class HDF5Dataset;
 class HDF5EOSParser;
 class BAGDataset;
-class S102Dataset;
+class S100BaseDataset;
 
 namespace GDAL
 {
@@ -119,7 +119,7 @@ class HDF5SharedResources
 {
     friend class ::HDF5Dataset;
     friend class ::BAGDataset;
-    friend class ::S102Dataset;
+    friend class ::S100BaseDataset;
 
     std::weak_ptr<HDF5SharedResources> m_poSelf{};
     bool m_bReadOnly = true;
@@ -151,6 +151,7 @@ class HDF5SharedResources
     {
         return m_hHDF5;
     }
+
     inline bool IsReadOnly() const
     {
         return m_bReadOnly;
@@ -239,8 +240,6 @@ class HDF5Dataset CPL_NON_FINAL : public GDALPamDataset
     char *CreatePath(HDF5GroupObjects *);
     static void DestroyH5Objects(HDF5GroupObjects *);
 
-    static const char *GetDataTypeName(hid_t);
-
     /**
      * Reads an array of double attributes from the HDF5 metadata.
      * It reads the attributes directly on its binary form directly,
@@ -273,8 +272,9 @@ class HDF5Dataset CPL_NON_FINAL : public GDALPamDataset
     static GDALDataset *OpenMultiDim(GDALOpenInfo *);
     static std::shared_ptr<GDALGroup> OpenGroup(
         const std::shared_ptr<GDAL::HDF5SharedResources> &poSharedResources);
-    static int Identify(GDALOpenInfo *);
 
+    static bool IsNativeCFloat16(hid_t hDataType);
+    static const char *GetDataTypeName(hid_t);
     static GDALDataType GetDataType(hid_t);
 };
 
