@@ -36,8 +36,8 @@
 #endif
 
 #ifdef _MSC_VER
-#pragma warning( push )
-#pragma warning( disable : 4458 )
+#pragma warning(push)
+#pragma warning(disable : 4458)
 #endif
 
 #include "jp2_local.h"
@@ -68,13 +68,25 @@
 // #define USE_JPIP
 
 #ifdef USE_JPIP
-#  include "kdu_client.h"
+#include "kdu_client.h"
 #else
-#  define kdu_client void
+#define kdu_client void
 #endif
 
 #ifdef _MSC_VER
-#pragma warning( pop )
+#pragma warning(pop)
+#endif
+
+// Before v7.5 Kakadu does not advertise its version well
+// After v7.5 Kakadu has KDU_{MAJOR,MINOR,PATCH}_VERSION defines so it is easier
+// For older releases compile with them manually specified
+#ifndef KDU_MAJOR_VERSION
+#error Compile with eg. -DKDU_MAJOR_VERSION=7 -DKDU_MINOR_VERSION=3 -DKDU_PATCH_VERSION=2 to specify Kakadu library version
+#endif
+
+#if KDU_MAJOR_VERSION > 7 || (KDU_MAJOR_VERSION == 7 && KDU_MINOR_VERSION >= 8)
+// Before Kakadu 7.8, kdu_roi_rect was missing from libkdu_aXY
+#define KDU_HAS_ROI_RECT
 #endif
 
 #endif  // JP2KAK_HEADERS_H
