@@ -50,6 +50,7 @@ class SG_Exception_NVOOB : public SG_Exception
                   std::string(dsname) + std::string(" was made"))
     {
     }
+
     const char *get_err_msg() override
     {
         return this->err_msg.c_str();
@@ -66,6 +67,7 @@ class SG_Exception_DupName : public SG_Exception
                   std::string(" already exists in") + std::string(dsname))
     {
     }
+
     const char *get_err_msg() override
     {
         return this->err_msg.c_str();
@@ -135,8 +137,7 @@ class netCDFVGeneralAttribute : public netCDFVAttribute
     VClass value;
 
   public:
-    netCDFVGeneralAttribute<VClass, ntype>(const char *a_name,
-                                           const VClass *a_value)
+    netCDFVGeneralAttribute(const char *a_name, const VClass *a_value)
         : name(a_name), value(*a_value)
     {
     }
@@ -193,7 +194,9 @@ class netCDFVDimension
     {
         this->r_did = realID;
     }
+
     void invalidate();
+
     void setLen(size_t len)
     {
         this->dim_len = len;
@@ -209,18 +212,22 @@ class netCDFVDimension
     {
         return this->real_dim_name;
     }
+
     size_t getLen()
     {
         return this->dim_len;
     }
+
     int getRealID()
     {
         return this->r_did;
     }
+
     int getVirtualID()
     {
         return this->v_did;
     }
+
     bool isValid()
     {
         return this->valid;
@@ -248,7 +255,9 @@ class netCDFVVariable
     {
         return attribs;
     }
+
     void invalidate();
+
     void setRealID(int realID)
     {
         this->r_vid = realID;
@@ -257,26 +266,32 @@ class netCDFVVariable
   public:
     netCDFVVariable(const char *name, nc_type xtype, int ndims,
                     const int *dimidsp);
+
     std::string &getName()
     {
         return real_var_name;
     }
+
     int getRealID()
     {
         return r_vid;
     }
+
     nc_type getType()
     {
         return ntype;
     }
+
     int getDimCount()
     {
         return this->ndimc;
     }
+
     const int *getDimIds()
     {
         return this->dimid.get();
     }
+
     bool isValid()
     {
         return this->valid;
@@ -444,20 +459,18 @@ class netCDFVID
     void nc_put_vvar1_text(int varid, const size_t *index, const char *value);
     void nc_put_vvara_text(int varid, const size_t *start, const size_t *index,
                            const char *value);
-
-#ifdef NETCDF_HAS_NC4
     void nc_put_vvar1_string(int varid, const size_t *index,
                              const char **value);
-#endif
 
     // Equivalent "enquiry" functions
     netCDFVVariable &
     virtualVIDToVar(int virtualID);  // converts a virtual var ID to a real ID
     netCDFVDimension &
     virtualDIDToDim(int virtualID);  // converts a virtual dim ID to a real ID
-    int nameToVirtualVID(std::string &name);
-    int nameToVirtualDID(std::string &name);
-    bool virtualVarNameDefined(std::string &nm)
+    int nameToVirtualVID(const std::string &name);
+    int nameToVirtualDID(const std::string &name);
+
+    bool virtualVarNameDefined(const std::string &nm)
     {
         return nameVarTable.count(nm) > 0;
     }
