@@ -67,8 +67,8 @@ namespace PCIDSK
 
         std::string     GetRst() override { return ""; }
         std::vector<double> GetProjection( std::string &geosys ) override;
-        void            SetProjection(std::string geosys,
-                                      std::vector<double> params) override;
+        void            SetProjection(const std::string& geosys,
+                                      const std::vector<double>& params) override;
 
         int             GetFieldCount() override;
         std::string     GetFieldName(int) override;
@@ -88,9 +88,9 @@ namespace PCIDSK
         void            GetVertices( ShapeId, std::vector<ShapeVertex>& ) override;
         void            GetFields( ShapeId, std::vector<ShapeField>& ) override;
 
-        void            AddField( std::string name, ShapeFieldType type,
-                                  std::string description,
-                                  std::string format,
+        void            AddField( const std::string& name, ShapeFieldType type,
+                                  const std::string& description,
+                                  const std::string& format,
                                   ShapeField *default_value ) override;
 
         ShapeId         CreateShape( ShapeId id ) override;
