@@ -81,16 +81,19 @@ class MFFDataset final : public RawDataset
     char **GetFileList() override;
 
     int GetGCPCount() override;
+
     const OGRSpatialReference *GetGCPSpatialRef() const override
     {
         return m_oGCPSRS.IsEmpty() ? nullptr : &m_oGCPSRS;
     }
+
     const GDAL_GCP *GetGCPs() override;
 
     const OGRSpatialReference *GetSpatialRef() const override
     {
         return m_oSRS.IsEmpty() ? nullptr : &m_oSRS;
     }
+
     CPLErr GetGeoTransform(double *) override;
 
     static GDALDataset *Open(GDALOpenInfo *);
@@ -208,6 +211,7 @@ class MFFSpheroidList : public SpheroidList
 {
   public:
     MFFSpheroidList();
+
     ~MFFSpheroidList()
     {
     }
@@ -648,7 +652,7 @@ void MFFDataset::ScanForProjectionInfo()
     }
 
     m_oSRS = oProj;
-    m_oGCPSRS = oProj;
+    m_oGCPSRS = std::move(oProj);
 
     if (!transform_ok)
     {
@@ -729,7 +733,7 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Create a corresponding GDALDataset.                             */
     /* -------------------------------------------------------------------- */
-    auto poDS = cpl::make_unique<MFFDataset>();
+    auto poDS = std::make_unique<MFFDataset>();
 
     poDS->papszHdrLines = papszHdrLines;
 
@@ -821,7 +825,8 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
                 continue;
 
             pszExtension = CPLGetExtension(papszDirFiles[i]);
-            if (strlen(pszExtension) >= 2 && isdigit(pszExtension[1]) &&
+            if (strlen(pszExtension) >= 2 &&
+                isdigit(static_cast<unsigned char>(pszExtension[1])) &&
                 atoi(pszExtension + 1) == nRawBand &&
                 strchr("bBcCiIjJrRxXzZ", pszExtension[0]) != nullptr)
                 break;
@@ -937,7 +942,7 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
 
         if (bTiled)
         {
-            poBand = cpl::make_unique<MFFTiledBand>(poDS.get(), nBand, fpRaw,
+            poBand = std::make_unique<MFFTiledBand>(poDS.get(), nBand, fpRaw,
                                                     nTileXSize, nTileYSize,
                                                     eDataType, eByteOrder);
         }
@@ -1248,7 +1253,7 @@ GDALDataset *MFFDataset::CreateCopy(const char *pszFilename,
         GDALRasterBand *poSrcBand = poSrcDS->GetRasterBand(iBand + 1);
         GDALRasterBand *poDstBand = poDS->GetRasterBand(iBand + 1);
 
-        void *pData = CPLMalloc(nBlockXSize * nBlockYSize *
+        void *pData = CPLMalloc(static_cast<size_t>(nBlockXSize) * nBlockYSize *
                                 GDALGetDataTypeSizeBytes(eType));
 
         for (int iYOffset = 0; iYOffset < nYSize; iYOffset += nBlockYSize)
