@@ -1,5 +1,5 @@
 /*****************************************************************************
- * $Id: gdalhttp.h 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $
+ * $Id$
  *
  * Project:  WMS Client Driver
  * Purpose:  Implementation of Dataset and RasterBand classes for WMS
@@ -34,9 +34,14 @@
 #include "cpl_port.h"
 #include "cpl_http.h"
 
-struct WMSHTTPRequest {
+struct WMSHTTPRequest
+{
     WMSHTTPRequest()
-        :options(nullptr), nStatus(0), pabyData(nullptr), nDataLen(0), nDataAlloc(0), m_curl_handle(nullptr), m_headers(nullptr), x(0), y(0) {}
+        : options(nullptr), nStatus(0), pabyData(nullptr), nDataLen(0),
+          nDataAlloc(0), m_curl_handle(nullptr), m_headers(nullptr), x(0), y(0)
+    {
+    }
+
     ~WMSHTTPRequest();
 
     /* Input */
@@ -49,18 +54,19 @@ struct WMSHTTPRequest {
     CPLString ContentType;
     CPLString Error;
 
-    int nStatus;  /* 200 = success, 404 = not found, 0 = no response / error */
+    int nStatus; /* 200 = success, 404 = not found, 0 = no response / error */
     GByte *pabyData;
     size_t nDataLen;
     size_t nDataAlloc;
 
     /* curl internal stuff */
     CURL *m_curl_handle;
-    struct curl_slist* m_headers;
+    struct curl_slist *m_headers;
     // Which tile is being requested
     int x, y;
 
-    // Space for error message, doesn't seem to be used by the multi-request interface
+    // Space for error message, doesn't seem to be used by the multi-request
+    // interface
     std::vector<char> m_curl_error;
 };
 
