@@ -43,6 +43,7 @@
 #include "minidriver_mrf.h"
 #include "minidriver_ogcapimaps.h"
 #include "minidriver_ogcapicoverage.h"
+#include "wmsdrivercore.h"
 
 #include "cpl_json.h"
 
@@ -67,18 +68,20 @@ static CPLXMLNode *GDALWMSDatasetGetConfigFromURL(GDALOpenInfo *poOpenInfo)
     if (STARTS_WITH_CI(pszBaseURL, "WMS:"))
         pszBaseURL += 4;
 
-    CPLString osLayer = CPLURLGetValue(pszBaseURL, "LAYERS");
+    const CPLString osLayer = CPLURLGetValue(pszBaseURL, "LAYERS");
     CPLString osVersion = CPLURLGetValue(pszBaseURL, "VERSION");
-    CPLString osSRS = CPLURLGetValue(pszBaseURL, "SRS");
-    CPLString osCRS = CPLURLGetValue(pszBaseURL, "CRS");
+    const CPLString osSRS = CPLURLGetValue(pszBaseURL, "SRS");
+    const CPLString osCRS = CPLURLGetValue(pszBaseURL, "CRS");
     CPLString osBBOX = CPLURLGetValue(pszBaseURL, "BBOX");
     CPLString osFormat = CPLURLGetValue(pszBaseURL, "FORMAT");
-    CPLString osTransparent = CPLURLGetValue(pszBaseURL, "TRANSPARENT");
+    const CPLString osTransparent = CPLURLGetValue(pszBaseURL, "TRANSPARENT");
 
     /* GDAL specific extensions to alter the default settings */
-    CPLString osOverviewCount = CPLURLGetValue(pszBaseURL, "OVERVIEWCOUNT");
-    CPLString osTileSize = CPLURLGetValue(pszBaseURL, "TILESIZE");
-    CPLString osMinResolution = CPLURLGetValue(pszBaseURL, "MINRESOLUTION");
+    const CPLString osOverviewCount =
+        CPLURLGetValue(pszBaseURL, "OVERVIEWCOUNT");
+    const CPLString osTileSize = CPLURLGetValue(pszBaseURL, "TILESIZE");
+    const CPLString osMinResolution =
+        CPLURLGetValue(pszBaseURL, "MINRESOLUTION");
     CPLString osBBOXOrder = CPLURLGetValue(pszBaseURL, "BBOXORDER");
 
     CPLString osBaseURL = pszBaseURL;
@@ -715,81 +718,6 @@ static CPLXMLNode *GDALWMSDatasetGetConfigFromArcGISJSON(const char *pszURL,
     return CPLParseXMLString(osXML);
 }
 
-/************************************************************************/
-/*                             Identify()                               */
-/************************************************************************/
-
-int GDALWMSDataset::Identify(GDALOpenInfo *poOpenInfo)
-{
-    const char *pszFilename = poOpenInfo->pszFilename;
-    const char *pabyHeader = (const char *)poOpenInfo->pabyHeader;
-    if (poOpenInfo->nHeaderBytes == 0 &&
-        STARTS_WITH_CI(pszFilename, "<GDAL_WMS>"))
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes >= 10 &&
-             STARTS_WITH_CI(pabyHeader, "<GDAL_WMS>"))
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes == 0 &&
-             (STARTS_WITH_CI(pszFilename, "WMS:") ||
-              CPLString(pszFilename).ifind("SERVICE=WMS") != std::string::npos))
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes != 0 &&
-             (strstr(pabyHeader, "<WMT_MS_Capabilities") != nullptr ||
-              strstr(pabyHeader, "<WMS_Capabilities") != nullptr ||
-              strstr(pabyHeader, "<!DOCTYPE WMT_MS_Capabilities") != nullptr))
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes != 0 &&
-             strstr(pabyHeader, "<WMS_Tile_Service") != nullptr)
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes != 0 &&
-             strstr(pabyHeader, "<TileMap version=\"1.0.0\"") != nullptr)
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes != 0 &&
-             strstr(pabyHeader, "<Services") != nullptr &&
-             strstr(pabyHeader, "<TileMapService version=\"1.0") != nullptr)
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes != 0 &&
-             strstr(pabyHeader, "<TileMapService version=\"1.0.0\"") != nullptr)
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes == 0 &&
-             STARTS_WITH_CI(pszFilename, "http") &&
-             (strstr(pszFilename, "/MapServer?f=json") != nullptr ||
-              strstr(pszFilename, "/MapServer/?f=json") != nullptr ||
-              strstr(pszFilename, "/ImageServer?f=json") != nullptr ||
-              strstr(pszFilename, "/ImageServer/?f=json") != nullptr))
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes == 0 &&
-             STARTS_WITH_CI(pszFilename, "AGS:"))
-    {
-        return TRUE;
-    }
-    else if (poOpenInfo->nHeaderBytes == 0 &&
-             STARTS_WITH_CI(pszFilename, "IIP:"))
-    {
-        return TRUE;
-    }
-    else
-        return FALSE;
-}
-
 /************************************************************************/
 /*                                 Open()                               */
 /************************************************************************/
@@ -802,7 +730,7 @@ GDALDataset *GDALWMSDataset::Open(GDALOpenInfo *poOpenInfo)
     const char *pszFilename = poOpenInfo->pszFilename;
     const char *pabyHeader = (const char *)poOpenInfo->pabyHeader;
 
-    if (!Identify(poOpenInfo))
+    if (!WMSDriverIdentify(poOpenInfo))
         return nullptr;
 
     if (poOpenInfo->nHeaderBytes == 0 &&
@@ -1145,7 +1073,7 @@ void WMSDeregister(CPL_UNUSED GDALDriver *d)
 void GDALRegister_WMS()
 
 {
-    if (GDALGetDriverByName("WMS") != nullptr)
+    if (GDALGetDriverByName(DRIVER_NAME) != nullptr)
         return;
 
     // Register all minidrivers here
@@ -1162,16 +1090,9 @@ void GDALRegister_WMS()
     RegisterMinidriver(OGCAPICoverage);
 
     GDALDriver *poDriver = new GDALDriver();
-
-    poDriver->SetDescription("WMS");
-    poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_LONGNAME, "OGC Web Map Service");
-    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "drivers/raster/wms.html");
-    poDriver->SetMetadataItem(GDAL_DCAP_VIRTUALIO, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_SUBDATASETS, "YES");
+    WMSDriverSetCommonMetadata(poDriver);
 
     poDriver->pfnOpen = GDALWMSDataset::Open;
-    poDriver->pfnIdentify = GDALWMSDataset::Identify;
     poDriver->pfnUnloadDriver = WMSDeregister;
     poDriver->pfnCreateCopy = GDALWMSDataset::CreateCopy;
 
