@@ -27,6 +27,7 @@
  ****************************************************************************/
 
 #include "zarr.h"
+#include "zarrdrivercore.h"
 
 #include "cpl_minixml.h"
 
@@ -47,51 +48,6 @@ ZarrDataset::ZarrDataset(const std::shared_ptr<GDALGroup> &poRootGroup)
 {
 }
 
-/************************************************************************/
-/*                    CheckExistenceOfOneZarrFile()                     */
-/************************************************************************/
-
-static bool CheckExistenceOfOneZarrFile(const char *pszFilename)
-{
-
-    CPLString osMDFilename = CPLFormFilename(pszFilename, ".zarray", nullptr);
-
-    VSIStatBufL sStat;
-    if (VSIStatL(osMDFilename, &sStat) == 0)
-        return true;
-
-    osMDFilename = CPLFormFilename(pszFilename, ".zgroup", nullptr);
-    if (VSIStatL(osMDFilename, &sStat) == 0)
-        return true;
-
-    // Zarr V3
-    osMDFilename = CPLFormFilename(pszFilename, "zarr.json", nullptr);
-    if (VSIStatL(osMDFilename, &sStat) == 0)
-        return true;
-
-    return false;
-}
-
-/************************************************************************/
-/*                              Identify()                              */
-/************************************************************************/
-
-int ZarrDataset::Identify(GDALOpenInfo *poOpenInfo)
-
-{
-    if (STARTS_WITH(poOpenInfo->pszFilename, "ZARR:"))
-    {
-        return TRUE;
-    }
-
-    if (!poOpenInfo->bIsDirectory)
-    {
-        return FALSE;
-    }
-
-    return CheckExistenceOfOneZarrFile(poOpenInfo->pszFilename);
-}
-
 /************************************************************************/
 /*                           OpenMultidim()                             */
 /************************************************************************/
@@ -107,10 +63,9 @@ GDALDataset *ZarrDataset::OpenMultidim(const char *pszFilename,
     auto poSharedResource = ZarrSharedResource::Create(osFilename, bUpdateMode);
     poSharedResource->SetOpenOptions(papszOpenOptionsIn);
 
-    auto poRG = poSharedResource->OpenRootGroup();
+    auto poRG = poSharedResource->GetRootGroup();
     if (!poRG)
         return nullptr;
-    poSharedResource->SetRootGroup(poRG);
     return new ZarrDataset(poRG);
 }
 
@@ -182,22 +137,113 @@ char **ZarrDataset::GetMetadata(const char *pszDomain)
     return nullptr;
 }
 
+/************************************************************************/
+/*                      GetXYDimensionIndices()                         */
+/************************************************************************/
+
+static void GetXYDimensionIndices(const std::shared_ptr<GDALMDArray> &poArray,
+                                  const GDALOpenInfo *poOpenInfo, size_t &iXDim,
+                                  size_t &iYDim)
+{
+    const size_t nDims = poArray->GetDimensionCount();
+    iYDim = nDims >= 2 ? nDims - 2 : 0;
+    iXDim = nDims >= 2 ? nDims - 1 : 0;
+
+    if (nDims >= 2)
+    {
+        const char *pszDimX =
+            CSLFetchNameValue(poOpenInfo->papszOpenOptions, "DIM_X");
+        const char *pszDimY =
+            CSLFetchNameValue(poOpenInfo->papszOpenOptions, "DIM_Y");
+        bool bFoundX = false;
+        bool bFoundY = false;
+        const auto &apoDims = poArray->GetDimensions();
+        for (size_t i = 0; i < nDims; ++i)
+        {
+            if (pszDimX && apoDims[i]->GetName() == pszDimX)
+            {
+                bFoundX = true;
+                iXDim = i;
+            }
+            else if (pszDimY && apoDims[i]->GetName() == pszDimY)
+            {
+                bFoundY = true;
+                iYDim = i;
+            }
+            else if (!pszDimX &&
+                     (apoDims[i]->GetType() == GDAL_DIM_TYPE_HORIZONTAL_X ||
+                      apoDims[i]->GetName() == "X"))
+                iXDim = i;
+            else if (!pszDimY &&
+                     (apoDims[i]->GetType() == GDAL_DIM_TYPE_HORIZONTAL_Y ||
+                      apoDims[i]->GetName() == "Y"))
+                iYDim = i;
+        }
+        if (pszDimX)
+        {
+            if (!bFoundX && CPLGetValueType(pszDimX) == CPL_VALUE_INTEGER)
+            {
+                const int nTmp = atoi(pszDimX);
+                if (nTmp >= 0 && nTmp <= static_cast<int>(nDims))
+                {
+                    iXDim = nTmp;
+                    bFoundX = true;
+                }
+            }
+            if (!bFoundX)
+            {
+                CPLError(CE_Warning, CPLE_AppDefined,
+                         "Cannot find dimension DIM_X=%s", pszDimX);
+            }
+        }
+        if (pszDimY)
+        {
+            if (!bFoundY && CPLGetValueType(pszDimY) == CPL_VALUE_INTEGER)
+            {
+                const int nTmp = atoi(pszDimY);
+                if (nTmp >= 0 && nTmp <= static_cast<int>(nDims))
+                {
+                    iYDim = nTmp;
+                    bFoundY = true;
+                }
+            }
+            if (!bFoundY)
+            {
+                CPLError(CE_Warning, CPLE_AppDefined,
+                         "Cannot find dimension DIM_Y=%s", pszDimY);
+            }
+        }
+    }
+}
+
+/************************************************************************/
+/*                       GetExtraDimSampleCount()                       */
+/************************************************************************/
+
+static uint64_t
+GetExtraDimSampleCount(const std::shared_ptr<GDALMDArray> &poArray,
+                       size_t iXDim, size_t iYDim)
+{
+    uint64_t nExtraDimSamples = 1;
+    const auto &apoDims = poArray->GetDimensions();
+    for (size_t i = 0; i < apoDims.size(); ++i)
+    {
+        if (i != iXDim && i != iYDim)
+            nExtraDimSamples *= apoDims[i]->GetSize();
+    }
+    return nExtraDimSamples;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
 GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
 {
-    if (!Identify(poOpenInfo))
+    if (!ZARRDriverIdentify(poOpenInfo))
     {
         return nullptr;
     }
-    if (poOpenInfo->eAccess == GA_Update &&
-        (poOpenInfo->nOpenFlags & GDAL_OF_MULTIDIM_RASTER) == 0)
-    {
-        CPLError(CE_Failure, CPLE_NotSupported, "Update not supported");
-        return nullptr;
-    }
 
     CPLString osFilename(poOpenInfo->pszFilename);
     CPLString osArrayOfInterest;
@@ -209,36 +255,27 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
         if (aosTokens.size() < 2)
             return nullptr;
         osFilename = aosTokens[1];
-        std::string osErrorMsgSuffix;
+        std::string osErrorMsg;
         if (osFilename == "http" || osFilename == "https")
         {
-            osErrorMsgSuffix = "\nThere is likely a quoting error of the whole "
-                               "connection string, and the filename should "
-                               "likely be prefixed with /vsicurl/";
+            osErrorMsg = "There is likely a quoting error of the whole "
+                         "connection string, and the filename should "
+                         "likely be prefixed with /vsicurl/";
         }
         else if (osFilename == "/vsicurl/http" ||
                  osFilename == "/vsicurl/https")
         {
-            osErrorMsgSuffix = "\nThere is likely a quoting error of the whole "
-                               "connection string.";
+            osErrorMsg = "There is likely a quoting error of the whole "
+                         "connection string.";
         }
-        if (!osErrorMsgSuffix.empty() ||
-            !CheckExistenceOfOneZarrFile(osFilename.c_str()))
+        else if (STARTS_WITH(osFilename.c_str(), "http://") ||
+                 STARTS_WITH(osFilename.c_str(), "https://"))
+        {
+            osErrorMsg =
+                "The filename should likely be prefixed with /vsicurl/";
+        }
+        if (!osErrorMsg.empty())
         {
-            std::string osErrorMsg(
-                "Cannot find any of Zarr metadata files in '");
-            osErrorMsg += osFilename;
-            osErrorMsg += "'.";
-            if (!osErrorMsgSuffix.empty())
-            {
-                osErrorMsg += osErrorMsgSuffix;
-            }
-            else if (STARTS_WITH(osFilename.c_str(), "http://") ||
-                     STARTS_WITH(osFilename.c_str(), "https://"))
-            {
-                osErrorMsg +=
-                    "\nThe filename should likely be prefixed with /vsicurl/";
-            }
             CPLError(CE_Failure, CPLE_AppDefined, "%s", osErrorMsg.c_str());
             return nullptr;
         }
@@ -264,10 +301,15 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
 
     auto poRG = poDSMultiDim->GetRootGroup();
 
-    auto poDS = cpl::make_unique<ZarrDataset>(nullptr);
+    auto poDS = std::make_unique<ZarrDataset>(nullptr);
     std::shared_ptr<GDALMDArray> poMainArray;
     std::vector<std::string> aosArrays;
     std::string osMainArray;
+    const bool bMultiband = CPLTestBool(
+        CSLFetchNameValueDef(poOpenInfo->papszOpenOptions, "MULTIBAND", "YES"));
+    size_t iXDim = 0;
+    size_t iYDim = 0;
+
     if (!osArrayOfInterest.empty())
     {
         poMainArray = osArrayOfInterest == "/"
@@ -275,15 +317,42 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
                           : poRG->OpenMDArrayFromFullname(osArrayOfInterest);
         if (poMainArray == nullptr)
             return nullptr;
+        GetXYDimensionIndices(poMainArray, poOpenInfo, iXDim, iYDim);
+
         if (poMainArray->GetDimensionCount() > 2)
         {
             if (anExtraDimIndices.empty())
             {
-                uint64_t nExtraDimSamples = 1;
-                const auto &apoDims = poMainArray->GetDimensions();
-                for (size_t i = 2; i < apoDims.size(); ++i)
-                    nExtraDimSamples *= apoDims[i]->GetSize();
-                if (nExtraDimSamples != 1)
+                const uint64_t nExtraDimSamples =
+                    GetExtraDimSampleCount(poMainArray, iXDim, iYDim);
+                if (bMultiband)
+                {
+                    if (nExtraDimSamples > 65536)  // arbitrary limit
+                    {
+                        if (poMainArray->GetDimensionCount() == 3)
+                        {
+                            CPLError(CE_Warning, CPLE_AppDefined,
+                                     "Too many samples along the > 2D "
+                                     "dimensions of %s. "
+                                     "Use ZARR:\"%s\":%s:{i} syntax",
+                                     osArrayOfInterest.c_str(),
+                                     osFilename.c_str(),
+                                     osArrayOfInterest.c_str());
+                        }
+                        else
+                        {
+                            CPLError(CE_Warning, CPLE_AppDefined,
+                                     "Too many samples along the > 2D "
+                                     "dimensions of %s. "
+                                     "Use ZARR:\"%s\":%s:{i}:{j} syntax",
+                                     osArrayOfInterest.c_str(),
+                                     osFilename.c_str(),
+                                     osArrayOfInterest.c_str());
+                        }
+                        return nullptr;
+                    }
+                }
+                else if (nExtraDimSamples != 1)
                 {
                     CPLError(CE_Failure, CPLE_AppDefined,
                              "Indices of extra dimensions must be specified");
@@ -305,6 +374,7 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
                     if (poMainArray == nullptr)
                         return nullptr;
                 }
+                GetXYDimensionIndices(poMainArray, poOpenInfo, iXDim, iYDim);
             }
         }
         else if (!anExtraDimIndices.empty())
@@ -334,7 +404,7 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
                 {
                     if (osMainArray.empty())
                     {
-                        poMainArray = poArray;
+                        poMainArray = std::move(poArray);
                         osMainArray = osArrayName;
                     }
                     else
@@ -347,15 +417,17 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
             }
         }
 
+        if (poMainArray)
+            GetXYDimensionIndices(poMainArray, poOpenInfo, iXDim, iYDim);
+
         int iCountSubDS = 1;
 
         if (poMainArray && poMainArray->GetDimensionCount() > 2)
         {
-            uint64_t nExtraDimSamples = 1;
             const auto &apoDims = poMainArray->GetDimensions();
-            for (size_t i = 0; i < apoDims.size() - 2; ++i)
-                nExtraDimSamples *= apoDims[i]->GetSize();
-            if (nExtraDimSamples > 1024)  // arbitrary limit
+            const uint64_t nExtraDimSamples =
+                GetExtraDimSampleCount(poMainArray, iXDim, iYDim);
+            if (nExtraDimSamples > 65536)  // arbitrary limit
             {
                 if (apoDims.size() == 3)
                 {
@@ -376,6 +448,10 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
                         osMainArray.c_str());
                 }
             }
+            else if (nExtraDimSamples > 1 && bMultiband)
+            {
+                // nothing to do
+            }
             else if (nExtraDimSamples > 1 && apoDims.size() == 3)
             {
                 for (int i = 0; i < static_cast<int>(nExtraDimSamples); ++i)
@@ -432,13 +508,17 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
         }
     }
 
-    if (poMainArray && poMainArray->GetDimensionCount() <= 2)
+    if (poMainArray && (bMultiband || poMainArray->GetDimensionCount() <= 2))
     {
-        std::unique_ptr<GDALDataset> poNewDS;
-        if (poMainArray->GetDimensionCount() == 2)
-        {
-            poNewDS.reset(poMainArray->AsClassicDataset(1, 0));
+        // Pass papszOpenOptions for LOAD_EXTRA_DIM_METADATA_DELAY
+        auto poNewDS =
+            std::unique_ptr<GDALDataset>(poMainArray->AsClassicDataset(
+                iXDim, iYDim, poRG, poOpenInfo->papszOpenOptions));
+        if (!poNewDS)
+            return nullptr;
 
+        if (poMainArray->GetDimensionCount() >= 2)
+        {
             // If we have 3 arrays, check that the 2 ones that are not the main
             // 2D array are indexing variables of its dimensions. If so, don't
             // expose them as subdatasets
@@ -456,7 +536,8 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
                 for (int i = 0; i < 2; i++)
                 {
                     auto poIndexingVar =
-                        poMainArray->GetDimensions()[i]->GetIndexingVariable();
+                        poMainArray->GetDimensions()[i == 0 ? iXDim : iYDim]
+                            ->GetIndexingVariable();
                     if (poIndexingVar)
                     {
                         for (int j = 0; j < 2; j++)
@@ -476,12 +557,6 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
                 }
             }
         }
-        else
-        {
-            poNewDS.reset(poMainArray->AsClassicDataset(0, 0));
-        }
-        if (!poNewDS)
-            return nullptr;
         if (!poDS->m_aosSubdatasets.empty())
         {
             poNewDS->SetMetadata(poDS->m_aosSubdatasets.List(), "SUBDATASETS");
@@ -492,6 +567,59 @@ GDALDataset *ZarrDataset::Open(GDALOpenInfo *poOpenInfo)
     return poDS.release();
 }
 
+/************************************************************************/
+/*                       ZarrDatasetDelete()                            */
+/************************************************************************/
+
+static CPLErr ZarrDatasetDelete(const char *pszFilename)
+{
+    if (STARTS_WITH(pszFilename, "ZARR:"))
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Delete() only supported on ZARR connection names "
+                 "not starting with the ZARR: prefix");
+        return CE_Failure;
+    }
+    return VSIRmdirRecursive(pszFilename) == 0 ? CE_None : CE_Failure;
+}
+
+/************************************************************************/
+/*                       ZarrDatasetRename()                            */
+/************************************************************************/
+
+static CPLErr ZarrDatasetRename(const char *pszNewName, const char *pszOldName)
+{
+    if (STARTS_WITH(pszNewName, "ZARR:") || STARTS_WITH(pszOldName, "ZARR:"))
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Rename() only supported on ZARR connection names "
+                 "not starting with the ZARR: prefix");
+        return CE_Failure;
+    }
+    return VSIRename(pszOldName, pszNewName) == 0 ? CE_None : CE_Failure;
+}
+
+/************************************************************************/
+/*                       ZarrDatasetCopyFiles()                         */
+/************************************************************************/
+
+static CPLErr ZarrDatasetCopyFiles(const char *pszNewName,
+                                   const char *pszOldName)
+{
+    if (STARTS_WITH(pszNewName, "ZARR:") || STARTS_WITH(pszOldName, "ZARR:"))
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "CopyFiles() only supported on ZARR connection names "
+                 "not starting with the ZARR: prefix");
+        return CE_Failure;
+    }
+    // VSISync() returns true in case of success
+    return VSISync((std::string(pszOldName) + '/').c_str(), pszNewName, nullptr,
+                   nullptr, nullptr, nullptr)
+               ? CE_None
+               : CE_Failure;
+}
+
 /************************************************************************/
 /*                           ZarrDriver()                               */
 /************************************************************************/
@@ -803,6 +931,34 @@ void ZarrDriver::InitMetadata()
                 "V2 only)");
             CPLAddXMLAttributeAndValue(psCreateZMetadata, "default", "YES");
 
+            auto psSingleArrayNode =
+                CPLCreateXMLNode(oTree.get(), CXT_Element, "Option");
+            CPLAddXMLAttributeAndValue(psSingleArrayNode, "name",
+                                       "SINGLE_ARRAY");
+            CPLAddXMLAttributeAndValue(psSingleArrayNode, "type", "boolean");
+            CPLAddXMLAttributeAndValue(
+                psSingleArrayNode, "description",
+                "Whether to write a multi-band dataset as a single array, or "
+                "one array per band");
+            CPLAddXMLAttributeAndValue(psSingleArrayNode, "default", "YES");
+
+            auto psInterleaveNode =
+                CPLCreateXMLNode(oTree.get(), CXT_Element, "Option");
+            CPLAddXMLAttributeAndValue(psInterleaveNode, "name", "INTERLEAVE");
+            CPLAddXMLAttributeAndValue(psInterleaveNode, "type",
+                                       "string-select");
+            CPLAddXMLAttributeAndValue(psInterleaveNode, "default", "BAND");
+            {
+                auto poValueNode =
+                    CPLCreateXMLNode(psInterleaveNode, CXT_Element, "Value");
+                CPLCreateXMLNode(poValueNode, CXT_Text, "BAND");
+            }
+            {
+                auto poValueNode =
+                    CPLCreateXMLNode(psInterleaveNode, CXT_Element, "Value");
+                CPLCreateXMLNode(poValueNode, CXT_Text, "PIXEL");
+            }
+
             char *pszXML = CPLSerializeXMLTree(oTree.get());
             GDALDriver::SetMetadataItem(GDAL_DMD_CREATIONOPTIONLIST, pszXML);
             CPLFree(pszXML);
@@ -826,7 +982,7 @@ ZarrDataset::CreateMultiDimensional(const char *pszFilename,
         ZarrSharedResource::Create(pszFilename, /*bUpdatable=*/true);
     if (EQUAL(pszFormat, "ZARR_V3"))
     {
-        poRG = ZarrGroupV3::CreateOnDisk(poSharedResource, std::string(), "/",
+        poRG = ZarrV3Group::CreateOnDisk(poSharedResource, std::string(), "/",
                                          pszFilename);
     }
     else
@@ -837,12 +993,11 @@ ZarrDataset::CreateMultiDimensional(const char *pszFilename,
         {
             poSharedResource->EnableZMetadata();
         }
-        poRG = ZarrGroupV2::CreateOnDisk(poSharedResource, std::string(), "/",
+        poRG = ZarrV2Group::CreateOnDisk(poSharedResource, std::string(), "/",
                                          pszFilename);
     }
     if (!poRG)
         return nullptr;
-    poSharedResource->SetRootGroup(poRG);
 
     auto poDS = new ZarrDataset(poRG);
     poDS->SetDescription(pszFilename);
@@ -895,7 +1050,7 @@ GDALDataset *ZarrDataset::Create(const char *pszName, int nXSize, int nYSize,
             ZarrSharedResource::Create(pszName, /*bUpdatable=*/true);
         if (EQUAL(pszFormat, "ZARR_V3"))
         {
-            poRG = ZarrGroupV3::CreateOnDisk(poSharedResource, std::string(),
+            poRG = ZarrV3Group::CreateOnDisk(poSharedResource, std::string(),
                                              "/", pszName);
         }
         else
@@ -906,7 +1061,7 @@ GDALDataset *ZarrDataset::Create(const char *pszName, int nXSize, int nYSize,
             {
                 poSharedResource->EnableZMetadata();
             }
-            poRG = ZarrGroupV2::CreateOnDisk(poSharedResource, std::string(),
+            poRG = ZarrV2Group::CreateOnDisk(poSharedResource, std::string(),
                                              "/", pszName);
         }
         poSharedResource->SetRootGroup(poRG);
@@ -914,7 +1069,7 @@ GDALDataset *ZarrDataset::Create(const char *pszName, int nXSize, int nYSize,
     if (!poRG)
         return nullptr;
 
-    auto poDS = cpl::make_unique<ZarrDataset>(poRG);
+    auto poDS = std::make_unique<ZarrDataset>(poRG);
     poDS->SetDescription(pszName);
     poDS->nRasterYSize = nYSize;
     poDS->nRasterXSize = nXSize;
@@ -958,26 +1113,116 @@ GDALDataset *ZarrDataset::Create(const char *pszName, int nXSize, int nYSize,
     }
     if (poDS->m_poDimY == nullptr || poDS->m_poDimX == nullptr)
         return nullptr;
-    const auto aoDims = std::vector<std::shared_ptr<GDALDimension>>{
-        poDS->m_poDimY, poDS->m_poDimX};
 
-    for (int i = 0; i < nBandsIn; i++)
+    const bool bSingleArray =
+        CPLTestBool(CSLFetchNameValueDef(papszOptions, "SINGLE_ARRAY", "YES"));
+    const bool bBandInterleave =
+        EQUAL(CSLFetchNameValueDef(papszOptions, "INTERLEAVE", "BAND"), "BAND");
+    const std::shared_ptr<GDALDimension> poBandDim(
+        (bSingleArray && nBandsIn > 1)
+            ? poRG->CreateDimension("Band", std::string(), std::string(),
+                                    nBandsIn)
+            : nullptr);
+
+    const char *pszNonNullArrayName =
+        pszArrayName ? pszArrayName : CPLGetBasename(pszName);
+    if (poBandDim)
     {
-        auto poArray = poRG->CreateMDArray(
-            pszArrayName
-                ? (nBandsIn == 1 ? pszArrayName
-                                 : CPLSPrintf("%s_band%d", pszArrayName, i + 1))
-                : (nBandsIn == 1 ? CPLGetBasename(pszName)
-                                 : CPLSPrintf("Band%d", i + 1)),
-            aoDims, GDALExtendedDataType::Create(eType), papszOptions);
-        if (poArray == nullptr)
+        const std::vector<std::shared_ptr<GDALDimension>> apoDims(
+            bBandInterleave
+                ? std::vector<std::shared_ptr<GDALDimension>>{poBandDim,
+                                                              poDS->m_poDimY,
+                                                              poDS->m_poDimX}
+                : std::vector<std::shared_ptr<GDALDimension>>{
+                      poDS->m_poDimY, poDS->m_poDimX, poBandDim});
+        poDS->m_poSingleArray = poRG->CreateMDArray(
+            pszNonNullArrayName, apoDims, GDALExtendedDataType::Create(eType),
+            papszOptions);
+        if (!poDS->m_poSingleArray)
             return nullptr;
-        poDS->SetBand(i + 1, new ZarrRasterBand(poArray));
+        poDS->SetMetadataItem("INTERLEAVE", bBandInterleave ? "BAND" : "PIXEL",
+                              "IMAGE_STRUCTURE");
+        for (int i = 0; i < nBandsIn; i++)
+        {
+            auto poSlicedArray = poDS->m_poSingleArray->GetView(
+                CPLSPrintf(bBandInterleave ? "[%d,::,::]" : "[::,::,%d]", i));
+            poDS->SetBand(i + 1, new ZarrRasterBand(poSlicedArray));
+        }
+    }
+    else
+    {
+        const auto apoDims = std::vector<std::shared_ptr<GDALDimension>>{
+            poDS->m_poDimY, poDS->m_poDimX};
+        for (int i = 0; i < nBandsIn; i++)
+        {
+            auto poArray = poRG->CreateMDArray(
+                nBandsIn == 1  ? pszNonNullArrayName
+                : pszArrayName ? CPLSPrintf("%s_band%d", pszArrayName, i + 1)
+                               : CPLSPrintf("Band%d", i + 1),
+                apoDims, GDALExtendedDataType::Create(eType), papszOptions);
+            if (poArray == nullptr)
+                return nullptr;
+            poDS->SetBand(i + 1, new ZarrRasterBand(poArray));
+        }
     }
 
     return poDS.release();
 }
 
+/************************************************************************/
+/*                           ~ZarrDataset()                             */
+/************************************************************************/
+
+ZarrDataset::~ZarrDataset()
+{
+    ZarrDataset::FlushCache(true);
+}
+
+/************************************************************************/
+/*                            FlushCache()                              */
+/************************************************************************/
+
+CPLErr ZarrDataset::FlushCache(bool bAtClosing)
+{
+    CPLErr eErr = GDALDataset::FlushCache(bAtClosing);
+    if (m_poSingleArray)
+    {
+        bool bFound = false;
+        for (int i = 0; i < nBands; ++i)
+        {
+            if (papoBands[i]->GetColorInterpretation() != GCI_Undefined)
+                bFound = true;
+        }
+        if (bFound)
+        {
+            const auto oStringDT = GDALExtendedDataType::CreateString();
+            auto poAttr = m_poSingleArray->GetAttribute("COLOR_INTERPRETATION");
+            if (!poAttr)
+                poAttr = m_poSingleArray->CreateAttribute(
+                    "COLOR_INTERPRETATION", {static_cast<GUInt64>(nBands)},
+                    oStringDT);
+            if (poAttr)
+            {
+                const GUInt64 nStartIndex = 0;
+                const size_t nCount = nBands;
+                const GInt64 arrayStep = 1;
+                const GPtrDiff_t bufferStride = 1;
+                std::vector<const char *> apszValues;
+                for (int i = 0; i < nBands; ++i)
+                {
+                    const auto eColorInterp =
+                        papoBands[i]->GetColorInterpretation();
+                    apszValues.push_back(
+                        GDALGetColorInterpretationName(eColorInterp));
+                }
+                poAttr->Write(&nStartIndex, &nCount, &arrayStep, &bufferStride,
+                              oStringDT, apszValues.data());
+            }
+        }
+    }
+    return eErr;
+}
+
 /************************************************************************/
 /*                          GetSpatialRef()                             */
 /************************************************************************/
@@ -1112,10 +1357,14 @@ CPLErr ZarrDataset::SetMetadata(char **papszMetadata, const char *pszDomain)
     if (nBands >= 1 && (pszDomain == nullptr || pszDomain[0] == '\0'))
     {
         const auto oStringDT = GDALExtendedDataType::CreateString();
-        for (int i = 0; i < nBands; ++i)
+        const auto bSingleArray = m_poSingleArray != nullptr;
+        const int nIters = bSingleArray ? 1 : nBands;
+        for (int i = 0; i < nIters; ++i)
         {
-            auto &poArray =
-                cpl::down_cast<ZarrRasterBand *>(papoBands[i])->m_poArray;
+            auto *poArray = bSingleArray
+                                ? m_poSingleArray.get()
+                                : cpl::down_cast<ZarrRasterBand *>(papoBands[i])
+                                      ->m_poArray.get();
             for (auto iter = papszMetadata; iter && *iter; ++iter)
             {
                 char *pszKey = nullptr;
@@ -1283,6 +1532,47 @@ CPLErr ZarrRasterBand::SetUnitType(const char *pszNewValue)
                                                               : CE_Failure;
 }
 
+/************************************************************************/
+/*                      GetColorInterpretation()                        */
+/************************************************************************/
+
+GDALColorInterp ZarrRasterBand::GetColorInterpretation()
+{
+    return m_eColorInterp;
+}
+
+/************************************************************************/
+/*                      SetColorInterpretation()                        */
+/************************************************************************/
+
+CPLErr ZarrRasterBand::SetColorInterpretation(GDALColorInterp eColorInterp)
+{
+    auto poGDS = cpl::down_cast<ZarrDataset *>(poDS);
+    m_eColorInterp = eColorInterp;
+    if (!poGDS->m_poSingleArray)
+    {
+        const auto oStringDT = GDALExtendedDataType::CreateString();
+        auto poAttr = m_poArray->GetAttribute("COLOR_INTERPRETATION");
+        if (poAttr && (poAttr->GetDimensionCount() != 0 ||
+                       poAttr->GetDataType().GetClass() != GEDTC_STRING))
+            return CE_None;
+        if (!poAttr)
+            poAttr = m_poArray->CreateAttribute("COLOR_INTERPRETATION", {},
+                                                oStringDT);
+        if (poAttr)
+        {
+            const GUInt64 nStartIndex = 0;
+            const size_t nCount = 1;
+            const GInt64 arrayStep = 1;
+            const GPtrDiff_t bufferStride = 1;
+            const char *pszValue = GDALGetColorInterpretationName(eColorInterp);
+            poAttr->Write(&nStartIndex, &nCount, &arrayStep, &bufferStride,
+                          oStringDT, &pszValue);
+        }
+    }
+    return CE_None;
+}
+
 /************************************************************************/
 /*                    ZarrRasterBand::IReadBlock()                      */
 /************************************************************************/
@@ -1382,47 +1672,18 @@ CPLErr ZarrRasterBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
 void GDALRegister_Zarr()
 
 {
-    if (GDALGetDriverByName("Zarr") != nullptr)
+    if (GDALGetDriverByName(DRIVER_NAME) != nullptr)
         return;
 
     GDALDriver *poDriver = new ZarrDriver();
+    ZARRDriverSetCommonMetadata(poDriver);
 
-    poDriver->SetDescription("Zarr");
-    poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
-    poDriver->SetMetadataItem(GDAL_DCAP_MULTIDIM_RASTER, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_LONGNAME, "Zarr");
-    poDriver->SetMetadataItem(GDAL_DMD_CREATIONDATATYPES,
-                              "Byte Int16 UInt16 Int32 UInt32 Int64 UInt64 "
-                              "Float32 Float64 CFloat32 CFloat64");
-    poDriver->SetMetadataItem(GDAL_DCAP_VIRTUALIO, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_SUBDATASETS, "YES");
-
-    poDriver->SetMetadataItem(
-        GDAL_DMD_OPENOPTIONLIST,
-        "<OpenOptionList>"
-        "   <Option name='USE_ZMETADATA' type='boolean' description='Whether "
-        "to use consolidated metadata from .zmetadata' default='YES'/>"
-        "   <Option name='CACHE_TILE_PRESENCE' type='boolean' "
-        "description='Whether to establish an initial listing of present "
-        "tiles' default='NO'/>"
-        "</OpenOptionList>");
-
-    poDriver->SetMetadataItem(
-        GDAL_DMD_MULTIDIM_DATASET_CREATIONOPTIONLIST,
-        "<MultiDimDatasetCreationOptionList>"
-        "   <Option name='FORMAT' type='string-select' default='ZARR_V2'>"
-        "     <Value>ZARR_V2</Value>"
-        "     <Value>ZARR_V3</Value>"
-        "   </Option>"
-        "   <Option name='CREATE_ZMETADATA' type='boolean' "
-        "description='Whether to create consolidated metadata into .zmetadata "
-        "(Zarr V2 only)' default='YES'/>"
-        "</MultiDimDatasetCreationOptionList>");
-
-    poDriver->pfnIdentify = ZarrDataset::Identify;
     poDriver->pfnOpen = ZarrDataset::Open;
     poDriver->pfnCreateMultiDimensional = ZarrDataset::CreateMultiDimensional;
     poDriver->pfnCreate = ZarrDataset::Create;
+    poDriver->pfnDelete = ZarrDatasetDelete;
+    poDriver->pfnRename = ZarrDatasetRename;
+    poDriver->pfnCopyFiles = ZarrDatasetCopyFiles;
 
     GetGDALDriverManager()->RegisterDriver(poDriver);
 }
