@@ -32,6 +32,7 @@
 
 #include "cpl_port.h"
 
+#include <cmath>
 #include <cstdint>
 #include <limits>
 
@@ -54,13 +55,13 @@ inline void GDALGetDataLimits(Tin &tMaxValue, Tin &tMinValue)
     tMinValue = std::numeric_limits<Tin>::min();
 
     // Compute the actual minimum value of Tout in terms of Tin.
-    if (std::numeric_limits<Tout>::is_signed &&
-        std::numeric_limits<Tout>::is_integer)
+    if constexpr (std::numeric_limits<Tout>::is_signed &&
+                  std::numeric_limits<Tout>::is_integer)
     {
         // the minimum value is less than zero
-        if (std::numeric_limits<Tout>::digits <
-                std::numeric_limits<Tin>::digits ||
-            !std::numeric_limits<Tin>::is_integer)
+        if constexpr (std::numeric_limits<Tout>::digits <
+                          std::numeric_limits<Tin>::digits ||
+                      !std::numeric_limits<Tin>::is_integer)
         {
             // Tout is smaller than Tin, so we need to clamp values in input
             // to the range of Tout's min/max values
@@ -71,12 +72,12 @@ inline void GDALGetDataLimits(Tin &tMaxValue, Tin &tMinValue)
             tMaxValue = static_cast<Tin>(std::numeric_limits<Tout>::max());
         }
     }
-    else if (std::numeric_limits<Tout>::is_integer)
+    else if constexpr (std::numeric_limits<Tout>::is_integer)
     {
         // the output is unsigned, so we just need to determine the max
         /* coverity[same_on_both_sides] */
-        if (std::numeric_limits<Tout>::digits <=
-            std::numeric_limits<Tin>::digits)
+        if constexpr (std::numeric_limits<Tout>::digits <=
+                      std::numeric_limits<Tin>::digits)
         {
             // Tout is smaller than Tin, so we need to clamp the input values
             // to the range of Tout's max
@@ -102,6 +103,34 @@ inline T GDALClampValue(const T tValue, const T tMax, const T tMin)
     return tValue > tMax ? tMax : tValue < tMin ? tMin : tValue;
 }
 
+/************************************************************************/
+/*                          GDALClampDoubleValue()                            */
+/************************************************************************/
+/**
+ * Clamp double values to a specified range, this uses the same
+ * argument ordering as std::clamp, returns TRUE if the value was clamped.
+ *
+ * @param tValue the value
+ * @param tMin the min value
+ * @param tMax the max value
+ *
+ */
+template <class T2, class T3>
+inline bool GDALClampDoubleValue(double &tValue, const T2 tMin, const T3 tMax)
+{
+    const double tMin2{static_cast<double>(tMin)};
+    const double tMax2{static_cast<double>(tMax)};
+    if (tValue > tMax2 || tValue < tMin2)
+    {
+        tValue = tValue > tMax2 ? tMax2 : tValue < tMin2 ? tMin2 : tValue;
+        return true;
+    }
+    else
+    {
+        return false;
+    }
+}
+
 /************************************************************************/
 /*                         GDALIsValueInRange()                         */
 /************************************************************************/
@@ -114,7 +143,7 @@ inline T GDALClampValue(const T tValue, const T tMax, const T tMin)
  */
 template <class T> inline bool GDALIsValueInRange(double dfValue)
 {
-    return dfValue >= static_cast<double>(std::numeric_limits<T>::min()) &&
+    return dfValue >= static_cast<double>(std::numeric_limits<T>::lowest()) &&
            dfValue <= static_cast<double>(std::numeric_limits<T>::max());
 }
 
@@ -130,6 +159,57 @@ template <> inline bool GDALIsValueInRange<float>(double dfValue)
             dfValue <= std::numeric_limits<float>::max());
 }
 
+template <> inline bool GDALIsValueInRange<int64_t>(double dfValue)
+{
+    // Values in the range [INT64_MAX - 1023, INT64_MAX - 1]
+    // get converted to a double that once cast to int64_t is
+    // INT64_MAX + 1, hence the < strict comparison.
+    return dfValue >=
+               static_cast<double>(std::numeric_limits<int64_t>::min()) &&
+           dfValue < static_cast<double>(std::numeric_limits<int64_t>::max());
+}
+
+template <> inline bool GDALIsValueInRange<uint64_t>(double dfValue)
+{
+    // Values in the range [UINT64_MAX - 2047, UINT64_MAX - 1]
+    // get converted to a double that once cast to uint64_t is
+    // UINT64_MAX + 1, hence the < strict comparison.
+    return dfValue >= 0 &&
+           dfValue < static_cast<double>(std::numeric_limits<uint64_t>::max());
+}
+
+/************************************************************************/
+/*                         GDALIsValueExactAs()                         */
+/************************************************************************/
+/**
+ * Returns whether a value can be exactly represented on type T.
+ *
+ * That is static_cast\<double\>(static_cast\<T\>(dfValue)) is legal and is
+ * equal to dfValue.
+ *
+ * Note: for T=float or double, a NaN input leads to true
+ *
+ * @param dfValue the value
+ * @return whether the value can be exactly represented on type T.
+ */
+template <class T> inline bool GDALIsValueExactAs(double dfValue)
+{
+    return GDALIsValueInRange<T>(dfValue) &&
+           static_cast<double>(static_cast<T>(dfValue)) == dfValue;
+}
+
+template <> inline bool GDALIsValueExactAs<float>(double dfValue)
+{
+    return std::isnan(dfValue) ||
+           (GDALIsValueInRange<float>(dfValue) &&
+            static_cast<double>(static_cast<float>(dfValue)) == dfValue);
+}
+
+template <> inline bool GDALIsValueExactAs<double>(double)
+{
+    return true;
+}
+
 /************************************************************************/
 /*                          GDALCopyWord()                              */
 /************************************************************************/
@@ -581,6 +661,7 @@ inline void GDALCopy4Words(const float *pValueIn, GUInt16 *const pValueOut)
 #ifdef __AVX2__
 
 #include <immintrin.h>
+
 template <>
 inline void GDALCopy8Words(const float *pValueIn, GByte *const pValueOut)
 {
