@@ -33,6 +33,7 @@
 
 #include <algorithm>
 #include <cstring>
+#include <utility>
 
 #include "cpl_conv.h"
 #include "cpl_error.h"
@@ -46,24 +47,50 @@
 /************************************************************************/
 
 GDALNoDataMaskBand::GDALNoDataMaskBand(GDALRasterBand *poParentIn)
-    : poParent(poParentIn)
+    : m_poParent(poParentIn)
 {
     poDS = nullptr;
     nBand = 0;
 
-    nRasterXSize = poParent->GetXSize();
-    nRasterYSize = poParent->GetYSize();
+    nRasterXSize = m_poParent->GetXSize();
+    nRasterYSize = m_poParent->GetYSize();
 
     eDataType = GDT_Byte;
-    poParent->GetBlockSize(&nBlockXSize, &nBlockYSize);
+    m_poParent->GetBlockSize(&nBlockXSize, &nBlockYSize);
 
-    const auto eParentDT = poParent->GetRasterDataType();
+    const auto eParentDT = m_poParent->GetRasterDataType();
     if (eParentDT == GDT_Int64)
-        nNoDataValueInt64 = poParent->GetNoDataValueAsInt64();
+        m_nNoDataValueInt64 = m_poParent->GetNoDataValueAsInt64();
     else if (eParentDT == GDT_UInt64)
-        nNoDataValueUInt64 = poParent->GetNoDataValueAsUInt64();
+        m_nNoDataValueUInt64 = m_poParent->GetNoDataValueAsUInt64();
     else
-        dfNoDataValue = poParent->GetNoDataValue();
+        m_dfNoDataValue = m_poParent->GetNoDataValue();
+}
+
+/************************************************************************/
+/*                        GDALNoDataMaskBand()                          */
+/************************************************************************/
+
+GDALNoDataMaskBand::GDALNoDataMaskBand(GDALRasterBand *poParentIn,
+                                       double dfNoDataValue)
+    : m_poParent(poParentIn)
+{
+    poDS = nullptr;
+    nBand = 0;
+
+    nRasterXSize = m_poParent->GetXSize();
+    nRasterYSize = m_poParent->GetYSize();
+
+    eDataType = GDT_Byte;
+    m_poParent->GetBlockSize(&nBlockXSize, &nBlockYSize);
+
+    const auto eParentDT = m_poParent->GetRasterDataType();
+    if (eParentDT == GDT_Int64)
+        m_nNoDataValueInt64 = static_cast<int64_t>(dfNoDataValue);
+    else if (eParentDT == GDT_UInt64)
+        m_nNoDataValueUInt64 = static_cast<uint64_t>(dfNoDataValue);
+    else
+        m_dfNoDataValue = dfNoDataValue;
 }
 
 /************************************************************************/
@@ -140,7 +167,6 @@ bool GDALNoDataMaskBand::IsNoDataInRange(double dfNoDataValue,
         {
             return GDALIsValueInRange<GUInt32>(dfNoDataValue);
         }
-
         case GDT_Int32:
         {
             return GDALIsValueInRange<GInt32>(dfNoDataValue);
@@ -213,7 +239,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
     {
         return CE_Failure;
     }
-    const auto eParentDT = poParent->GetRasterDataType();
+    const auto eParentDT = m_poParent->GetRasterDataType();
     const GDALDataType eWrkDT = GetWorkDataType(eParentDT);
 
     // Optimization in common use case (#4488).
@@ -221,14 +247,14 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
     // reducing the global block cache consumption.
     if (eBufType == GDT_Byte && eWrkDT == GDT_Byte)
     {
-        const CPLErr eErr = poParent->RasterIO(
+        const CPLErr eErr = m_poParent->RasterIO(
             GF_Read, nXOff, nYOff, nXSize, nYSize, pData, nBufXSize, nBufYSize,
             eBufType, nPixelSpace, nLineSpace, psExtraArg);
         if (eErr != CE_None)
             return eErr;
 
         GByte *pabyData = static_cast<GByte *>(pData);
-        const GByte byNoData = static_cast<GByte>(dfNoDataValue);
+        const GByte byNoData = static_cast<GByte>(m_dfNoDataValue);
 
         if (nPixelSpace == 1 && nLineSpace == nBufXSize)
         {
@@ -253,28 +279,75 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
         return CE_None;
     }
 
+    const auto AllocTempBufferOrFallback =
+        [this, eRWFlag, nXOff, nYOff, nXSize, nYSize, pData, nBufXSize,
+         nBufYSize, eBufType, nPixelSpace, nLineSpace,
+         psExtraArg](int nWrkDTSize) -> std::pair<CPLErr, void *>
+    {
+        auto poParentDS = m_poParent->GetDataset();
+        // Check if we must simulate a memory allocation failure
+        // Before checking the env variable, which is slightly expensive,
+        // check first for a special dataset name, which is a cheap test.
+        const char *pszOptVal =
+            poParentDS && strcmp(poParentDS->GetDescription(), "__debug__") == 0
+                ? CPLGetConfigOption(
+                      "GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND", "NO")
+                : "NO";
+        const bool bSimulMemAllocFailure =
+            EQUAL(pszOptVal, "ALWAYS") ||
+            (CPLTestBool(pszOptVal) &&
+             GDALMajorObject::GetMetadataItem(__func__, "__INTERNAL__") ==
+                 nullptr);
+        void *pTemp = nullptr;
+        if (!bSimulMemAllocFailure)
+        {
+            CPLErrorStateBackuper oErrorStateBackuper(CPLQuietErrorHandler);
+            pTemp = VSI_MALLOC3_VERBOSE(nWrkDTSize, nBufXSize, nBufYSize);
+        }
+        if (!pTemp)
+        {
+            const bool bAllocHasAlreadyFailed =
+                GDALMajorObject::GetMetadataItem(__func__, "__INTERNAL__") !=
+                nullptr;
+            CPLError(bAllocHasAlreadyFailed ? CE_Failure : CE_Warning,
+                     CPLE_OutOfMemory,
+                     "GDALNoDataMaskBand::IRasterIO(): cannot allocate %d x %d "
+                     "x %d bytes%s",
+                     nBufXSize, nBufYSize, nWrkDTSize,
+                     bAllocHasAlreadyFailed
+                         ? ""
+                         : ". Falling back to block-based approach");
+            if (bAllocHasAlreadyFailed)
+                return std::pair(CE_Failure, nullptr);
+            // Sets a metadata item to prevent potential infinite recursion
+            GDALMajorObject::SetMetadataItem(__func__, "IN", "__INTERNAL__");
+            const CPLErr eErr = GDALRasterBand::IRasterIO(
+                eRWFlag, nXOff, nYOff, nXSize, nYSize, pData, nBufXSize,
+                nBufYSize, eBufType, nPixelSpace, nLineSpace, psExtraArg);
+            GDALMajorObject::SetMetadataItem(__func__, nullptr, "__INTERNAL__");
+            return std::pair(eErr, nullptr);
+        }
+        return std::pair(CE_None, pTemp);
+    };
+
     if (eBufType == GDT_Byte)
     {
         const int nWrkDTSize = GDALGetDataTypeSizeBytes(eWrkDT);
-        void *pTemp = VSI_MALLOC3_VERBOSE(nWrkDTSize, nBufXSize, nBufYSize);
-        if (pTemp == nullptr)
-        {
-            return GDALRasterBand::IRasterIO(
-                eRWFlag, nXOff, nYOff, nXSize, nYSize, pTemp, nBufXSize,
-                nBufYSize, eWrkDT, nWrkDTSize, nBufXSize * nWrkDTSize,
-                psExtraArg);
-        }
+        auto [eErr, pTemp] = AllocTempBufferOrFallback(nWrkDTSize);
+        if (!pTemp)
+            return eErr;
 
-        const CPLErr eErr = poParent->RasterIO(
+        eErr = m_poParent->RasterIO(
             GF_Read, nXOff, nYOff, nXSize, nYSize, pTemp, nBufXSize, nBufYSize,
-            eWrkDT, nWrkDTSize, nBufXSize * nWrkDTSize, psExtraArg);
+            eWrkDT, nWrkDTSize, static_cast<GSpacing>(nBufXSize) * nWrkDTSize,
+            psExtraArg);
         if (eErr != CE_None)
         {
             VSIFree(pTemp);
             return eErr;
         }
 
-        const bool bIsNoDataNan = CPLIsNan(dfNoDataValue) != 0;
+        const bool bIsNoDataNan = CPLIsNan(m_dfNoDataValue) != 0;
         GByte *pabyDest = static_cast<GByte *>(pData);
 
         /* --------------------------------------------------------------------
@@ -286,7 +359,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
         {
             case GDT_UInt32:
             {
-                const GUInt32 nNoData = static_cast<GUInt32>(dfNoDataValue);
+                const GUInt32 nNoData = static_cast<GUInt32>(m_dfNoDataValue);
                 const GUInt32 *panSrc = static_cast<const GUInt32 *>(pTemp);
 
                 size_t i = 0;
@@ -305,7 +378,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
 
             case GDT_Int32:
             {
-                const GInt32 nNoData = static_cast<GInt32>(dfNoDataValue);
+                const GInt32 nNoData = static_cast<GInt32>(m_dfNoDataValue);
                 const GInt32 *panSrc = static_cast<const GInt32 *>(pTemp);
 
                 size_t i = 0;
@@ -324,7 +397,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
 
             case GDT_Float32:
             {
-                const float fNoData = static_cast<float>(dfNoDataValue);
+                const float fNoData = static_cast<float>(m_dfNoDataValue);
                 const float *pafSrc = static_cast<const float *>(pTemp);
 
                 size_t i = 0;
@@ -360,7 +433,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                         const double dfVal = padfSrc[i];
                         if (bIsNoDataNan && CPLIsNan(dfVal))
                             *pabyLineDest = 0;
-                        else if (ARE_REAL_EQUAL(dfVal, dfNoDataValue))
+                        else if (ARE_REAL_EQUAL(dfVal, m_dfNoDataValue))
                             *pabyLineDest = 0;
                         else
                             *pabyLineDest = 255;
@@ -382,7 +455,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                     for (int iX = 0; iX < nBufXSize; iX++)
                     {
                         const auto nVal = panSrc[i];
-                        if (nVal == nNoDataValueInt64)
+                        if (nVal == m_nNoDataValueInt64)
                             *pabyLineDest = 0;
                         else
                             *pabyLineDest = 255;
@@ -404,7 +477,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                     for (int iX = 0; iX < nBufXSize; iX++)
                     {
                         const auto nVal = panSrc[i];
-                        if (nVal == nNoDataValueUInt64)
+                        if (nVal == m_nNoDataValueUInt64)
                             *pabyLineDest = 0;
                         else
                             *pabyLineDest = 255;
@@ -426,30 +499,27 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
 
     // Output buffer is non-Byte. Ask for Byte and expand to user requested
     // type
-    GByte *pabyBuf =
-        static_cast<GByte *>(VSI_MALLOC2_VERBOSE(nBufXSize, nBufYSize));
-    if (pabyBuf == nullptr)
-    {
-        return GDALRasterBand::IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize,
-                                         pData, nBufXSize, nBufYSize, eBufType,
-                                         nPixelSpace, nLineSpace, psExtraArg);
-    }
-    const CPLErr eErr =
-        IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize, pabyBuf, nBufXSize,
-                  nBufYSize, GDT_Byte, 1, nBufXSize, psExtraArg);
+    auto [eErr, pTemp] = AllocTempBufferOrFallback(sizeof(GByte));
+    if (!pTemp)
+        return eErr;
+
+    eErr = IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize, pTemp, nBufXSize,
+                     nBufYSize, GDT_Byte, 1, nBufXSize, psExtraArg);
     if (eErr != CE_None)
     {
-        VSIFree(pabyBuf);
+        VSIFree(pTemp);
         return eErr;
     }
 
     for (int iY = 0; iY < nBufYSize; iY++)
     {
-        GDALCopyWords(pabyBuf + static_cast<size_t>(iY) * nBufXSize, GDT_Byte,
-                      1, static_cast<GByte *>(pData) + iY * nLineSpace,
-                      eBufType, static_cast<int>(nPixelSpace), nBufXSize);
+        GDALCopyWords(
+            static_cast<GByte *>(pTemp) + static_cast<size_t>(iY) * nBufXSize,
+            GDT_Byte, 1, static_cast<GByte *>(pData) + iY * nLineSpace,
+            eBufType, static_cast<int>(nPixelSpace), nBufXSize);
     }
-    VSIFree(pabyBuf);
+    VSIFree(pTemp);
     return CE_None;
 }
+
 //! @endcond
