@@ -279,16 +279,22 @@ OGRErr OGRPolyhedralSurface::importFromWkb(const unsigned char *pabyData,
 /*                            exportToWkb()                             */
 /************************************************************************/
 
-OGRErr OGRPolyhedralSurface::exportToWkb(OGRwkbByteOrder eByteOrder,
-                                         unsigned char *pabyData,
-                                         OGRwkbVariant /*eWkbVariant*/) const
+OGRErr
+OGRPolyhedralSurface::exportToWkb(unsigned char *pabyData,
+                                  const OGRwkbExportOptions *psOptions) const
 
 {
+    if (!psOptions)
+    {
+        static const OGRwkbExportOptions defaultOptions;
+        psOptions = &defaultOptions;
+    }
+
     /* -------------------------------------------------------------------- */
     /*      Set the byte order.                                             */
     /* -------------------------------------------------------------------- */
-    pabyData[0] =
-        DB2_V72_UNFIX_BYTE_ORDER(static_cast<unsigned char>(eByteOrder));
+    pabyData[0] = DB2_V72_UNFIX_BYTE_ORDER(
+        static_cast<unsigned char>(psOptions->eByteOrder));
 
     /* -------------------------------------------------------------------- */
     /*      Set the geometry feature type, ensuring that 3D flag is         */
@@ -296,7 +302,7 @@ OGRErr OGRPolyhedralSurface::exportToWkb(OGRwkbByteOrder eByteOrder,
     /* -------------------------------------------------------------------- */
     GUInt32 nGType = getIsoGeometryType();
 
-    if (OGR_SWAP(eByteOrder))
+    if (OGR_SWAP(psOptions->eByteOrder))
     {
         nGType = CPL_SWAP32(nGType);
     }
@@ -304,7 +310,7 @@ OGRErr OGRPolyhedralSurface::exportToWkb(OGRwkbByteOrder eByteOrder,
     memcpy(pabyData + 1, &nGType, 4);
 
     // Copy the raw data
-    if (OGR_SWAP(eByteOrder))
+    if (OGR_SWAP(psOptions->eByteOrder))
     {
         int nCount = CPL_SWAP32(oMP.nGeomCount);
         memcpy(pabyData + 5, &nCount, 4);
@@ -317,7 +323,7 @@ OGRErr OGRPolyhedralSurface::exportToWkb(OGRwkbByteOrder eByteOrder,
     // serialize each of the geometries
     for (auto &&poSubGeom : *this)
     {
-        poSubGeom->exportToWkb(eByteOrder, pabyData + nOffset, wkbVariantIso);
+        poSubGeom->exportToWkb(pabyData + nOffset, psOptions);
         nOffset += poSubGeom->WkbSize();
     }
 
@@ -492,6 +498,7 @@ std::string OGRPolyhedralSurface::exportToWktInternal(const OGRWktOptions &opts,
         return std::string();
     }
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -532,6 +539,7 @@ OGRSurfaceCasterToPolygon OGRPolyhedralSurface::GetCasterToPolygon() const
 {
     return ::CasterToPolygon;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -552,6 +560,7 @@ OGRPolyhedralSurface::GetCasterToCurvePolygon() const
 {
     return ::CasterToCurvePolygon;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -564,6 +573,7 @@ OGRPolyhedralSurface::isCompatibleSubType(OGRwkbGeometryType eSubType) const
 {
     return wkbFlatten(eSubType) == wkbPolygon;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -575,6 +585,7 @@ const char *OGRPolyhedralSurface::getSubGeometryName() const
 {
     return "POLYGON";
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -586,6 +597,7 @@ OGRwkbGeometryType OGRPolyhedralSurface::getSubGeometryType() const
 {
     return wkbPolygon;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -656,6 +668,20 @@ double OGRPolyhedralSurface::get_Area() const
 #endif
 }
 
+/************************************************************************/
+/*                        get_GeodesicArea()                            */
+/************************************************************************/
+
+double OGRPolyhedralSurface::get_GeodesicArea(const OGRSpatialReference *) const
+{
+    if (IsEmpty())
+        return 0;
+
+    CPLError(CE_Failure, CPLE_NotSupported,
+             "get_GeodesicArea() not implemented for PolyhedralSurface");
+    return -1;
+}
+
 /************************************************************************/
 /*                           PointOnSurface()                           */
 /************************************************************************/
@@ -687,6 +713,7 @@ OGRPolyhedralSurface::CastToMultiPolygonImpl(OGRPolyhedralSurface *poPS)
     delete poPS;
     return poMultiPolygon;
 }
+
 //! @endcond
 
 /************************************************************************/
