@@ -156,23 +156,21 @@ void CADLayer::addHandle( long handle, CADObject::ObjectType type, long cadinser
 #endif //_DEBUG
     if( type == CADObject::ATTRIB || type == CADObject::ATTDEF )
     {
-        CADGeometry* pCADGeometry = pCADFile->GetGeometry( getId() - 1, handle );
-        std::unique_ptr<CADAttdef> attdef( dynamic_cast<CADAttdef*>(pCADGeometry) );
+        auto pCADGeometryPtr = pCADFile->GetGeometry( getId() - 1, handle );
+        std::unique_ptr<CADGeometry> pCADGeometry( pCADGeometryPtr );
+        CADAttdef* attdef = dynamic_cast<CADAttdef*>(pCADGeometry.get());
         if(attdef)
         {
             attributesNames.insert( attdef->getTag() );
         }
-        else
-        {
-            delete pCADGeometry;
-        }
     }
 
     if( type == CADObject::INSERT )
     {
         // TODO: transform insert to block of objects (do we need to transform
         // coordinates according to insert point)?
-        std::unique_ptr<CADObject> insert( pCADFile->GetObject( handle, false ) );
+        auto insertPtr = pCADFile->GetObject( handle, false );
+        std::unique_ptr<CADObject> insert( insertPtr );
         CADInsertObject * pInsert = dynamic_cast<CADInsertObject *>(insert.get());
         if( nullptr != pInsert )
         {
@@ -188,6 +186,10 @@ void CADLayer::addHandle( long handle, CADObject::ObjectType type, long cadinser
                     assert( 0 );
                 }
 #endif //_DEBUG
+                if( pBlockHeader->hEntities.empty() )
+                {
+                    return;
+                }
                 auto dCurrentEntHandle = pBlockHeader->hEntities[0].getAsLong();
                 auto dLastEntHandle    = pBlockHeader->hEntities.back().getAsLong(); // FIXME: in 2000+ entities probably has no links to each other.
 
@@ -203,33 +205,35 @@ void CADLayer::addHandle( long handle, CADObject::ObjectType type, long cadinser
                     CADEntityObject* pEntity =
                             dynamic_cast<CADEntityObject *>( entity.get() );
 
-                    if( dCurrentEntHandle == dLastEntHandle )
+                    if( nullptr == pEntity )
                     {
-                        if( nullptr != pEntity )
-                        {
-                            addHandle( dCurrentEntHandle, pEntity->getType(), handle );
-                            Matrix mat;
-                            mat.translate( pInsert->vertInsertionPoint );
-                            mat.scale( pInsert->vertScales );
-                            mat.rotate( pInsert->dfRotation );
-                            transformations[dCurrentEntHandle] = mat;
-                            break;
-                        }
-                        else
-                        {
-                            assert( 0 );
-                        }
+                        // shouldn't happen on a valid file, but can happen
+                        // on broken ones
+                        break;
                     }
 
-                    if( nullptr != pEntity )
+                    if( dCurrentEntHandle == handle && type == pEntity->getType() )
                     {
-                        addHandle( dCurrentEntHandle, pEntity->getType(), handle );
-                        Matrix mat;
-                        mat.translate( pInsert->vertInsertionPoint );
-                        mat.scale( pInsert->vertScales );
-                        mat.rotate( pInsert->dfRotation );
-                        transformations[dCurrentEntHandle] = mat;
+                        // If the above condition is true, infinite recursion
+                        // would occur in the following addHandle() call.
+                        // Shouldn't happen on a valid file, but can happen
+                        // on broken ones, such as in https://bugs.chromium.org/p/oss-fuzz/issues/detail?id=46887
+                        break;
+                    }
 
+                    addHandle( dCurrentEntHandle, pEntity->getType(), handle );
+                    Matrix mat;
+                    mat.translate( pInsert->vertInsertionPoint );
+                    mat.scale( pInsert->vertScales );
+                    mat.rotate( pInsert->dfRotation );
+                    transformations[dCurrentEntHandle] = mat;
+
+                    if( dCurrentEntHandle == dLastEntHandle )
+                    {
+                        break;
+                    }
+                    else
+                    {
                         if( pEntity->stCed.bNoLinks )
                         {
                             ++dCurrentEntHandle;
@@ -241,10 +245,6 @@ void CADLayer::addHandle( long handle, CADObject::ObjectType type, long cadinser
                                     pEntity->stCed.hObjectHandle );
                         }
                     }
-                    else
-                    {
-                        assert ( 0 );
-                    }
                 }
             }
         }
