@@ -101,7 +101,7 @@ OGRMultiPoint *GeometryReader::readMultiPoint()
     auto length = m_length / 2;
     if (length >= feature_max_buffer_size)
         return CPLErrorInvalidLength("MultiPoint");
-    auto mp = cpl::make_unique<OGRMultiPoint>();
+    auto mp = std::make_unique<OGRMultiPoint>();
     for (uint32_t i = 0; i < length; i++)
     {
         m_offset = i;
@@ -118,7 +118,7 @@ OGRMultiLineString *GeometryReader::readMultiLineString()
     const auto pEnds = m_geometry->ends();
     if (pEnds == nullptr)
         return CPLErrorInvalidPointer("MultiLineString ends data");
-    auto mls = cpl::make_unique<OGRMultiLineString>();
+    auto mls = std::make_unique<OGRMultiLineString>();
     m_offset = 0;
     for (uint32_t i = 0; i < pEnds->size(); i++)
     {
@@ -234,7 +234,7 @@ OGRErr GeometryReader::readSimpleCurve(OGRSimpleCurve *sc)
 OGRPolygon *GeometryReader::readPolygon()
 {
     const auto ends = m_geometry->ends();
-    auto p = cpl::make_unique<OGRPolygon>();
+    auto p = std::make_unique<OGRPolygon>();
     if (ends == nullptr || ends->size() < 2)
     {
         m_length = m_length / 2;
@@ -268,7 +268,7 @@ OGRMultiPolygon *GeometryReader::readMultiPolygon()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto mp = cpl::make_unique<OGRMultiPolygon>();
+    auto mp = std::make_unique<OGRMultiPolygon>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(
@@ -285,7 +285,7 @@ OGRGeometryCollection *GeometryReader::readGeometryCollection()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto gc = cpl::make_unique<OGRGeometryCollection>();
+    auto gc = std::make_unique<OGRGeometryCollection>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(readPart(parts->Get(i)));
@@ -301,7 +301,7 @@ OGRCompoundCurve *GeometryReader::readCompoundCurve()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto cc = cpl::make_unique<OGRCompoundCurve>();
+    auto cc = std::make_unique<OGRCompoundCurve>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(readPart(parts->Get(i)));
@@ -322,7 +322,7 @@ OGRCurvePolygon *GeometryReader::readCurvePolygon()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto cp = cpl::make_unique<OGRCurvePolygon>();
+    auto cp = std::make_unique<OGRCurvePolygon>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(readPart(parts->Get(i)));
@@ -343,7 +343,7 @@ OGRMultiCurve *GeometryReader::readMultiCurve()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto mc = cpl::make_unique<OGRMultiCurve>();
+    auto mc = std::make_unique<OGRMultiCurve>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(readPart(parts->Get(i)));
@@ -359,7 +359,7 @@ OGRMultiSurface *GeometryReader::readMultiSurface()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto ms = cpl::make_unique<OGRMultiSurface>();
+    auto ms = std::make_unique<OGRMultiSurface>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(readPart(parts->Get(i)));
@@ -380,7 +380,7 @@ OGRPolyhedralSurface *GeometryReader::readPolyhedralSurface()
     auto parts = m_geometry->parts();
     if (parts == nullptr)
         return CPLErrorInvalidPointer("parts data");
-    auto ps = cpl::make_unique<OGRPolyhedralSurface>();
+    auto ps = std::make_unique<OGRPolyhedralSurface>();
     for (uoffset_t i = 0; i < parts->size(); i++)
     {
         auto g = std::unique_ptr<OGRGeometry>(readPart(parts->Get(i)));
@@ -399,7 +399,7 @@ OGRPolyhedralSurface *GeometryReader::readPolyhedralSurface()
 OGRTriangulatedSurface *GeometryReader::readTIN()
 {
     const auto ends = m_geometry->ends();
-    auto ts = cpl::make_unique<OGRTriangulatedSurface>();
+    auto ts = std::make_unique<OGRTriangulatedSurface>();
     if (ends == nullptr || ends->size() < 2)
     {
         m_length = m_length / 2;
