@@ -161,8 +161,6 @@ uint32_t hilbert(const NodeItem &r, uint32_t hilbertMax, const double minX,
     return v;
 }
 
-const uint32_t hilbertMax = (1 << 16) - 1;
-
 void hilbertSort(std::vector<std::shared_ptr<Item>> &items)
 {
     NodeItem extent = calcExtent(items);
@@ -174,9 +172,9 @@ void hilbertSort(std::vector<std::shared_ptr<Item>> &items)
               [minX, minY, width, height](std::shared_ptr<Item> a,
                                           std::shared_ptr<Item> b)
               {
-                  uint32_t ha = hilbert(a->nodeItem, hilbertMax, minX, minY,
+                  uint32_t ha = hilbert(a->nodeItem, HILBERT_MAX, minX, minY,
                                         width, height);
-                  uint32_t hb = hilbert(b->nodeItem, hilbertMax, minX, minY,
+                  uint32_t hb = hilbert(b->nodeItem, HILBERT_MAX, minX, minY,
                                         width, height);
                   return ha > hb;
               });
@@ -193,24 +191,25 @@ void hilbertSort(std::vector<NodeItem> &items)
               [minX, minY, width, height](const NodeItem &a, const NodeItem &b)
               {
                   uint32_t ha =
-                      hilbert(a, hilbertMax, minX, minY, width, height);
+                      hilbert(a, HILBERT_MAX, minX, minY, width, height);
                   uint32_t hb =
-                      hilbert(b, hilbertMax, minX, minY, width, height);
+                      hilbert(b, HILBERT_MAX, minX, minY, width, height);
                   return ha > hb;
               });
 }
 
 NodeItem calcExtent(const std::vector<std::shared_ptr<Item>> &items)
 {
-    return std::accumulate(items.begin(), items.end(), NodeItem::create(0),
-                           [](NodeItem a, const std::shared_ptr<Item> &b)
-                           { return a.expand(b->nodeItem); });
+    return std::accumulate(
+        items.begin(), items.end(), NodeItem::create(0),
+        [](NodeItem a, const std::shared_ptr<Item> &b) -> NodeItem
+        { return a.expand(b->nodeItem); });
 }
 
 NodeItem calcExtent(const std::vector<NodeItem> &nodes)
 {
     return std::accumulate(nodes.begin(), nodes.end(), NodeItem::create(0),
-                           [](NodeItem a, const NodeItem &b)
+                           [](NodeItem a, const NodeItem &b) -> NodeItem
                            { return a.expand(b); });
 }
 
@@ -256,13 +255,10 @@ PackedRTree::generateLevelBounds(const uint64_t numItems,
     n = numNodes;
     for (auto size : levelNumNodes)
         levelOffsets.push_back(n -= size);
-    std::reverse(levelOffsets.begin(), levelOffsets.end());
-    std::reverse(levelNumNodes.begin(), levelNumNodes.end());
     std::vector<std::pair<uint64_t, uint64_t>> levelBounds;
     for (size_t i = 0; i < levelNumNodes.size(); i++)
         levelBounds.push_back(std::pair<uint64_t, uint64_t>(
             levelOffsets[i], levelOffsets[i] + levelNumNodes[i]));
-    std::reverse(levelBounds.begin(), levelBounds.end());
     return levelBounds;
 }
 
@@ -323,6 +319,16 @@ PackedRTree::PackedRTree(const void *data, const uint64_t numItems,
     fromData(data);
 }
 
+PackedRTree::PackedRTree(std::function<void(NodeItem *)> fillNodeItems,
+                         const uint64_t numItems, const NodeItem &extent,
+                         const uint16_t nodeSize)
+    : _extent(extent), _numItems(numItems)
+{
+    init(nodeSize);
+    fillNodeItems(_nodeItems + _numNodes - _numItems);
+    generateNodes();
+}
+
 std::vector<SearchResultItem>
 PackedRTree::search(double minX, double minY, double maxX, double maxY) const
 {
@@ -345,7 +351,7 @@ PackedRTree::search(double minX, double minY, double maxX, double maxY) const
         // search through child nodes
         for (uint64_t pos = nodeIndex; pos < end; pos++)
         {
-            auto nodeItem = _nodeItems[static_cast<size_t>(pos)];
+            const auto &nodeItem = _nodeItems[static_cast<size_t>(pos)];
             if (!n.intersects(nodeItem))
                 continue;
             if (isLeafNode)
@@ -398,7 +404,7 @@ std::vector<SearchResultItem> PackedRTree::streamSearch(
         for (uint64_t pos = nodeIndex; pos < end; pos++)
         {
             uint64_t nodePos = pos - nodeIndex;
-            auto nodeItem = nodeItems[static_cast<size_t>(nodePos)];
+            const auto &nodeItem = nodeItems[static_cast<size_t>(nodePos)];
             if (!item.intersects(nodeItem))
                 continue;
             if (isLeafNode)
