@@ -236,7 +236,8 @@ int OGRLayerDecorator::TestCapability(const char *pszCapability)
     return m_poDecoratedLayer->TestCapability(pszCapability);
 }
 
-OGRErr OGRLayerDecorator::CreateField(OGRFieldDefn *poField, int bApproxOK)
+OGRErr OGRLayerDecorator::CreateField(const OGRFieldDefn *poField,
+                                      int bApproxOK)
 {
     if (!m_poDecoratedLayer)
         return OGRERR_FAILURE;
@@ -275,7 +276,7 @@ OGRErr OGRLayerDecorator::AlterGeomFieldDefn(
                                                   poNewGeomFieldDefn, nFlagsIn);
 }
 
-OGRErr OGRLayerDecorator::CreateGeomField(OGRGeomFieldDefn *poField,
+OGRErr OGRLayerDecorator::CreateGeomField(const OGRGeomFieldDefn *poField,
                                           int bApproxOK)
 {
     if (!m_poDecoratedLayer)
@@ -346,7 +347,7 @@ const char *OGRLayerDecorator::GetGeometryColumn()
     return m_poDecoratedLayer->GetGeometryColumn();
 }
 
-OGRErr OGRLayerDecorator::SetIgnoredFields(const char **papszFields)
+OGRErr OGRLayerDecorator::SetIgnoredFields(CSLConstList papszFields)
 {
     if (!m_poDecoratedLayer)
         return OGRERR_FAILURE;
