@@ -39,6 +39,8 @@ OGRMutexedDataSource::OGRMutexedDataSource(OGRDataSource *poBaseDataSource,
       m_hGlobalMutex(hMutexIn),
       m_bWrapLayersInMutexedLayer(bWrapLayersInMutexedLayer)
 {
+    SetDescription(poBaseDataSource->GetDescription());
+    poDriver = poBaseDataSource->GetDriver();
 }
 
 OGRMutexedDataSource::~OGRMutexedDataSource()
@@ -127,13 +129,14 @@ int OGRMutexedDataSource::TestCapability(const char *pszCap)
     return m_poBaseDataSource->TestCapability(pszCap);
 }
 
-OGRLayer *OGRMutexedDataSource::ICreateLayer(
-    const char *pszName, const OGRSpatialReference *poSpatialRef,
-    OGRwkbGeometryType eGType, char **papszOptions)
+OGRLayer *
+OGRMutexedDataSource::ICreateLayer(const char *pszName,
+                                   const OGRGeomFieldDefn *poGeomFieldDefn,
+                                   CSLConstList papszOptions)
 {
     CPLMutexHolderOptionalLockD(m_hGlobalMutex);
     return WrapLayerIfNecessary(m_poBaseDataSource->CreateLayer(
-        pszName, poSpatialRef, eGType, papszOptions));
+        pszName, poGeomFieldDefn, papszOptions));
 }
 
 OGRLayer *OGRMutexedDataSource::CopyLayer(OGRLayer *poSrcLayer,
@@ -298,10 +301,11 @@ std::shared_ptr<GDALGroup> OGRMutexedDataSource::GetRootGroup() const
     return m_poBaseDataSource->GetRootGroup();
 }
 
-#if defined(WIN32) && defined(_MSC_VER)
+#if defined(_WIN32) && defined(_MSC_VER)
 // Horrible hack: for some reason MSVC doesn't export the class
 // if it is not referenced from the DLL itself
 void OGRRegisterMutexedDataSource();
+
 void OGRRegisterMutexedDataSource()
 {
     delete new OGRMutexedDataSource(NULL, FALSE, NULL, FALSE);
