@@ -38,6 +38,8 @@
 #include "ogr_expat.h"
 #endif
 
+constexpr int PARSER_BUF_SIZE = 8192;
+
 class OGRGeoRSSDataSource;
 
 typedef enum
@@ -135,7 +137,7 @@ class OGRGeoRSSLayer final : public OGRLayer
     OGRFeature *GetNextFeature() override;
 
     OGRErr ICreateFeature(OGRFeature *poFeature) override;
-    OGRErr CreateField(OGRFieldDefn *poField, int bApproxOK) override;
+    OGRErr CreateField(const OGRFieldDefn *poField, int bApproxOK) override;
 
     OGRFeatureDefn *GetLayerDefn() override;
 
@@ -143,6 +145,8 @@ class OGRGeoRSSLayer final : public OGRLayer
 
     GIntBig GetFeatureCount(int bForce) override;
 
+    GDALDataset *GetDataset() override;
+
     void LoadSchema();
 
 #ifdef HAVE_EXPAT
@@ -206,27 +210,29 @@ class OGRGeoRSSDataSource final : public OGRDataSource
     {
         return nLayers;
     }
+
     OGRLayer *GetLayer(int) override;
 
-    OGRLayer *ICreateLayer(const char *pszLayerName,
-                           const OGRSpatialReference *poSRS,
-                           OGRwkbGeometryType eType,
-                           char **papszOptions) override;
-
+    OGRLayer *ICreateLayer(const char *pszName,
+                           const OGRGeomFieldDefn *poGeomFieldDefn,
+                           CSLConstList papszOptions) override;
     int TestCapability(const char *) override;
 
     VSILFILE *GetOutputFP()
     {
         return fpOutput;
     }
+
     OGRGeoRSSFormat GetFormat()
     {
         return eFormat;
     }
+
     OGRGeoRSSGeomDialect GetGeomDialect()
     {
         return eGeomDialect;
     }
+
     bool GetUseExtensions()
     {
         return bUseExtensions;
