@@ -59,10 +59,12 @@ class OGRGPSBabelDataSource final : public OGRDataSource
     {
         return pszName;
     }
+
     virtual int GetLayerCount() override
     {
         return nLayers;
     }
+
     virtual OGRLayer *GetLayer(int) override;
 
     virtual int TestCapability(const char *) override;
@@ -96,15 +98,15 @@ class OGRGPSBabelWriteDataSource final : public OGRDataSource
     {
         return pszName;
     }
+
     virtual int GetLayerCount() override;
     virtual OGRLayer *GetLayer(int) override;
 
     virtual int TestCapability(const char *) override;
 
-    virtual OGRLayer *ICreateLayer(const char *pszLayerName,
-                                   const OGRSpatialReference *poSRS,
-                                   OGRwkbGeometryType eType,
-                                   char **papszOptions) override;
+    OGRLayer *ICreateLayer(const char *pszName,
+                           const OGRGeomFieldDefn *poGeomFieldDefn,
+                           CSLConstList papszOptions) override;
 
     int Create(const char *pszFilename, char **papszOptions);
 };
