@@ -91,8 +91,7 @@ OGRLIBKMLDataSource::OGRLIBKMLDataSource(KmlFactory *poKmlFactory)
       bUpdate(false), bUpdated(false), m_papszOptions(nullptr), m_isKml(false),
       m_poKmlDSKml(nullptr), m_poKmlDSContainer(nullptr),
       m_poKmlUpdate(nullptr), m_isKmz(false), m_poKmlDocKml(nullptr),
-      m_poKmlDocKmlRoot(nullptr), m_poKmlStyleKml(nullptr),
-      pszStylePath(const_cast<char *>("")), m_isDir(false),
+      m_poKmlDocKmlRoot(nullptr), m_poKmlStyleKml(nullptr), m_isDir(false),
       m_poKmlFactory(poKmlFactory)
 {
 }
@@ -207,37 +206,57 @@ static void OGRLIBKMLPreProcessInput(std::string &oKml)
 /*                       OGRLIBKMLRemoveSpaces()                        */
 /************************************************************************/
 
-static void OGRLIBKMLRemoveSpaces(std::string &oKml,
+static void OGRLIBKMLRemoveSpaces(std::string &osKml,
                                   const std::string &osNeedle)
 {
     size_t nPos = 0;
+    const std::string osLtNeedle(std::string("<").append(osNeedle));
+    std::string osTmp;
+    std::string osRet;
     while (true)
     {
-        nPos = oKml.find("<" + osNeedle, nPos);
-        if (nPos == std::string::npos)
+        auto nPosNew = osKml.find(osLtNeedle, nPos);
+        if (nPosNew == std::string::npos)
         {
+            osRet.append(osKml, nPos);
             break;
         }
-        const size_t nPosOri = nPos;
-        nPos = oKml.find(">", nPos);
-        if (nPos == std::string::npos || oKml[nPos + 1] != '\n')
+        const size_t nPosOri = nPosNew;
+        nPosNew = osKml.find(">\n", nPosNew);
+        if (nPosNew == std::string::npos || nPosNew + 2 == osKml.size())
         {
+            osRet.append(osKml, nPos);
             break;
         }
-        oKml = oKml.substr(0, nPos) + ">" + oKml.substr(nPos + strlen(">\n"));
-        CPLString osSpaces;
-        for (size_t nPosTmp = nPosOri - 1; oKml[nPosTmp] == ' '; nPosTmp--)
+        // Skip \n character
+        osRet.append(osKml, nPos, nPosNew - nPos + 1);
+        nPos = nPosNew + 2;
+
+        // Remove leading spaces of "    </{osNeedle}>"
+        osTmp.clear();
+        for (size_t nPosTmp = nPosOri - 1; osKml[nPosTmp] == ' '; nPosTmp--)
         {
-            osSpaces += ' ';
+            osTmp += ' ';
+        }
+        osTmp += "</";
+        osTmp += osNeedle;
+        osTmp += '>';
+        nPosNew = osKml.find(osTmp, nPos);
+        if (nPosNew != std::string::npos)
+        {
+            osRet.append(osKml, nPos, nPosNew - nPos);
+            osRet += "</";
+            osRet += osNeedle;
+            osRet += '>';
+            nPos = nPosNew + osTmp.size();
         }
-        nPos = oKml.find(osSpaces + "</" + osNeedle + ">", nPos);
-        if (nPos != std::string::npos)
-            oKml = oKml.substr(0, nPos) + "</" + osNeedle + ">" +
-                   oKml.substr(nPos + osSpaces.size() + strlen("</>") +
-                               osNeedle.size());
         else
+        {
+            osRet.append(osKml, nPos);
             break;
+        }
     }
+    osKml = std::move(osRet);
 }
 
 /************************************************************************/
@@ -425,7 +444,7 @@ bool OGRLIBKMLDataSource::WriteKmz()
                 }
             }
 
-            ParseDocumentOptions(kml, AsDocument(m_poKmlDocKml));
+            ParseDocumentOptions(std::move(kml), AsDocument(m_poKmlDocKml));
         }
 
         std::string oKmlOut = kmldom::SerializePretty(m_poKmlDocKmlRoot);
@@ -455,10 +474,10 @@ bool OGRLIBKMLDataSource::WriteKmz()
             if (!poKmlDocument->get_schema_array_size() && poKmlSchema &&
                 poKmlSchema->get_simplefield_array_size())
             {
-                poKmlDocument->add_schema(poKmlSchema);
+                poKmlDocument->add_schema(std::move(poKmlSchema));
             }
 
-            papoLayers[iLayer]->Finalize(poKmlDocument);
+            papoLayers[iLayer]->Finalize(std::move(poKmlDocument));
         }
 
         // If we do not have the layers root
@@ -550,7 +569,7 @@ bool OGRLIBKMLDataSource::WriteDir()
                     kml->set_feature(m_poKmlDocKml);
             }
 
-            ParseDocumentOptions(kml, AsDocument(m_poKmlDocKml));
+            ParseDocumentOptions(std::move(kml), AsDocument(m_poKmlDocKml));
         }
 
         std::string oKmlOut = kmldom::SerializePretty(m_poKmlDocKmlRoot);
@@ -585,10 +604,10 @@ bool OGRLIBKMLDataSource::WriteDir()
             if (!poKmlDocument->get_schema_array_size() && poKmlSchema &&
                 poKmlSchema->get_simplefield_array_size())
             {
-                poKmlDocument->add_schema(poKmlSchema);
+                poKmlDocument->add_schema(std::move(poKmlSchema));
             }
 
-            papoLayers[iLayer]->Finalize(poKmlDocument);
+            papoLayers[iLayer]->Finalize(std::move(poKmlDocument));
         }
 
         // If we do not have the layers root
@@ -701,9 +720,6 @@ OGRLIBKMLDataSource::~OGRLIBKMLDataSource()
 
     CPLFree(m_pszName);
 
-    if (!EQUAL(pszStylePath, ""))
-        CPLFree(pszStylePath);
-
     for (int i = 0; i < nLayers; i++)
         delete papoLayers[i];
 
@@ -729,7 +745,6 @@ SchemaPtr OGRLIBKMLDataSource::FindSchema(const char *pszSchemaUrl)
     char *pszID = nullptr;
     char *pszFile = nullptr;
     char *pszSchemaName = nullptr;
-    char *pszPound = nullptr;
     DocumentPtr poKmlDocument = nullptr;
     SchemaPtr poKmlSchemaResult = nullptr;
 
@@ -746,13 +761,11 @@ SchemaPtr OGRLIBKMLDataSource::FindSchema(const char *pszSchemaUrl)
                  m_poKmlDocKml->IsA(kmldom::Type_Document))
             poKmlDocument = AsDocument(m_poKmlDocKml);
     }
-    else if ((pszPound = strchr(const_cast<char *>(pszSchemaUrl), '#')) !=
-             nullptr)
+    else if (const char *pszPound = strchr(pszSchemaUrl, '#'))
     {
         pszFile = CPLStrdup(pszSchemaUrl);
         pszID = CPLStrdup(pszPound + 1);
-        pszPound = strchr(pszFile, '#');
-        *pszPound = '\0';
+        pszFile[pszPound - pszSchemaUrl] = '\0';
     }
     else
     {
@@ -781,7 +794,7 @@ SchemaPtr OGRLIBKMLDataSource::FindSchema(const char *pszSchemaUrl)
             {
                 if (EQUAL(pszID, poKmlSchema->get_id().c_str()))
                 {
-                    poKmlSchemaResult = poKmlSchema;
+                    poKmlSchemaResult = std::move(poKmlSchema);
                     break;
                 }
             }
@@ -790,7 +803,7 @@ SchemaPtr OGRLIBKMLDataSource::FindSchema(const char *pszSchemaUrl)
             {
                 if (EQUAL(pszSchemaName, poKmlSchema->get_name().c_str()))
                 {
-                    poKmlSchemaResult = poKmlSchema;
+                    poKmlSchemaResult = std::move(poKmlSchema);
                     break;
                 }
             }
@@ -854,7 +867,8 @@ OGRLIBKMLLayer *OGRLIBKMLDataSource::AddLayer(
     /***** add the layer to the array *****/
     const int iLayer = nLayers++;
     papoLayers[iLayer] = poOgrLayer;
-    m_oMapLayers[CPLString(osUniqueLayername).toupper()] = poOgrLayer;
+    osUniqueLayername.toupper();
+    m_oMapLayers[std::move(osUniqueLayername)] = poOgrLayer;
 
     return poOgrLayer;
 }
@@ -951,7 +965,7 @@ int OGRLIBKMLDataSource::ParseLayers(ContainerPtr poKmlContainer, bool bRecurse)
 
 ******************************************************************************/
 
-static ContainerPtr GetContainerFromRoot(KmlFactory *m_poKmlFactory,
+static ContainerPtr GetContainerFromRoot(KmlFactory *poKmlFactory,
                                          ElementPtr poKmlRoot)
 {
     ContainerPtr poKmlContainer = nullptr;
@@ -976,7 +990,7 @@ static ContainerPtr GetContainerFromRoot(KmlFactory *m_poKmlFactory,
                          (bReadGroundOverlay &&
                           poKmlFeat->IsA(kmldom::Type_GroundOverlay)))
                 {
-                    poKmlContainer = m_poKmlFactory->CreateDocument();
+                    poKmlContainer = poKmlFactory->CreateDocument();
                     poKmlContainer->add_feature(
                         kmldom::AsFeature(kmlengine::Clone(poKmlFeat)));
                 }
@@ -984,7 +998,7 @@ static ContainerPtr GetContainerFromRoot(KmlFactory *m_poKmlFactory,
         }
         else if (poKmlRoot->IsA(kmldom::Type_Container))
         {
-            poKmlContainer = AsContainer(poKmlRoot);
+            poKmlContainer = AsContainer(std::move(poKmlRoot));
         }
     }
 
@@ -1005,19 +1019,20 @@ int OGRLIBKMLDataSource::ParseIntoStyleTable(std::string *poKmlStyleKml,
     if (!poKmlRoot)
     {
         CPLError(CE_Failure, CPLE_OpenFailed, "ERROR parsing style kml %s :%s",
-                 pszStylePath, oKmlErrors.c_str());
+                 pszMyStylePath, oKmlErrors.c_str());
         return false;
     }
 
     ContainerPtr poKmlContainer = nullptr;
 
-    if (!(poKmlContainer = GetContainerFromRoot(m_poKmlFactory, poKmlRoot)))
+    if (!(poKmlContainer =
+              GetContainerFromRoot(m_poKmlFactory, std::move(poKmlRoot))))
     {
         return false;
     }
 
-    ParseStyles(AsDocument(poKmlContainer), &m_poStyleTable);
-    pszStylePath = CPLStrdup(pszMyStylePath);
+    ParseStyles(AsDocument(std::move(poKmlContainer)), &m_poStyleTable);
+    m_osStylePath = pszMyStylePath;
 
     return true;
 }
@@ -1224,15 +1239,15 @@ int OGRLIBKMLDataSource::OpenKmz(const char *pszFilename, int bUpdateIn)
         if (!poKmlLink->has_href())
             continue;
 
-        kmlengine::Href *poKmlHref = new kmlengine::Href(poKmlLink->get_href());
+        const kmlengine::Href oKmlHref(poKmlLink->get_href());
 
         /***** is the link relative? *****/
-        if (poKmlHref->IsRelativePath())
+        if (oKmlHref.IsRelativePath())
         {
             nLinks++;
 
             std::string oKml;
-            if (poKmlKmzfile->ReadFile(poKmlHref->get_path().c_str(), &oKml))
+            if (poKmlKmzfile->ReadFile(oKmlHref.get_path().c_str(), &oKml))
             {
                 /***** parse the kml into the DOM *****/
                 oKmlErrors.clear();
@@ -1242,9 +1257,8 @@ int OGRLIBKMLDataSource::OpenKmz(const char *pszFilename, int bUpdateIn)
                 {
                     CPLError(CE_Failure, CPLE_OpenFailed,
                              "ERROR parsing kml layer %s from %s :%s",
-                             poKmlHref->get_path().c_str(), pszFilename,
+                             oKmlHref.get_path().c_str(), pszFilename,
                              oKmlErrors.c_str());
-                    delete poKmlHref;
 
                     continue;
                 }
@@ -1257,34 +1271,30 @@ int OGRLIBKMLDataSource::OpenKmz(const char *pszFilename, int bUpdateIn)
                 {
                     CPLError(CE_Failure, CPLE_OpenFailed,
                              "ERROR parsing kml layer %s from %s :%s",
-                             poKmlHref->get_path().c_str(), pszFilename,
+                             oKmlHref.get_path().c_str(), pszFilename,
                              oKmlErrors.c_str());
-                    delete poKmlHref;
 
                     continue;
                 }
 
                 /***** create the layer *****/
-                AddLayer(CPLGetBasename(poKmlHref->get_path().c_str()),
-                         wkbUnknown, nullptr, this, poKmlLyrRoot,
-                         poKmlLyrContainer, poKmlHref->get_path().c_str(),
-                         FALSE, bUpdateIn, static_cast<int>(nKmlFeatures));
+                AddLayer(CPLGetBasename(oKmlHref.get_path().c_str()),
+                         wkbUnknown, nullptr, this, std::move(poKmlLyrRoot),
+                         poKmlLyrContainer, oKmlHref.get_path().c_str(), FALSE,
+                         bUpdateIn, static_cast<int>(nKmlFeatures));
 
                 /***** check if any features are another layer *****/
-                ParseLayers(poKmlLyrContainer, true);
+                ParseLayers(std::move(poKmlLyrContainer), true);
             }
         }
-
-        /***** cleanup *****/
-        delete poKmlHref;
     }
 
     /***** if the doc.kml has links store it so if were in update mode we can
      * write it *****/
     if (nLinks)
     {
-        m_poKmlDocKml = poKmlContainer;
-        m_poKmlDocKmlRoot = poKmlDocKmlRoot;
+        m_poKmlDocKml = std::move(poKmlContainer);
+        m_poKmlDocKmlRoot = std::move(poKmlDocKmlRoot);
     }
     /***** if the doc.kml has no links treat it as a normal kml file *****/
     else
@@ -1311,11 +1321,11 @@ int OGRLIBKMLDataSource::OpenKmz(const char *pszFilename, int bUpdateIn)
             }
 
             AddLayer(layername_default.c_str(), wkbUnknown, nullptr, this,
-                     poKmlDocKmlRoot, poKmlContainer, pszFilename, FALSE,
-                     bUpdateIn, 1);
+                     std::move(poKmlDocKmlRoot), poKmlContainer, pszFilename,
+                     FALSE, bUpdateIn, 1);
         }
 
-        ParseLayers(poKmlContainer, true);
+        ParseLayers(std::move(poKmlContainer), true);
     }
 
     /***** read the style table if it has one *****/
@@ -1417,17 +1427,17 @@ int OGRLIBKMLDataSource::OpenDir(const char *pszFilename, int bUpdateIn)
         if (EQUAL(papszDirList[iFile], "style.kml"))
         {
             ParseStyles(AsDocument(poKmlContainer), &m_poStyleTable);
-            pszStylePath = CPLStrdup(const_cast<char *>("style.kml"));
+            m_osStylePath = "style.kml";
             continue;
         }
 
         /***** create the layer *****/
         AddLayer(CPLGetBasename(osFilePath.c_str()), wkbUnknown, nullptr, this,
-                 poKmlRoot, poKmlContainer, osFilePath.c_str(), FALSE,
-                 bUpdateIn, nFiles);
+                 std::move(poKmlRoot), poKmlContainer, osFilePath.c_str(),
+                 FALSE, bUpdateIn, nFiles);
 
         /***** check if any features are another layer *****/
-        ParseLayers(poKmlContainer, true);
+        ParseLayers(std::move(poKmlContainer), true);
     }
 
     CSLDestroy(papszDirList);
@@ -1564,7 +1574,7 @@ static bool IsValidPhoneNumber(const char *pszPhoneNumber)
 /************************************************************************/
 
 void OGRLIBKMLDataSource::SetCommonOptions(ContainerPtr poKmlContainer,
-                                           char **papszOptions)
+                                           CSLConstList papszOptions)
 {
     const char *l_pszName = CSLFetchNameValue(papszOptions, "NAME");
     if (l_pszName != nullptr)
@@ -1630,7 +1640,7 @@ void OGRLIBKMLDataSource::ParseDocumentOptions(KmlPtr poKml,
                 else
                 {
                     CPLError(CE_Warning, CPLE_AppDefined,
-                             "Invalid IRI for AUTHOR_URI");
+                             "Invalid URI for AUTHOR_URI");
                 }
             }
             if (pszAuthorEmail != nullptr)
@@ -1764,7 +1774,7 @@ void OGRLIBKMLDataSource::ParseDocumentOptions(KmlPtr poKml,
                 LinkSnippetPtr linksnippet =
                     m_poKmlFactory->CreateLinkSnippet();
                 linksnippet->set_text(pszNLCLinkSnippet);
-                nlc->set_linksnippet(linksnippet);
+                nlc->set_linksnippet(std::move(linksnippet));
             }
             if (pszNLCExpires != nullptr)
             {
@@ -1832,7 +1842,7 @@ int OGRLIBKMLDataSource::CreateKmz(const char * /* pszFilename */,
         }
     }
 
-    pszStylePath = CPLStrdup(const_cast<char *>("style/style.kml"));
+    m_osStylePath = "style/style.kml";
 
     m_isKmz = true;
     bUpdated = true;
@@ -1874,7 +1884,7 @@ int OGRLIBKMLDataSource::CreateDir(const char *pszFilename,
         }
     }
 
-    pszStylePath = CPLStrdup(const_cast<char *>("style.kml"));
+    m_osStylePath = "style.kml";
 
     return TRUE;
 }
@@ -2028,23 +2038,18 @@ OGRErr OGRLIBKMLDataSource::DeleteLayerKmz(int iLayer)
                     /***** does the link have a href? *****/
                     if (poKmlLink->has_href())
                     {
-                        kmlengine::Href *poKmlHref =
-                            new kmlengine::Href(poKmlLink->get_href());
+                        const kmlengine::Href oKmlHref(poKmlLink->get_href());
 
                         /***** is the link relative? *****/
-                        if (poKmlHref->IsRelativePath())
+                        if (oKmlHref.IsRelativePath())
                         {
-                            const char *pszLink = poKmlHref->get_path().c_str();
-
-                            if (EQUAL(pszLink, poOgrLayer->GetFileName()))
+                            if (EQUAL(oKmlHref.get_path().c_str(),
+                                      poOgrLayer->GetFileName()))
                             {
                                 m_poKmlDocKml->DeleteFeatureAt(iKmlFeature);
-                                delete poKmlHref;
                                 break;
                             }
                         }
-
-                        delete poKmlHref;
                     }
                 }
             }
@@ -2122,7 +2127,7 @@ OGRErr OGRLIBKMLDataSource::DeleteLayer(int iLayer)
 
 OGRLIBKMLLayer *OGRLIBKMLDataSource::CreateLayerKml(
     const char *pszLayerName, const OGRSpatialReference *poSRS,
-    OGRwkbGeometryType eGType, char **papszOptions)
+    OGRwkbGeometryType eGType, CSLConstList papszOptions)
 {
     ContainerPtr poKmlLayerContainer = nullptr;
 
@@ -2168,7 +2173,7 @@ OGRLIBKMLLayer *OGRLIBKMLDataSource::CreateLayerKml(
 
 OGRLIBKMLLayer *OGRLIBKMLDataSource::CreateLayerKmz(
     const char *pszLayerName, const OGRSpatialReference *poSRS,
-    OGRwkbGeometryType eGType, char ** /* papszOptions */)
+    OGRwkbGeometryType eGType, CSLConstList /* papszOptions */)
 {
     DocumentPtr poKmlDocument = nullptr;
 
@@ -2228,10 +2233,10 @@ OGRLIBKMLLayer *OGRLIBKMLDataSource::CreateLayerKmz(
 
 ******************************************************************************/
 
-OGRLayer *OGRLIBKMLDataSource::ICreateLayer(const char *pszLayerName,
-                                            const OGRSpatialReference *poOgrSRS,
-                                            OGRwkbGeometryType eGType,
-                                            char **papszOptions)
+OGRLayer *
+OGRLIBKMLDataSource::ICreateLayer(const char *pszLayerName,
+                                  const OGRGeomFieldDefn *poGeomFieldDefn,
+                                  CSLConstList papszOptions)
 {
     if (!bUpdate)
         return nullptr;
@@ -2243,6 +2248,10 @@ OGRLayer *OGRLIBKMLDataSource::ICreateLayer(const char *pszLayerName,
         return nullptr;
     }
 
+    const auto eGType = poGeomFieldDefn ? poGeomFieldDefn->GetType() : wkbNone;
+    const auto poOgrSRS =
+        poGeomFieldDefn ? poGeomFieldDefn->GetSpatialRef() : nullptr;
+
     OGRLIBKMLLayer *poOgrLayer = nullptr;
 
     /***** kml DS *****/
