@@ -364,7 +364,7 @@ int MITABCoordSys2TABProjInfo(const char *pszCoordSys, TABProjInfo *psProj)
     // Fetch the units string.
     if (CSLCount(papszNextField) > 0)
     {
-        if (isdigit(papszNextField[0][0]))
+        if (isdigit(static_cast<unsigned char>(papszNextField[0][0])))
         {
             psProj->nUnitsId = static_cast<GByte>(atoi(papszNextField[0]));
         }
