@@ -33,6 +33,10 @@
 #include "cpl_string.h"
 #include "ogr_xerces.h"
 
+#define NASDebug(fmt, ...)                                                     \
+    CPLDebugOnly("NAS", "%s:%d %s " fmt, __FILE__, __LINE__, __FUNCTION__,     \
+                 __VA_ARGS__)
+
 /*
   Update modes:
 
@@ -123,9 +127,7 @@ NASHandler::NASHandler(NASReader *poReader)
     : m_poReader(poReader), m_pszCurField(nullptr), m_pszGeometry(nullptr),
       m_nGeomAlloc(0), m_nGeomLen(0), m_nGeometryDepth(0),
       m_nGeometryPropertyIndex(-1), m_nDepth(0), m_nDepthFeature(0),
-      m_bIgnoreFeature(false), m_bInUpdate(false), m_bInUpdateProperty(false),
-      m_nUpdateOrDeleteDepth(0), m_nUpdatePropertyDepth(0),
-      m_nNameOrValueDepth(0)
+      m_bIgnoreFeature(false), m_Locator(nullptr)
 {
 }
 
@@ -151,14 +153,24 @@ CPLString NASHandler::GetAttributes(const Attributes *attrs)
     for (unsigned int i = 0; i < attrs->getLength(); i++)
     {
         osRes += " ";
-        osRes += transcode(attrs->getQName(i), m_osAttrName);
+        osRes += transcode(attrs->getQName(i));
         osRes += "=\"";
-        osRes += transcode(attrs->getValue(i), m_osAttrValue);
+        osRes += transcode(attrs->getValue(i));
         osRes += "\"";
     }
     return osRes;
 }
 
+/************************************************************************/
+/*                   setDocumentLocator()                               */
+/************************************************************************/
+
+void NASHandler::setDocumentLocator(const Locator *locator)
+{
+    m_Locator = locator;
+    return DefaultHandler::setDocumentLocator(locator);
+}
+
 /************************************************************************/
 /*                            startElement()                            */
 /************************************************************************/
@@ -172,49 +184,193 @@ void NASHandler::startElement(const XMLCh *const /* uri */,
     GMLReadState *poState = m_poReader->GetState();
 
     transcode(localname, m_osElementName);
-#ifdef DEBUG_TRACE_ELEMENTS
-    for (int k = 0; k < m_nDepth; k++)
-        printf(" ");                          /*ok*/
-    printf(">%s\n", m_osElementName.c_str()); /*ok*/
-#endif
 
-    if (m_bIgnoreFeature && m_nDepth >= m_nDepthFeature)
-    {
-        m_nDepth++;
+    NASDebug("element=%s poState=%s m_nDepth=%d m_nDepthFeature=%d context=%s",
+             m_osElementName.c_str(), poState ? "(state)" : "(no state)",
+             m_nDepth, m_nDepthFeature, m_osDeleteContext.c_str());
+
+    m_nDepth++;
+    if (m_bIgnoreFeature && m_nDepth > m_nDepthFeature)
         return;
-    }
 
-#ifdef DEBUG_VERBOSE
-    CPLDebug("NAS",
-             "[%d] startElement %s m_bIgnoreFeature:%d m_nDepth:%d "
-             "m_nDepthFeature:%d featureClass:%s lastComponent:%s",
-             m_nDepth, m_osElementName.c_str(), m_bIgnoreFeature, m_nDepth,
-             m_nDepthFeature,
-             poState->m_poFeature
-                 ? poState->m_poFeature->GetClass()->GetElementName()
-                 : "(no feature)",
-             m_poReader->GetState()->GetLastComponent());
-#endif
-
-    /* -------------------------------------------------------------------- */
-    /*      If we are in the midst of collecting a feature attribute        */
-    /*      value, then this must be a complex attribute which we don't     */
-    /*      try to collect for now, so just terminate the field             */
-    /*      collection.                                                     */
-    /* -------------------------------------------------------------------- */
-    if (m_pszCurField != nullptr)
+    if (m_nDepthFeature == 0)
     {
-        CPLFree(m_pszCurField);
-        m_pszCurField = nullptr;
+        if (m_osElementName == "Replace")
+        {
+            const XMLCh achSafeToIgnore[] = {'s', 'a', 'f', 'e', 'T', 'o', 'I',
+                                             'g', 'n', 'o', 'r', 'e', 0};
+            int nIndex = attrs.getIndex(achSafeToIgnore);
+            if (nIndex != -1)
+                transcode(attrs.getValue(nIndex), m_osSafeToIgnore);
+            else
+                m_osSafeToIgnore = "true";
+            m_osReplacingFID = "";
+
+            CPLAssert(m_osDeleteContext == "");
+            m_osDeleteContext = m_osElementName;
+        }
+        else if (m_osElementName == "Update" || m_osElementName == "Delete")
+        {
+            const XMLCh achTypeNames[] = {'t', 'y', 'p', 'e', 'N',
+                                          'a', 'm', 'e', 's', 0};
+            const XMLCh achTypeName[] = {'t', 'y', 'p', 'e', 'N',
+                                         'a', 'm', 'e', 0};
+            int nIndex = attrs.getIndex(achTypeNames);
+            if (nIndex == -1)
+                nIndex = attrs.getIndex(achTypeName);
+
+            if (nIndex == -1)
+            {
+                CPLError(CE_Failure, CPLE_AssertionFailed,
+                         "NAS: expected type name missing at %s:%d:%d",
+                         m_poReader->GetSourceFileName(),
+                         static_cast<int>(m_Locator->getLineNumber()),
+                         static_cast<int>(m_Locator->getColumnNumber()));
+                return;
+            }
+
+            transcode(attrs.getValue(nIndex), m_osTypeName);
+
+            const char *pszTypeName = strchr(m_osTypeName.c_str(), ':');
+            pszTypeName = pszTypeName ? pszTypeName + 1 : m_osTypeName.c_str();
+            m_osTypeName = pszTypeName;
+
+            CPLAssert(m_osDeleteContext == "");
+            m_osDeleteContext = m_osElementName;
+        }
+        else if (m_osDeleteContext == "Update" &&
+                 (m_osElementName == "Name" ||
+                  m_osElementName == "ValueReference" ||
+                  m_osElementName == "Value"))
+        {
+            // fetch value
+            CPLFree(m_pszCurField);
+            m_pszCurField = CPLStrdup("");
+        }
+        else if (m_osDeleteContext != "" && (m_osElementName == "ResourceId" ||
+                                             m_osElementName == "FeatureId"))
+        {
+            const char *pszFilteredClassName =
+                m_poReader->GetFilteredClassName();
+            if (!pszFilteredClassName || EQUAL(pszFilteredClassName, "Delete"))
+            {
+                const XMLCh achRid[] = {'r', 'i', 'd', 0};
+                const XMLCh achFid[] = {'f', 'i', 'd', 0};
+                if (m_osTypeName == "")
+                {
+                    CPLError(CE_Failure, CPLE_AssertionFailed,
+                             "NAS: type name(s) missing at %s:%d:%d",
+                             m_poReader->GetSourceFileName(),
+                             static_cast<int>(m_Locator->getLineNumber()),
+                             static_cast<int>(m_Locator->getColumnNumber()));
+                    return;
+                }
+
+                int nIndex = attrs.getIndex(
+                    m_osElementName == "ResourceId" ? achRid : achFid);
+                if (nIndex == -1)
+                {
+                    CPLError(CE_Failure, CPLE_AssertionFailed,
+                             "NAS: expected feature id missing at %s,%d:%d",
+                             m_poReader->GetSourceFileName(),
+                             static_cast<int>(m_Locator->getLineNumber()),
+                             static_cast<int>(m_Locator->getColumnNumber()));
+                    return;
+                }
+
+                CPLString osFeatureId;
+                transcode(attrs.getValue(nIndex), osFeatureId);
+
+                m_poReader->PushFeature("Delete", attrs);
+                m_poReader->SetFeaturePropertyDirectly("typeName",
+                                                       CPLStrdup(m_osTypeName));
+                m_poReader->SetFeaturePropertyDirectly(
+                    "context", CPLStrdup(m_osDeleteContext));
+                m_poReader->SetFeaturePropertyDirectly("FeatureId",
+                                                       CPLStrdup(osFeatureId));
+
+                if (m_osDeleteContext == "Replace")
+                {
+                    if (m_osReplacingFID == "")
+                    {
+                        CPLError(
+                            CE_Failure, CPLE_AssertionFailed,
+                            "NAS: replacing feature id not set at %s:%d:%d",
+                            m_poReader->GetSourceFileName(),
+                            static_cast<int>(m_Locator->getLineNumber()),
+                            static_cast<int>(m_Locator->getColumnNumber()));
+                        return;
+                    }
+
+                    m_poReader->SetFeaturePropertyDirectly(
+                        "replacedBy", CPLStrdup(m_osReplacingFID));
+                    m_poReader->SetFeaturePropertyDirectly(
+                        "safeToIgnore", CPLStrdup(m_osSafeToIgnore));
+                    m_osReplacingFID = "";
+                    m_osSafeToIgnore = "";
+                }
+                else if (m_osDeleteContext == "Update")
+                {
+                    m_poReader->SetFeaturePropertyDirectly(
+                        "endet", CPLStrdup(m_osUpdateEnds));
+                    for (std::list<CPLString>::iterator it =
+                             m_UpdateOccasions.begin();
+                         it != m_UpdateOccasions.end(); ++it)
+                    {
+                        m_poReader->SetFeaturePropertyDirectly("anlass",
+                                                               CPLStrdup(*it));
+                    }
+
+                    m_osUpdateEnds = "";
+                    m_UpdateOccasions.clear();
+                }
+
+                return;
+            }
+            else
+            {
+                // we don't issue Delete features
+                m_osDeleteContext = "";
+            }
+        }
+        else if (m_poReader->IsFeatureElement(m_osElementName))
+        {
+            m_nDepthFeature = m_nDepth - 1;
+
+            // record id of replacing feature
+            if (m_osDeleteContext == "Replace")
+            {
+                const XMLCh achGmlId[] = {'g', 'm', 'l', ':', 'i', 'd', 0};
+                int nIndex = attrs.getIndex(achGmlId);
+                if (nIndex == -1)
+                {
+                    CPLError(CE_Failure, CPLE_AssertionFailed,
+                             "NAS: id of replacing feature not set at %s:%d:%d",
+                             m_poReader->GetSourceFileName(),
+                             static_cast<int>(m_Locator->getLineNumber()),
+                             static_cast<int>(m_Locator->getColumnNumber()));
+                    m_bIgnoreFeature = true;
+                    return;
+                }
+
+                CPLAssert(m_osReplacingFID == "");
+                transcode(attrs.getValue(nIndex), m_osReplacingFID);
+            }
+
+            m_osTypeName = m_osElementName;
+
+            const char *pszFilteredClassName =
+                m_poReader->GetFilteredClassName();
+            m_bIgnoreFeature = pszFilteredClassName &&
+                               !EQUAL(m_osElementName, pszFilteredClassName);
+
+            if (!m_bIgnoreFeature)
+                m_poReader->PushFeature(m_osElementName, attrs);
+
+            return;
+        }
     }
-
-    /* -------------------------------------------------------------------- */
-    /*      If we are collecting geometry, or if we determine this is a     */
-    /*      geometry element then append to the geometry info.              */
-    /* -------------------------------------------------------------------- */
-    const char *pszLast = nullptr;
-
-    if (m_pszGeometry != nullptr || IsGeometryElement(m_osElementName))
+    else if (m_pszGeometry != nullptr || IsGeometryElement(m_osElementName))
     {
         if (m_nGeometryPropertyIndex == -1 && poState->m_poFeature &&
             poState->m_poFeature->GetClass())
@@ -253,282 +409,23 @@ void NASHandler::startElement(const XMLCh *const /* uri */,
         strcat(m_pszGeometry + m_nGeomLen, ">");
         m_nGeomLen += static_cast<int>(strlen(m_pszGeometry + m_nGeomLen));
     }
-
-    /* -------------------------------------------------------------------- */
-    /*      Is this the ogc:Filter element in a update operation            */
-    /*      (wfs:Delete, wfsext:Replace or wfs:Update)?                     */
-    /*      specialized sort of feature.                                    */
-    /*      Issue a "Delete" feature for each ResourceId                    */
-    /* -------------------------------------------------------------------- */
-    else if (m_nDepthFeature == 0 &&
-             (m_osElementName == "Filter" || m_osElementName == "ResourceId") &&
-             (pszLast = m_poReader->GetState()->GetLastComponent()) !=
-                 nullptr &&
-             (EQUAL(pszLast, "Delete") || EQUAL(pszLast, "Replace") ||
-              EQUAL(pszLast, "Update")))
+    else if (m_poReader->IsAttributeElement(m_osElementName, attrs))
     {
-        const char *pszFilteredClassName = m_poReader->GetFilteredClassName();
-        if (pszFilteredClassName != nullptr &&
-            strcmp("Delete", pszFilteredClassName) != 0)
-        {
-            m_bIgnoreFeature = true;
-            m_nDepthFeature = m_nDepth;
-            m_nDepth++;
-
-            return;
-        }
-
-        if (m_osLastTypeName == "")
-        {
-            CPLError(CE_Failure, CPLE_AssertionFailed,
-                     "m_osLastTypeName == \"\"");
-
-            m_bIgnoreFeature = true;
-            m_nDepthFeature = m_nDepth;
-            m_nDepth++;
-            return;
-        }
-
-        if (EQUAL(pszLast, "Replace") &&
-            (m_osLastReplacingFID == "" || m_osLastSafeToIgnore == ""))
-        {
-            CPLError(CE_Failure, CPLE_AssertionFailed,
-                     "m_osLastReplacingFID == \"\" || "
-                     "m_osLastSafeToIgnore == \"\"");
-
-            m_bIgnoreFeature = true;
-            m_nDepthFeature = m_nDepth;
-            m_nDepth++;
-            return;
-        }
-
-        if (EQUAL(pszLast, "Update") && m_osLastEnded == "")
-        {
-            CPLError(CE_Failure, CPLE_AssertionFailed, "m_osLastEnded == \"\"");
-
-            m_bIgnoreFeature = true;
-            m_nDepthFeature = m_nDepth;
-            m_nDepth++;
-            return;
-        }
-
-        m_bIgnoreFeature = false;
-
-        m_poReader->PushFeature("Delete", attrs);
-
-        m_nDepthFeature = m_nDepth;
-        m_nDepth++;
-
-        m_poReader->SetFeaturePropertyDirectly("typeName",
-                                               CPLStrdup(m_osLastTypeName));
-        m_poReader->SetFeaturePropertyDirectly("context", CPLStrdup(pszLast));
-
-        if (EQUAL(pszLast, "Delete") && m_osElementName == "ResourceId")
-        {
-            char *rid = CPLStrdup("");
-
-            m_poReader->CheckForRID(attrs, &rid);
-
-            m_poReader->SetFeaturePropertyDirectly("FeatureId", rid);
-        }
-
-        if (EQUAL(pszLast, "Replace"))
-        {
-            // CPLAssert( m_osLastReplacingFID != "" );
-            // CPLAssert( m_osLastSafeToIgnore != "" );
-            m_poReader->SetFeaturePropertyDirectly(
-                "replacedBy", CPLStrdup(m_osLastReplacingFID));
-            m_poReader->SetFeaturePropertyDirectly(
-                "safeToIgnore", CPLStrdup(m_osLastSafeToIgnore));
-        }
-        else if (EQUAL(pszLast, "Update"))
-        {
-            m_poReader->SetFeaturePropertyDirectly("endet",
-                                                   CPLStrdup(m_osLastEnded));
-
-            for (std::list<CPLString>::iterator it = m_LastOccasions.begin();
-                 it != m_LastOccasions.end(); ++it)
-            {
-                m_poReader->SetFeaturePropertyDirectly("anlass",
-                                                       CPLStrdup(*it));
-            }
-
-            m_osLastEnded = "";
-            m_LastOccasions.clear();
-        }
-
-        return;
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      Is it a feature?  If so push a whole new state, and return.     */
-    /* -------------------------------------------------------------------- */
-    else if (!m_bInUpdateProperty && m_nDepthFeature == 0 &&
-             m_poReader->IsFeatureElement(m_osElementName))
-    {
-        m_osLastTypeName = m_osElementName;
-
-        const char *pszFilteredClassName = m_poReader->GetFilteredClassName();
-
-        pszLast = m_poReader->GetState()->GetLastComponent();
-        if (pszLast != nullptr && EQUAL(pszLast, "Replace"))
-        {
-            const XMLCh achFID[] = {'g', 'm', 'l', ':', 'i', 'd', '\0'};
-            int nIndex = attrs.getIndex(achFID);
-
-            if (nIndex == -1 || m_osLastReplacingFID != "")
-            {
-                CPLError(CE_Failure, CPLE_AssertionFailed,
-                         "nIndex == -1 || m_osLastReplacingFID !=\"\"");
-
-                m_bIgnoreFeature = true;
-                m_nDepthFeature = m_nDepth;
-                m_nDepth++;
-
-                return;
-            }
-
-            // Capture "gml:id" attribute as part of the property value -
-            // primarily this is for the wfsext:Replace operation's attribute.
-            transcode(attrs.getValue(nIndex), m_osLastReplacingFID);
-
-#ifdef DEBUG_VERBOSE
-            CPLDebug("NAS", "[%d] ### Replace typeName=%s replacedBy=%s",
-                     m_nDepth, m_osLastTypeName.c_str(),
-                     m_osLastReplacingFID.c_str());
-#endif
-        }
-
-        if (pszFilteredClassName != nullptr &&
-            m_osElementName != pszFilteredClassName)
-        {
-            m_bIgnoreFeature = true;
-            m_nDepthFeature = m_nDepth;
-            m_nDepth++;
-
-            return;
-        }
-
-        m_bIgnoreFeature = false;
-
-        m_poReader->PushFeature(m_osElementName, attrs);
-
-        m_nDepthFeature = m_nDepth;
-        m_nDepth++;
-
-        return;
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      If it is the wfs:Delete or wfs:Update element, then remember    */
-    /*      the typeName attribute so we can assign it to the feature that  */
-    /*      will be produced when we process the Filter element.            */
-    /* -------------------------------------------------------------------- */
-    else if (m_nUpdateOrDeleteDepth == 0 &&
-             (m_osElementName == "Delete" || m_osElementName == "Update"))
-    {
-        const XMLCh Name0[] = {'t', 'y', 'p', 'e', 'N', 'a', 'm', 'e', '\0'};
-
-        int nIndex0 = attrs.getIndex(Name0);
-        if (nIndex0 != -1)
-        {
-            transcode(attrs.getValue(nIndex0), m_osLastTypeName);
-        }
-        else
-        {
-            const XMLCh Name1[] = {'t', 'y', 'p', 'e', 'N',
-                                   'a', 'm', 'e', 's', '\0'};
-
-            int nIndex1 = attrs.getIndex(Name1);
-            if (nIndex1 != -1)
-            {
-                transcode(attrs.getValue(nIndex1), m_osLastTypeName);
-            }
-        }
-
-        m_osLastSafeToIgnore = "";
-        m_osLastReplacingFID = "";
-
-        if (m_osElementName == "Update")
-        {
-            m_bInUpdate = true;
-        }
-        m_nUpdateOrDeleteDepth = m_nDepth;
-    }
-
-    else if (m_nUpdatePropertyDepth == 0 && m_bInUpdate &&
-             m_osElementName == "Property")
-    {
-        m_bInUpdateProperty = true;
-        m_nUpdatePropertyDepth = m_nDepth;
-    }
-
-    else if (m_nNameOrValueDepth == 0 && m_bInUpdateProperty &&
-             (m_osElementName == "Name" || m_osElementName == "Value" ||
-              m_osElementName == "ValueReference"))
-    {
-        // collect attribute name or value
+        m_poReader->DealWithAttributes(
+            m_osElementName, static_cast<int>(m_osElementName.length()), attrs);
         CPLFree(m_pszCurField);
         m_pszCurField = CPLStrdup("");
-        m_nNameOrValueDepth = m_nDepth;
     }
 
-    /* -------------------------------------------------------------------- */
-    /*      If it is the wfsext:Replace element, then remember the          */
-    /*      safeToIgnore attribute so we can assign it to the feature       */
-    /*      that will be produced when we process the Filter element.       */
-    /* -------------------------------------------------------------------- */
-    else if (m_osElementName == "Replace")
-    {
-        const XMLCh Name[] = {'s', 'a', 'f', 'e', 'T', 'o', 'I',
-                              'g', 'n', 'o', 'r', 'e', '\0'};
-
-        int nIndex = attrs.getIndex(Name);
-
-        if (nIndex != -1)
-        {
-            transcode(attrs.getValue(nIndex), m_osLastSafeToIgnore);
-        }
-        else
-        {
-            CPLError(CE_Warning, CPLE_AppDefined,
-                     "NAS: safeToIgnore attribute missing");
-            m_osLastSafeToIgnore = "false";
-        }
-
-        m_osLastReplacingFID = "";
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      If it is (or at least potentially is) a simple attribute,       */
-    /*      then start collecting it.                                       */
-    /* -------------------------------------------------------------------- */
-    else if (m_poReader->IsAttributeElement(m_osElementName))
-    {
-        CPLFree(m_pszCurField);
-        m_pszCurField = CPLStrdup("");
-
-        // Capture href as OB property.
-        m_poReader->CheckForRelations(m_osElementName, attrs, &m_pszCurField);
-
-        // Capture "fid" attribute as part of the property value -
-        // primarily this is for wfs:Delete operation's FeatureId attribute.
-        if (m_osElementName == "FeatureId")
-            m_poReader->CheckForFID(attrs, &m_pszCurField);
-        else if (m_osElementName == "ResourceId")
-            m_poReader->CheckForRID(attrs, &m_pszCurField);
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      Push the element onto the current state's path.                 */
-    /* -------------------------------------------------------------------- */
     poState->PushPath(m_osElementName);
 
-    m_nDepth++;
-
     if (poState->osPath.size() > 512)
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "Too long path. Stop parsing");
+        CPLError(CE_Failure, CPLE_AssertionFailed,
+                 "NAS: Too long path. Stop parsing at %s:%d:%d",
+                 m_poReader->GetSourceFileName(),
+                 static_cast<int>(m_Locator->getLineNumber()),
+                 static_cast<int>(m_Locator->getColumnNumber()));
         m_poReader->StopParsing();
     }
 }
@@ -545,13 +442,11 @@ void NASHandler::endElement(const XMLCh *const /* uri */,
 
     transcode(localname, m_osElementName);
 
-    m_nDepth--;
-#ifdef DEBUG_TRACE_ELEMENTS
-    for (int k = 0; k < m_nDepth; k++)
-        printf(" ");                          /*ok*/
-    printf("<%s\n", m_osElementName.c_str()); /*ok*/
-#endif
+    NASDebug("element=%s poState=%s m_nDepth=%d m_nDepthFeature=%d context=%s",
+             m_osElementName.c_str(), poState ? "(state)" : "(no state)",
+             m_nDepth, m_nDepthFeature, m_osDeleteContext.c_str());
 
+    m_nDepth--;
     if (m_bIgnoreFeature && m_nDepth >= m_nDepthFeature)
     {
         if (m_nDepth == m_nDepthFeature)
@@ -562,100 +457,49 @@ void NASHandler::endElement(const XMLCh *const /* uri */,
         return;
     }
 
-#ifdef DEBUG_VERBOSE
-    CPLDebug("NAS",
-             "[%d] endElement %s m_bIgnoreFeature:%d m_nDepth:%d "
-             "m_nDepthFeature:%d featureClass:%s",
-             m_nDepth, m_osElementName.c_str(), m_bIgnoreFeature, m_nDepth,
-             m_nDepthFeature,
-             poState->m_poFeature
-                 ? poState->m_poFeature->GetClass()->GetElementName()
-                 : "(no feature)");
-#endif
-
-    if (m_bInUpdateProperty)
+    if (m_osDeleteContext == "Update")
     {
-        if (m_nDepth == m_nNameOrValueDepth &&
-            (m_osElementName == "Name" || m_osElementName == "ValueReference"))
+        if (m_osElementName == "Name" || m_osElementName == "ValueReference")
         {
-            m_osLastPropertyName = m_pszCurField ? m_pszCurField : "";
+            const char *pszName;
+            pszName = strrchr(m_pszCurField, '/');
+            pszName = pszName ? pszName + 1 : m_pszCurField;
+            pszName = strrchr(pszName, ':');
+            pszName = pszName ? pszName + 1 : m_pszCurField;
+
+            CPLAssert(m_osUpdatePropertyName == "");
+            m_osUpdatePropertyName = pszName;
             CPLFree(m_pszCurField);
             m_pszCurField = nullptr;
-            m_nNameOrValueDepth = 0;
+
+            if (m_osUpdatePropertyName != "endet" &&
+                m_osUpdatePropertyName != "anlass")
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "NAS: Unexpected property name %s at %s:%d:%d",
+                         m_osUpdatePropertyName.c_str(),
+                         m_poReader->GetSourceFileName(),
+                         static_cast<int>(m_Locator->getLineNumber()),
+                         static_cast<int>(m_Locator->getColumnNumber()));
+                m_osUpdatePropertyName = "";
+            }
         }
-        else if (m_osElementName == "Value" && m_nDepth == m_nNameOrValueDepth)
+        else if (m_osElementName == "Value")
         {
-            m_osLastPropertyValue = m_pszCurField ? m_pszCurField : "";
+            CPLAssert(m_osUpdatePropertyName != "");
+            if (m_osUpdatePropertyName == "endet")
+                m_osUpdateEnds = m_pszCurField;
+            else if (m_osUpdatePropertyName == "anlass")
+                m_UpdateOccasions.push_back(m_pszCurField);
+            m_osUpdatePropertyName = "";
             CPLFree(m_pszCurField);
             m_pszCurField = nullptr;
-            m_nNameOrValueDepth = 0;
         }
-        else if (m_nDepth == m_nUpdatePropertyDepth &&
-                 m_osElementName == "Property")
-        {
-            if (EQUAL(m_osLastPropertyName,
-                      "adv:lebenszeitintervall/adv:AA_Lebenszeitintervall/"
-                      "adv:endet") ||
-                EQUAL(m_osLastPropertyName,
-                      "lebenszeitintervall/AA_Lebenszeitintervall/endet"))
-            {
-                CPLAssert(m_osLastPropertyValue != "");
-                m_osLastEnded = m_osLastPropertyValue;
-            }
-            else if (EQUAL(m_osLastPropertyName, "adv:anlass") ||
-                     EQUAL(m_osLastPropertyName, "anlass"))
-            {
-                CPLAssert(m_osLastPropertyValue != "");
-                m_LastOccasions.push_back(m_osLastPropertyValue);
-            }
-            else
-            {
-                CPLError(CE_Warning, CPLE_AppDefined,
-                         "NAS: Expected property name or value instead of %s",
-                         m_osLastPropertyName.c_str());
-            }
-
-            m_osLastPropertyName = "";
-            m_osLastPropertyValue = "";
-            m_bInUpdateProperty = false;
-            m_nUpdatePropertyDepth = 0;
-        }
-
-        poState->PopPath();
-
-        return;
     }
-
-    if (m_nUpdateOrDeleteDepth > 0 &&
-        (m_osElementName == "Delete" || m_osElementName == "Update"))
+    else if (m_pszCurField != nullptr && poState->m_poFeature != nullptr)
     {
-        if (m_bInUpdate && m_osElementName == "Update")
-        {
-            m_bInUpdate = false;
-        }
-        m_nUpdateOrDeleteDepth = 0;
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      Is this closing off an attribute value?  We assume so if        */
-    /*      we are collecting an attribute value and got to this point.     */
-    /*      We don't bother validating that the closing tag matches the     */
-    /*      opening tag.                                                    */
-    /* -------------------------------------------------------------------- */
-    if (m_pszCurField != nullptr)
-    {
-        CPLAssert(poState->m_poFeature != nullptr);
-
-        // keep using "featureid" for GID 7
-        const char *pszPath;
-        if (EQUAL(poState->m_poFeature->GetClass()->GetElementName(),
-                  "Delete") &&
-            poState->osPath == "ResourceId")
-            pszPath = "FeatureId";
-        else
-            pszPath = poState->osPath.c_str();
-
-        m_poReader->SetFeaturePropertyDirectly(pszPath, m_pszCurField);
+        m_poReader->SetFeaturePropertyDirectly(poState->osPath.c_str(),
+                                               m_pszCurField);
         m_pszCurField = nullptr;
     }
 
@@ -720,23 +564,30 @@ void NASHandler::endElement(const XMLCh *const /* uri */,
                             poState->m_poFeature
                                 ->GetGeometryList()[m_nGeometryPropertyIndex]);
 
-                        CPLDebug("NAS",
-                                 "Overwriting other geometry (%s; replace:%s; "
-                                 "with:%s)",
-                                 poIdProp && poIdProp->nSubProperties > 0 &&
-                                         poIdProp->papszSubProperties[0]
-                                     ? poIdProp->papszSubProperties[0]
-                                     : "(null)",
-                                 m_pszGeometry, pszOldGeom);
+                        NASDebug(
+                            "Overwriting other geometry (%s; replace:%s; "
+                            "with:%s) at %s:%d:%d",
+                            poIdProp && poIdProp->nSubProperties > 0 &&
+                                    poIdProp->papszSubProperties[0]
+                                ? poIdProp->papszSubProperties[0]
+                                : "(null)",
+                            m_pszGeometry, pszOldGeom,
+                            m_poReader->GetSourceFileName(),
+                            static_cast<int>(m_Locator->getLineNumber()),
+                            static_cast<int>(m_Locator->getColumnNumber()));
 
                         CPLFree(pszOldGeom);
 #else
-                        CPLError(CE_Warning, CPLE_AppDefined,
-                                 "NAS: Overwriting other geometry (%s)",
-                                 poIdProp && poIdProp->nSubProperties > 0 &&
-                                         poIdProp->papszSubProperties[0]
-                                     ? poIdProp->papszSubProperties[0]
-                                     : "(null)");
+                        CPLError(
+                            CE_Warning, CPLE_AppDefined,
+                            "NAS: Overwriting other geometry (%s) at %s:%d:%d",
+                            poIdProp && poIdProp->nSubProperties > 0 &&
+                                    poIdProp->papszSubProperties[0]
+                                ? poIdProp->papszSubProperties[0]
+                                : "(null)",
+                            m_poReader->GetSourceFileName(),
+                            static_cast<int>(m_Locator->getLineNumber()),
+                            static_cast<int>(m_Locator->getColumnNumber()));
 #endif
                     }
 
@@ -759,21 +610,31 @@ void NASHandler::endElement(const XMLCh *const /* uri */,
 
                     else
                     {
-                        CPLError(CE_Warning, CPLE_AppDefined,
-                                 "NAS: Unexpected geometry skipped (class:%s "
-                                 "path:%s geom:%s)",
-                                 poState->m_poFeature->GetClass()->GetName(),
-                                 poState->osPath.c_str(), m_pszGeometry);
+                        CPLError(
+                            CE_Warning, CPLE_AssertionFailed,
+                            "NAS: Unexpected geometry skipped (class:%s "
+                            "path:%s geom:%s) at %s:%d:%d",
+                            poState->m_poFeature->GetClass()->GetName(),
+                            poState->osPath.c_str(), m_pszGeometry,
+                            m_poReader->GetSourceFileName(),
+                            static_cast<int>(m_Locator->getLineNumber()),
+                            static_cast<int>(m_Locator->getColumnNumber()));
                         CPLDestroyXMLNode(psNode);
                     }
                 }
                 else
                     CPLError(CE_Warning, CPLE_AppDefined,
-                             "NAS: Invalid geometry skipped");
+                             "NAS: Invalid geometry skipped at %s:%d:%d",
+                             m_poReader->GetSourceFileName(),
+                             static_cast<int>(m_Locator->getLineNumber()),
+                             static_cast<int>(m_Locator->getColumnNumber()));
             }
             else
                 CPLError(CE_Warning, CPLE_AppDefined,
-                         "NAS: Skipping geometry without feature");
+                         "NAS: Skipping geometry without feature at %s:%d:%d",
+                         m_poReader->GetSourceFileName(),
+                         static_cast<int>(m_Locator->getLineNumber()),
+                         static_cast<int>(m_Locator->getColumnNumber()));
 
             CPLFree(m_pszGeometry);
             m_pszGeometry = nullptr;
@@ -782,56 +643,22 @@ void NASHandler::endElement(const XMLCh *const /* uri */,
         }
     }
 
-    /* -------------------------------------------------------------------- */
-    /*      If we are collecting a feature, and this element tag matches    */
-    /*      element name for the class, then we have finished the           */
-    /*      feature, and we pop the feature read state.                     */
-    /* -------------------------------------------------------------------- */
-    const char *pszLast = nullptr;
-
-    if (m_nDepth == m_nDepthFeature && poState->m_poFeature != nullptr &&
-        m_osElementName == poState->m_poFeature->GetClass()->GetElementName())
+    // Finished actual feature or ResourceId/FeatureId of Delete/Replace/Update operation
+    if ((m_nDepth == m_nDepthFeature && poState->m_poFeature != nullptr &&
+         EQUAL(m_osElementName,
+               poState->m_poFeature->GetClass()->GetElementName())) ||
+        (m_osDeleteContext != "" &&
+         (m_osElementName == "ResourceId" || m_osElementName == "FeatureId")))
     {
         m_nDepthFeature = 0;
         m_poReader->PopState();
     }
-
-    /* -------------------------------------------------------------------- */
-    /*      Ends of a wfs:Delete or wfs:Update should be triggered on the   */
-    /*      close of the <Filter> element.                                  */
-    /* -------------------------------------------------------------------- */
-    else if (m_nDepth == m_nDepthFeature && poState->m_poFeature != nullptr &&
-             m_osElementName == "Filter" &&
-             (pszLast = poState->m_poFeature->GetClass()->GetElementName()) !=
-                 nullptr &&
-             (EQUAL(pszLast, "Delete") || EQUAL(pszLast, "Update")))
-    {
-        m_nDepthFeature = 0;
-        m_poReader->PopState();
-    }
-
-    else if (m_nDepth >= m_nDepthFeature && poState->m_poFeature != nullptr &&
-             m_osElementName == "ResourceId" &&
-             (pszLast = poState->m_poFeature->GetClass()->GetElementName()) !=
-                 nullptr &&
-             EQUAL(pszLast, "Delete"))
-    {
-        m_nDepthFeature = 0;
-        m_poReader->PopState();
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      Otherwise, we just pop the element off the local read states    */
-    /*      element stack.                                                  */
-    /* -------------------------------------------------------------------- */
     else
+        poState->PopPath();
+
+    if (m_osDeleteContext == m_osElementName)
     {
-        if (m_osElementName == poState->GetLastComponent())
-            poState->PopPath();
-        else
-        {
-            CPLAssert(false);
-        }
+        m_osDeleteContext = "";
     }
 }
 
@@ -862,7 +689,8 @@ void NASHandler::characters(const XMLCh *const chars, const XMLSize_t length)
         memcpy(m_pszCurField + nCurFieldLength, m_osCharacters.c_str(),
                m_osCharacters.size() + 1);
     }
-    else if (m_pszGeometry != nullptr)
+
+    if (m_pszGeometry != nullptr)
     {
         int nSkipped = 0;
         if (m_nGeomLen == 0)
@@ -900,8 +728,8 @@ void NASHandler::fatalError(const SAXParseException &exception)
     CPLString osErrMsg;
     transcode(exception.getMessage(), osErrMsg);
     CPLError(CE_Failure, CPLE_AppDefined,
-             "XML Parsing Error: %s at line %d, column %d\n", osErrMsg.c_str(),
-             static_cast<int>(exception.getLineNumber()),
+             "NAS: XML Parsing Error: %s at line %d, column %d\n",
+             osErrMsg.c_str(), static_cast<int>(exception.getLineNumber()),
              static_cast<int>(exception.getColumnNumber()));
 }
 
@@ -927,4 +755,4 @@ bool NASHandler::IsGeometryElement(const char *pszElement)
            strcmp(pszElement, "LineString") == 0;
 }
 
-// vim: set sw=4 expandtab :
+// vim: set sw=4 expandtab ai :
