@@ -72,6 +72,7 @@ OGRODSLayer::OGRODSLayer(OGRODSDataSource *poDSIn, const char *pszName,
       bUpdated(CPL_TO_BOOL(bUpdatedIn)), bHasHeaderLine(false),
       m_poAttrQueryODS(nullptr)
 {
+    SetAdvertizeUTF8(true);
 }
 
 /************************************************************************/
@@ -110,6 +111,28 @@ OGRErr OGRODSLayer::SyncToDisk()
     return OGRERR_NONE;
 }
 
+/************************************************************************/
+/*                      TranslateFIDFromMemLayer()                      */
+/************************************************************************/
+
+// Translate a FID from MEM convention (0-based) to ODS convention
+GIntBig OGRODSLayer::TranslateFIDFromMemLayer(GIntBig nFID) const
+{
+    return nFID + (1 + (bHasHeaderLine ? 1 : 0));
+}
+
+/************************************************************************/
+/*                        TranslateFIDToMemLayer()                      */
+/************************************************************************/
+
+// Translate a FID from ODS convention to MEM convention (0-based)
+GIntBig OGRODSLayer::TranslateFIDToMemLayer(GIntBig nFID) const
+{
+    if (nFID > 0)
+        return nFID - (1 + (bHasHeaderLine ? 1 : 0));
+    return OGRNullFID;
+}
+
 /************************************************************************/
 /*                          GetNextFeature()                            */
 /************************************************************************/
@@ -121,7 +144,7 @@ OGRFeature *OGRODSLayer::GetNextFeature()
         OGRFeature *poFeature = OGRMemLayer::GetNextFeature();
         if (poFeature == nullptr)
             return nullptr;
-        poFeature->SetFID(poFeature->GetFID() + 1 + (bHasHeaderLine ? 1 : 0));
+        poFeature->SetFID(TranslateFIDFromMemLayer(poFeature->GetFID()));
         if (m_poAttrQueryODS == nullptr ||
             m_poAttrQueryODS->Evaluate(poFeature))
         {
@@ -138,7 +161,7 @@ OGRFeature *OGRODSLayer::GetNextFeature()
 OGRFeature *OGRODSLayer::GetFeature(GIntBig nFeatureId)
 {
     OGRFeature *poFeature =
-        OGRMemLayer::GetFeature(nFeatureId - (1 + (bHasHeaderLine ? 1 : 0)));
+        OGRMemLayer::GetFeature(TranslateFIDToMemLayer(nFeatureId));
     if (poFeature)
         poFeature->SetFID(nFeatureId);
     return poFeature;
@@ -156,20 +179,74 @@ GIntBig OGRODSLayer::GetFeatureCount(int bForce)
 }
 
 /************************************************************************/
-/*                           ISetFeature()                               */
+/*                           ISetFeature()                              */
 /************************************************************************/
 
 OGRErr OGRODSLayer::ISetFeature(OGRFeature *poFeature)
 {
-    if (poFeature == nullptr)
-        return OGRMemLayer::ISetFeature(poFeature);
-
-    GIntBig nFID = poFeature->GetFID();
-    if (nFID != OGRNullFID)
-        poFeature->SetFID(nFID - (1 + (bHasHeaderLine ? 1 : 0)));
+    const GIntBig nFIDOrigin = poFeature->GetFID();
+    if (nFIDOrigin > 0)
+    {
+        const GIntBig nFIDMemLayer = TranslateFIDToMemLayer(nFIDOrigin);
+        if (!GetFeatureRef(nFIDMemLayer))
+            return OGRERR_NON_EXISTING_FEATURE;
+        poFeature->SetFID(nFIDMemLayer);
+    }
+    else
+    {
+        return OGRERR_NON_EXISTING_FEATURE;
+    }
     SetUpdated();
     OGRErr eErr = OGRMemLayer::ISetFeature(poFeature);
-    poFeature->SetFID(nFID);
+    poFeature->SetFID(nFIDOrigin);
+    return eErr;
+}
+
+/************************************************************************/
+/*                         IUpdateFeature()                             */
+/************************************************************************/
+
+OGRErr OGRODSLayer::IUpdateFeature(OGRFeature *poFeature,
+                                   int nUpdatedFieldsCount,
+                                   const int *panUpdatedFieldsIdx,
+                                   int nUpdatedGeomFieldsCount,
+                                   const int *panUpdatedGeomFieldsIdx,
+                                   bool bUpdateStyleString)
+{
+    const GIntBig nFIDOrigin = poFeature->GetFID();
+    if (nFIDOrigin != OGRNullFID)
+        poFeature->SetFID(TranslateFIDToMemLayer(nFIDOrigin));
+    SetUpdated();
+    OGRErr eErr = OGRMemLayer::IUpdateFeature(
+        poFeature, nUpdatedFieldsCount, panUpdatedFieldsIdx,
+        nUpdatedGeomFieldsCount, panUpdatedGeomFieldsIdx, bUpdateStyleString);
+    poFeature->SetFID(nFIDOrigin);
+    return eErr;
+}
+
+/************************************************************************/
+/*                          ICreateFeature()                            */
+/************************************************************************/
+
+OGRErr OGRODSLayer::ICreateFeature(OGRFeature *poFeature)
+{
+    const GIntBig nFIDOrigin = poFeature->GetFID();
+    if (nFIDOrigin > 0)
+    {
+        const GIntBig nFIDModified = TranslateFIDToMemLayer(nFIDOrigin);
+        if (GetFeatureRef(nFIDModified))
+        {
+            SetUpdated();
+            poFeature->SetFID(nFIDModified);
+            OGRErr eErr = OGRMemLayer::ISetFeature(poFeature);
+            poFeature->SetFID(nFIDOrigin);
+            return eErr;
+        }
+    }
+    SetUpdated();
+    poFeature->SetFID(OGRNullFID);
+    OGRErr eErr = OGRMemLayer::ICreateFeature(poFeature);
+    poFeature->SetFID(TranslateFIDFromMemLayer(poFeature->GetFID()));
     return eErr;
 }
 
@@ -180,7 +257,7 @@ OGRErr OGRODSLayer::ISetFeature(OGRFeature *poFeature)
 OGRErr OGRODSLayer::DeleteFeature(GIntBig nFID)
 {
     SetUpdated();
-    return OGRMemLayer::DeleteFeature(nFID - (1 + (bHasHeaderLine ? 1 : 0)));
+    return OGRMemLayer::DeleteFeature(TranslateFIDToMemLayer(nFID));
 }
 
 /************************************************************************/
@@ -210,6 +287,15 @@ int OGRODSLayer::TestCapability(const char *pszCap)
     return OGRMemLayer::TestCapability(pszCap);
 }
 
+/************************************************************************/
+/*                             GetDataset()                             */
+/************************************************************************/
+
+GDALDataset *OGRODSLayer::GetDataset()
+{
+    return poDS;
+}
+
 /************************************************************************/
 /*                          OGRODSDataSource()                          */
 /************************************************************************/
@@ -846,7 +932,6 @@ void OGRODSDataSource::endElementTable(
 
             reinterpret_cast<OGRMemLayer *>(poCurLayer)
                 ->SetUpdatable(bUpdatable);
-            reinterpret_cast<OGRMemLayer *>(poCurLayer)->SetAdvertizeUTF8(true);
             reinterpret_cast<OGRODSLayer *>(poCurLayer)->SetUpdated(false);
         }
 
@@ -1214,7 +1299,7 @@ void OGRODSDataSource::endElementRow(
                                  eValType == OFTInteger &&
                                  eValSubType != OFSTBoolean)
                         {
-                            poFieldDefn->SetSubType(OFSTNone);
+                            whileUnsealing(poFieldDefn)->SetSubType(OFSTNone);
                         }
                     }
                 }
@@ -1524,9 +1609,10 @@ void OGRODSDataSource::AnalyseSettings()
 /*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRODSDataSource::ICreateLayer(
-    const char *pszLayerName, const OGRSpatialReference * /* poSRS */,
-    OGRwkbGeometryType /* eType */, char **papszOptions)
+OGRLayer *
+OGRODSDataSource::ICreateLayer(const char *pszLayerName,
+                               const OGRGeomFieldDefn * /*poGeomFieldDefn*/,
+                               CSLConstList papszOptions)
 {
     /* -------------------------------------------------------------------- */
     /*      Verify we are in update mode.                                   */
@@ -2329,8 +2415,7 @@ int ODSCellEvaluator::EvaluateRange(int nRow1, int nCol1, int nRow2, int nCol2,
 
 int ODSCellEvaluator::Evaluate(int nRow, int nCol)
 {
-    if (oVisisitedCells.find(std::pair<int, int>(nRow, nCol)) !=
-        oVisisitedCells.end())
+    if (oVisisitedCells.find(std::pair(nRow, nCol)) != oVisisitedCells.end())
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Circular dependency with (row=%d, col=%d)", nRow + 1,
@@ -2338,7 +2423,7 @@ int ODSCellEvaluator::Evaluate(int nRow, int nCol)
         return FALSE;
     }
 
-    oVisisitedCells.insert(std::pair<int, int>(nRow, nCol));
+    oVisisitedCells.insert(std::pair(nRow, nCol));
 
     if (poLayer->SetNextByIndex(nRow) != OGRERR_NONE)
     {
