@@ -78,7 +78,11 @@ typedef enum
     FGFT_RASTER = 9,
     FGFT_GUID = 10,
     FGFT_GLOBALID = 11,
-    FGFT_XML = 12
+    FGFT_XML = 12,
+    FGFT_INT64 = 13,                 // added in ArcGIS Pro 3.2
+    FGFT_DATE = 14,                  // added in ArcGIS Pro 3.2
+    FGFT_TIME = 15,                  // added in ArcGIS Pro 3.2
+    FGFT_DATETIME_WITH_OFFSET = 16,  // added in ArcGIS Pro 3.2
 } FileGDBFieldType;
 
 /************************************************************************/
@@ -98,7 +102,13 @@ class FileGDBField
     std::string m_osAlias{};
     FileGDBFieldType m_eType = FGFT_UNDEFINED;
 
-    bool m_bNullable = false;
+    bool m_bNullable = false;  // Bit 1 of flag field
+    bool m_bRequired =
+        false;  // Bit 2 of flag field. Set for ObjectID, geometry field and Shape_Area/Shape_Length
+    bool m_bEditable = false;       // Bit 3 of flag field.
+    bool m_bHighPrecision = false;  // for FGFT_DATETIME
+    bool m_bReadAsDouble =
+        false;           // used by FileGDBTable::CreateAttributeIndex()
     int m_nMaxWidth = 0; /* for string */
 
     OGRField m_sDefault{};
@@ -110,11 +120,17 @@ class FileGDBField
 
   public:
     static const OGRField UNSET_FIELD;
+    static constexpr int BIT_NULLABLE = 0;
+    static constexpr int BIT_REQUIRED = 1;
+    static constexpr int BIT_EDITABLE = 2;
+    static constexpr int MASK_NULLABLE = 1 << BIT_NULLABLE;
+    static constexpr int MASK_REQUIRED = 1 << BIT_REQUIRED;
+    static constexpr int MASK_EDITABLE = 1 << BIT_EDITABLE;
 
     explicit FileGDBField(FileGDBTable *m_poParent);
     FileGDBField(const std::string &osName, const std::string &osAlias,
-                 FileGDBFieldType eType, bool bNullable, int nMaxWidth,
-                 const OGRField &sDefault);
+                 FileGDBFieldType eType, bool bNullable, bool bRequired,
+                 bool bEditable, int nMaxWidth, const OGRField &sDefault);
     virtual ~FileGDBField();
 
     void SetParent(FileGDBTable *poParent)
@@ -126,27 +142,52 @@ class FileGDBField
     {
         return m_osName;
     }
+
     const std::string &GetAlias() const
     {
         return m_osAlias;
     }
+
     FileGDBFieldType GetType() const
     {
         return m_eType;
     }
+
     bool IsNullable() const
     {
         return m_bNullable;
     }
+
+    bool IsRequired() const
+    {
+        return m_bRequired;
+    }
+
+    bool IsEditable() const
+    {
+        return m_bEditable;
+    }
+
     int GetMaxWidth() const
     {
         return m_nMaxWidth;
     }
+
     const OGRField *GetDefault() const
     {
         return &m_sDefault;
     }
 
+    void SetHighPrecision()
+    {
+        m_bHighPrecision = true;
+    }
+
+    bool IsHighPrecision() const
+    {
+        return m_bHighPrecision;
+    }
+
     int HasIndex();
     FileGDBIndex *GetIndex();
 };
@@ -190,6 +231,7 @@ class FileGDBGeomField : public FileGDBField
                      bool bNullable, const std::string &osWKT, double dfXOrigin,
                      double dfYOrigin, double dfXYScale, double dfXYTolerance,
                      const std::vector<double> &adfSpatialIndexGridResolution);
+
     virtual ~FileGDBGeomField()
     {
     }
@@ -203,30 +245,37 @@ class FileGDBGeomField : public FileGDBField
     {
         return m_dfXMin;
     }
+
     double GetYMin() const
     {
         return m_dfYMin;
     }
+
     double GetZMin() const
     {
         return m_dfZMin;
     }  // only valid for m_bGeomTypeHasZ
+
     double GetMMin() const
     {
         return m_dfMMin;
     }  // only valid for m_bGeomTypeHasM
+
     double GetXMax() const
     {
         return m_dfXMax;
     }
+
     double GetYMax() const
     {
         return m_dfYMax;
     }
+
     double GetZMax() const
     {
         return m_dfZMax;
     }  // only valid for m_bGeomTypeHasZ
+
     double GetMMax() const
     {
         return m_dfMMax;
@@ -241,6 +290,7 @@ class FileGDBGeomField : public FileGDBField
     {
         return m_bHasZOriginScaleTolerance;
     }
+
     int HasMOriginScaleTolerance() const
     {
         return m_bHasMOriginScaleTolerance;
@@ -250,14 +300,17 @@ class FileGDBGeomField : public FileGDBField
     {
         return m_dfXOrigin;
     }
+
     double GetYOrigin() const
     {
         return m_dfYOrigin;
     }
+
     double GetXYScale() const
     {
         return m_dfXYScale;
     }
+
     double GetXYTolerance() const
     {
         return m_dfXYTolerance;
@@ -267,14 +320,17 @@ class FileGDBGeomField : public FileGDBField
     {
         return m_dfZOrigin;
     }
+
     double GetZScale() const
     {
         return m_dfZScale;
     }
+
     double GetZTolerance() const
     {
         return m_dfZTolerance;
     }
+
     void SetZOriginScaleTolerance(double dfZOrigin, double dfZScale,
                                   double dfZTolerance);
 
@@ -282,14 +338,17 @@ class FileGDBGeomField : public FileGDBField
     {
         return m_dfMOrigin;
     }
+
     double GetMScale() const
     {
         return m_dfMScale;
     }
+
     double GetMTolerance() const
     {
         return m_dfMTolerance;
     }
+
     void SetMOriginScaleTolerance(double dfMOrigin, double dfMScale,
                                   double dfMTolerance);
 
@@ -328,6 +387,7 @@ class FileGDBRasterField : public FileGDBGeomField
         : FileGDBGeomField(poParentIn)
     {
     }
+
     virtual ~FileGDBRasterField()
     {
     }
@@ -336,6 +396,7 @@ class FileGDBRasterField : public FileGDBGeomField
     {
         return m_osRasterColumnName;
     }
+
     Type GetRasterType() const
     {
         return m_eRasterType;
@@ -356,6 +417,7 @@ class FileGDBIndex
     FileGDBIndex()
     {
     }
+
     virtual ~FileGDBIndex()
     {
     }
@@ -364,10 +426,12 @@ class FileGDBIndex
     {
         return m_osIndexName;
     }
+
     const std::string &GetExpression() const
     {
         return m_osExpression;
     }
+
     std::string GetFieldName() const;
     int GetMaxWidthInBytes(const FileGDBTable *poTable) const;
 
@@ -386,6 +450,11 @@ class FileGDBTable
     vsi_l_offset m_nFileSize = 0; /* only read when needed */
     bool m_bUpdate = false;
 
+    //! This flag is set when we detect that a corruption of m_nHeaderBufferMaxSize
+    // prior to fix needs to  fdf39012788b1110b3bf0ae6b8422a528f0ae8b6 to be
+    // repaired
+    bool m_bHasWarnedAboutHeaderRepair = false;
+
     std::string m_osFilename{};
     bool m_bIsV9 = false;
     std::vector<std::unique_ptr<FileGDBField>> m_apoFields{};
@@ -500,6 +569,7 @@ class FileGDBTable
         explicit WholeFileRewriter(FileGDBTable &oTable) : m_oTable(oTable)
         {
         }
+
         ~WholeFileRewriter();
 
         bool Begin();
@@ -562,44 +632,54 @@ class FileGDBTable
     {
         return m_osFilename;
     }
+
     FileGDBTableGeometryType GetGeometryType() const
     {
         return m_eTableGeomType;
     }
+
     bool GetGeomTypeHasZ() const
     {
         return m_bGeomTypeHasZ;
     }
+
     bool GetGeomTypeHasM() const
     {
         return m_bGeomTypeHasM;
     }
+
     int GetValidRecordCount() const
     {
         return m_nValidRecordCount;
     }
+
     int GetTotalRecordCount() const
     {
         return m_nTotalRecordCount;
     }
+
     int GetFieldCount() const
     {
         return static_cast<int>(m_apoFields.size());
     }
+
     FileGDBField *GetField(int i) const
     {
         return m_apoFields[i].get();
     }
+
     int GetGeomFieldIdx() const
     {
         return m_iGeomField;
     }
+
     const FileGDBGeomField *GetGeomField() const
     {
         return (m_iGeomField >= 0) ? cpl::down_cast<FileGDBGeomField *>(
                                          m_apoFields[m_iGeomField].get())
                                    : nullptr;
     }
+
     int GetObjectIdFieldIdx() const
     {
         return m_iObjectIdField;
@@ -608,10 +688,12 @@ class FileGDBTable
     int GetFieldIdx(const std::string &osName) const;
 
     int GetIndexCount();
+
     const FileGDBIndex *GetIndex(int i) const
     {
         return m_apoIndexes[i].get();
     }
+
     bool HasSpatialIndex();
     bool CreateIndex(const std::string &osIndexName,
                      const std::string &osExpression);
@@ -630,18 +712,22 @@ class FileGDBTable
      * returned values */
     int SelectRow(int iRow);
     int GetAndSelectNextNonEmptyRow(int iRow);
+
     int HasGotError() const
     {
         return m_bError;
     }
+
     int GetCurRow() const
     {
         return m_nCurRow;
     }
+
     int IsCurRowDeleted() const
     {
         return m_bIsDeleted;
     }
+
     const OGRField *GetFieldValue(int iCol);
     std::vector<OGRField> GetAllFieldValues();
     void FreeAllFieldValues(std::vector<OGRField> &asFields);
@@ -653,6 +739,7 @@ class FileGDBTable
     {
         return m_adfSpatialIndexGridResolution;
     }
+
     void InstallFilterEnvelope(const OGREnvelope *psFilterEnvelope);
     int DoesGeometryIntersectsFilterEnvelope(const OGRField *psGeomField);
 
@@ -763,7 +850,11 @@ class FileGDBOGRGeometryConverter
     GetGeometryTypeFromESRI(const char *pszESRIGeometryType);
 };
 
-int FileGDBDoubleDateToOGRDate(double dfVal, OGRField *psField);
+int FileGDBDoubleDateToOGRDate(double dfVal, bool bHighPrecision,
+                               OGRField *psField);
+int FileGDBDoubleTimeToOGRTime(double dfVal, OGRField *psField);
+int FileGDBDateTimeWithOffsetToOGRDate(double dfVal, int16_t nUTCOffset,
+                                       OGRField *psField);
 
 } /* namespace OpenFileGDB */
 
