@@ -36,6 +36,7 @@
 #include <cstring>
 #include <cwchar>
 #include <algorithm>
+#include <limits>
 #include <string>
 
 #include "cpl_conv.h"
@@ -52,6 +53,7 @@
 #include "ogrsf_frmts.h"
 #include "filegdbtable.h"
 #include "ogr_swq.h"
+#include "filegdb_coordprec_read.h"
 
 /************************************************************************/
 /*                      OGROpenFileGDBLayer()                           */
@@ -81,6 +83,9 @@ OGROpenFileGDBLayer::OGROpenFileGDBLayer(
     {
         BuildGeometryColumnGDBv10(osParentDefinition);
     }
+
+    // bSealFields = false because we do lazy resolution of fields
+    m_poFeatureDefn->Seal(/* bSealFields = */ false);
 }
 
 /************************************************************************/
@@ -93,7 +98,10 @@ OGROpenFileGDBLayer::OGROpenFileGDBLayer(OGROpenFileGDBDataSource *poDS,
                                          OGRwkbGeometryType eType,
                                          CSLConstList papszOptions)
     : m_poDS(poDS), m_osGDBFilename(pszGDBFilename), m_osName(pszName),
-      m_aosCreationOptions(papszOptions), m_eGeomType(eType)
+      m_aosCreationOptions(papszOptions), m_eGeomType(eType),
+      m_bArcGISPro32OrLater(
+          EQUAL(CSLFetchNameValueDef(papszOptions, "TARGET_ARCGIS_VERSION", ""),
+                "ARCGIS_PRO_3_2_OR_LATER"))
 {
 }
 
@@ -149,7 +157,7 @@ int OGROpenFileGDBLayer::BuildGeometryColumnGDBv10(
 
     CPLStripXMLNamespace(psTree, nullptr, TRUE);
     /* CPLSerializeXMLTreeToFile( psTree, "/dev/stderr" ); */
-    CPLXMLNode *psInfo = CPLSearchXMLNode(psTree, "=DEFeatureClassInfo");
+    const CPLXMLNode *psInfo = CPLSearchXMLNode(psTree, "=DEFeatureClassInfo");
     if (psInfo == nullptr)
         psInfo = CPLSearchXMLNode(psTree, "=DETableInfo");
     if (psInfo == nullptr)
@@ -159,6 +167,12 @@ int OGROpenFileGDBLayer::BuildGeometryColumnGDBv10(
         return FALSE;
     }
 
+    const char *pszAliasName = CPLGetXMLValue(psInfo, "AliasName", nullptr);
+    if (pszAliasName && strcmp(pszAliasName, GetDescription()) != 0)
+    {
+        SetMetadataItem("ALIAS_NAME", pszAliasName);
+    }
+
     m_bTimeInUTC = CPLTestBool(CPLGetXMLValue(psInfo, "IsTimeInUTC", "false"));
 
     /* We cannot trust the XML definition to build the field definitions. */
@@ -206,14 +220,15 @@ int OGROpenFileGDBLayer::BuildGeometryColumnGDBv10(
         if (bHasM)
             m_eGeomType = wkbSetM(m_eGeomType);
 
-        auto poGeomFieldDefn = cpl::make_unique<OGROpenFileGDBGeomFieldDefn>(
+        auto poGeomFieldDefn = std::make_unique<OGROpenFileGDBGeomFieldDefn>(
             nullptr, pszShapeFieldName, m_eGeomType);
 
-        CPLXMLNode *psGPFieldInfoExs = CPLGetXMLNode(psInfo, "GPFieldInfoExs");
+        const CPLXMLNode *psGPFieldInfoExs =
+            CPLGetXMLNode(psInfo, "GPFieldInfoExs");
         if (psGPFieldInfoExs)
         {
-            for (CPLXMLNode *psChild = psGPFieldInfoExs->psChild;
-                 psChild != nullptr; psChild = psChild->psNext)
+            for (const CPLXMLNode *psChild = psGPFieldInfoExs->psChild; psChild;
+                 psChild = psChild->psNext)
             {
                 if (psChild->eType != CXT_Element)
                     continue;
@@ -228,6 +243,14 @@ int OGROpenFileGDBLayer::BuildGeometryColumnGDBv10(
             }
         }
 
+        const CPLXMLNode *psSpatialReference =
+            CPLGetXMLNode(psInfo, "SpatialReference");
+        if (psSpatialReference)
+        {
+            poGeomFieldDefn->SetCoordinatePrecision(
+                GDBGridSettingsToOGR(psSpatialReference));
+        }
+
         OGRSpatialReference *poParentSRS = nullptr;
         if (!osParentDefinition.empty())
         {
@@ -392,6 +415,7 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
     }
 
     m_bValidLayerDefn = TRUE;
+    auto oTemporaryUnsealer(m_poFeatureDefn->GetTemporaryUnsealer());
 
     m_iGeomFieldIdx = m_poLyrTable->GetGeomFieldIdx();
     if (m_iGeomFieldIdx >= 0)
@@ -501,7 +525,7 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
 
         {
             auto poGeomFieldDefn =
-                cpl::make_unique<OGROpenFileGDBGeomFieldDefn>(nullptr, pszName,
+                std::make_unique<OGROpenFileGDBGeomFieldDefn>(nullptr, pszName,
                                                               m_eGeomType);
             poGeomFieldDefn->SetNullable(poGDBGeomField->IsNullable());
 
@@ -559,7 +583,7 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
         if (i == m_poLyrTable->GetObjectIdFieldIdx())
             continue;
 
-        const FileGDBField *poGDBField = m_poLyrTable->GetField(i);
+        FileGDBField *poGDBField = m_poLyrTable->GetField(i);
         OGRFieldType eType = OFTString;
         OGRFieldSubType eSubType = OFSTNone;
         int nWidth = poGDBField->GetMaxWidth();
@@ -626,15 +650,32 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
                     eType = OFTBinary;
                 break;
             }
+            case FGFT_INT64:
+                m_bArcGISPro32OrLater = true;
+                eType = OFTInteger64;
+                break;
+            case FGFT_DATE:
+                m_bArcGISPro32OrLater = true;
+                eType = OFTDate;
+                break;
+            case FGFT_TIME:
+                m_bArcGISPro32OrLater = true;
+                eType = OFTTime;
+                break;
+            case FGFT_DATETIME_WITH_OFFSET:
+                m_bArcGISPro32OrLater = true;
+                eType = OFTDateTime;
+                break;
         }
         OGRFieldDefn oFieldDefn(poGDBField->GetName().c_str(), eType);
         oFieldDefn.SetAlternativeName(poGDBField->GetAlias().c_str());
         oFieldDefn.SetSubType(eSubType);
         // On creation in the FileGDB driver (GDBFieldTypeToLengthInBytes) if
-        // string width is 0, we pick up 65536 by default to mean unlimited
-        // string length, but we do not want to advertise such a big number.
+        // string width is 0, we pick up DEFAULT_STRING_WIDTH=65536 by default
+        // to mean unlimited string length, but we do not want to advertise
+        // such a big number.
         if (eType == OFTString &&
-            (nWidth < 65536 ||
+            (nWidth < DEFAULT_STRING_WIDTH ||
              CPLTestBool(CPLGetConfigOption(
                  "OPENFILEGDB_REPORT_GENUINE_FIELD_WIDTH", "NO"))))
         {
@@ -660,6 +701,14 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
             }
         }
 
+        if (psFieldDef && poGDBField->GetType() == FGFT_DATETIME)
+        {
+            if (EQUAL(CPLGetXMLValue(psFieldDef, "HighPrecision", ""), "true"))
+            {
+                poGDBField->SetHighPrecision();
+            }
+        }
+
         const OGRField *psDefault = poGDBField->GetDefault();
         if (!OGR_RawField_IsUnset(psDefault) && !OGR_RawField_IsNull(psDefault))
         {
@@ -673,7 +722,8 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
                 osDefault += "'";
                 oFieldDefn.SetDefault(osDefault);
             }
-            else if (eType == OFTInteger || eType == OFTReal)
+            else if (eType == OFTInteger || eType == OFTReal ||
+                     eType == OFTInteger64)
             {
                 // GDBs and the FileGDB SDK are not always reliable for
                 // numeric values It often occurs that the XML definition in
@@ -694,6 +744,11 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
                     if (pszDefaultValue == nullptr)
                         pszDefaultValue =
                             CPLGetXMLValue(psFieldDef, "DefaultValue", nullptr);
+                    // For ArcGIS Pro 3.2 and esriFieldTypeBigInteger, this is
+                    // DefaultValueInteger
+                    if (pszDefaultValue == nullptr)
+                        pszDefaultValue = CPLGetXMLValue(
+                            psFieldDef, "DefaultValueInteger", nullptr);
                 }
                 if (pszDefaultValue != nullptr)
                 {
@@ -726,14 +781,54 @@ int OGROpenFileGDBLayer::BuildLayerDefinition()
                         }
                         oFieldDefn.SetDefault(pszDefaultValue);
                     }
+                    else if (eType == OFTInteger64)
+                    {
+                        if (CPLAtoGIntBig(pszDefaultValue) !=
+                            psDefault->Integer64)
+                        {
+                            CPLDebug(
+                                "OpenFileGDB",
+                                "For field %s, XML definition mentions %s "
+                                "as default value whereas .gdbtable header "
+                                "mentions " CPL_FRMT_GIB ". Using %s",
+                                poGDBField->GetName().c_str(), pszDefaultValue,
+                                psDefault->Integer64, pszDefaultValue);
+                        }
+                        oFieldDefn.SetDefault(pszDefaultValue);
+                    }
                 }
             }
             else if (eType == OFTDateTime)
-                oFieldDefn.SetDefault(CPLSPrintf(
-                    "'%04d/%02d/%02d %02d:%02d:%02d'", psDefault->Date.Year,
-                    psDefault->Date.Month, psDefault->Date.Day,
-                    psDefault->Date.Hour, psDefault->Date.Minute,
-                    static_cast<int>(psDefault->Date.Second)));
+            {
+                if (poGDBField->GetType() == FGFT_DATETIME_WITH_OFFSET)
+                {
+                    oFieldDefn.SetDefault(CPLSPrintf(
+                        "'%04d/%02d/%02d %02d:%02d:%06.03f%c%02d:%02d'",
+                        psDefault->Date.Year, psDefault->Date.Month,
+                        psDefault->Date.Day, psDefault->Date.Hour,
+                        psDefault->Date.Minute, psDefault->Date.Second,
+                        psDefault->Date.TZFlag >= 100 ? '+' : '-',
+                        std::abs(psDefault->Date.TZFlag - 100) / 4,
+                        (std::abs(psDefault->Date.TZFlag - 100) % 4) * 15));
+                }
+                else
+                {
+                    oFieldDefn.SetDefault(CPLSPrintf(
+                        "'%04d/%02d/%02d %02d:%02d:%02d'", psDefault->Date.Year,
+                        psDefault->Date.Month, psDefault->Date.Day,
+                        psDefault->Date.Hour, psDefault->Date.Minute,
+                        static_cast<int>(psDefault->Date.Second)));
+                }
+            }
+            else if (eType == OFTDate)
+                oFieldDefn.SetDefault(
+                    CPLSPrintf("'%04d/%02d/%02d'", psDefault->Date.Year,
+                               psDefault->Date.Month, psDefault->Date.Day));
+            else if (eType == OFTTime)
+                oFieldDefn.SetDefault(
+                    CPLSPrintf("'%02d:%02d:%02d'", psDefault->Date.Hour,
+                               psDefault->Date.Minute,
+                               static_cast<int>(psDefault->Date.Second)));
         }
 
         if (psFieldDef)
@@ -1083,6 +1178,14 @@ static int FillTargetValueFromSrcExpr(OGRFieldDefn *poFieldDefn,
                     static_cast<int>(poSrcValue->int_value);
             break;
 
+        case OFTInteger64:
+            if (poSrcValue->field_type == SWQ_FLOAT)
+                poTargetValue->Integer64 =
+                    static_cast<GIntBig>(poSrcValue->float_value);
+            else
+                poTargetValue->Integer64 = poSrcValue->int_value;
+            break;
+
         case OFTReal:
             poTargetValue->Real = poSrcValue->float_value;
             break;
@@ -1161,6 +1264,7 @@ static swq_expr_node *GetConstantSubNode(swq_expr_node *poNode)
     }
     return nullptr;
 }
+
 /***********************************************************************/
 /*                     BuildIteratorFromExprNode()                     */
 /***********************************************************************/
@@ -1701,7 +1805,11 @@ OGRFeature *OGROpenFileGDBLayer::GetCurrentFeature()
                     else if (poFieldDefn->GetType() == OFTDateTime)
                     {
                         OGRField sField = *psField;
-                        sField.Date.TZFlag = m_bTimeInUTC ? 100 : 0;
+                        if (m_poLyrTable->GetField(iGDBIdx)->GetType() ==
+                            FGFT_DATETIME)
+                        {
+                            sField.Date.TZFlag = m_bTimeInUTC ? 100 : 0;
+                        }
                         poFeature->SetField(iOGRIdx, &sField);
                     }
                     else
@@ -1926,6 +2034,48 @@ OGRErr OGROpenFileGDBLayer::GetExtent(OGREnvelope *psExtent, int /* bForce */)
     return OGRERR_FAILURE;
 }
 
+/***********************************************************************/
+/*                           GetExtent3D()                             */
+/***********************************************************************/
+
+OGRErr OGROpenFileGDBLayer::GetExtent3D(int iGeomField, OGREnvelope3D *psExtent,
+                                        int bForce)
+{
+    if (!BuildLayerDefinition())
+        return OGRERR_FAILURE;
+
+    if (m_poFilterGeom == nullptr && m_poAttrQuery == nullptr &&
+        m_iGeomFieldIdx >= 0 && m_poLyrTable->GetValidRecordCount() > 0)
+    {
+        FileGDBGeomField *poGDBGeomField = reinterpret_cast<FileGDBGeomField *>(
+            m_poLyrTable->GetField(m_iGeomFieldIdx));
+        if (!std::isnan(poGDBGeomField->GetXMin()))
+        {
+            psExtent->MinX = poGDBGeomField->GetXMin();
+            psExtent->MinY = poGDBGeomField->GetYMin();
+            psExtent->MaxX = poGDBGeomField->GetXMax();
+            psExtent->MaxY = poGDBGeomField->GetYMax();
+            if (!std::isnan(poGDBGeomField->GetZMin()))
+            {
+                psExtent->MinZ = poGDBGeomField->GetZMin();
+                psExtent->MaxZ = poGDBGeomField->GetZMax();
+            }
+            else
+            {
+                if (OGR_GT_HasZ(m_eGeomType))
+                {
+                    return OGRLayer::GetExtent3D(iGeomField, psExtent, bForce);
+                }
+                psExtent->MinZ = std::numeric_limits<double>::infinity();
+                psExtent->MaxZ = -std::numeric_limits<double>::infinity();
+            }
+            return OGRERR_NONE;
+        }
+    }
+
+    return OGRLayer::GetExtent3D(iGeomField, psExtent, bForce);
+}
+
 /***********************************************************************/
 /*                         GetFeatureCount()                           */
 /***********************************************************************/
@@ -2114,6 +2264,28 @@ int OGROpenFileGDBLayer::TestCapability(const char *pszCap)
     {
         return TRUE;
     }
+    else if (EQUAL(pszCap, OLCFastGetExtent3D))
+    {
+        if (m_poFilterGeom == nullptr && m_poAttrQuery == nullptr &&
+            m_iGeomFieldIdx >= 0 && m_poLyrTable->GetValidRecordCount() > 0)
+        {
+            FileGDBGeomField *poGDBGeomField =
+                reinterpret_cast<FileGDBGeomField *>(
+                    m_poLyrTable->GetField(m_iGeomFieldIdx));
+            if (!std::isnan(poGDBGeomField->GetXMin()))
+            {
+                if (!std::isnan(poGDBGeomField->GetZMin()))
+                {
+                    return TRUE;
+                }
+                else
+                {
+                    return !OGR_GT_HasZ(m_eGeomType);
+                }
+            }
+        }
+        return FALSE;
+    }
     else if (EQUAL(pszCap, OLCIgnoreFields))
     {
         return TRUE;
@@ -2271,3 +2443,12 @@ int OGROpenFileGDBLayer::GetMinMaxSumCount(OGRFieldDefn *poFieldDefn,
     }
     return FALSE;
 }
+
+/************************************************************************/
+/*                             GetDataset()                             */
+/************************************************************************/
+
+GDALDataset *OGROpenFileGDBLayer::GetDataset()
+{
+    return m_poDS;
+}
