@@ -366,9 +366,9 @@ static double OGR2SQLITE_GetValAsDouble(sqlite3_value *val, int *pbGotVal)
 /*                      OGR2SQLITE_GetGeom()                            */
 /************************************************************************/
 
-static OGRGeometry *OGR2SQLITE_GetGeom(CPL_UNUSED sqlite3_context *pContext,
-                                       CPL_UNUSED int argc,
-                                       sqlite3_value **argv, int *pnSRSId)
+static std::unique_ptr<OGRGeometry>
+OGR2SQLITE_GetGeom(sqlite3_context * /*pContext*/, int /* argc */,
+                   sqlite3_value **argv, int *pnSRSId)
 {
     if (sqlite3_value_type(argv[0]) != SQLITE_BLOB)
     {
@@ -381,12 +381,11 @@ static OGRGeometry *OGR2SQLITE_GetGeom(CPL_UNUSED sqlite3_context *pContext,
     if (OGRSQLiteLayer::ImportSpatiaLiteGeometry(pabySLBLOB, nBLOBLen, &poGeom,
                                                  pnSRSId) != OGRERR_NONE)
     {
-        if (poGeom != nullptr)
-            delete poGeom;
+        delete poGeom;
         return nullptr;
     }
 
-    return poGeom;
+    return std::unique_ptr<OGRGeometry>(poGeom);
 }
 
 /************************************************************************/
@@ -419,18 +418,16 @@ static void OGR2SQLITE_ogr_geocode_reverse(sqlite3_context *pContext, int argc,
     else if (argc >= 2 && sqlite3_value_type(argv[0]) == SQLITE_BLOB &&
              sqlite3_value_type(argv[1]) == SQLITE_TEXT)
     {
-        OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
+        auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
         if (poGeom != nullptr &&
             wkbFlatten(poGeom->getGeometryType()) == wkbPoint)
         {
-            OGRPoint *poPoint = poGeom->toPoint();
+            const OGRPoint *poPoint = poGeom->toPoint();
             dfLon = poPoint->getX();
             dfLat = poPoint->getY();
-            delete poGeom;
         }
         else
         {
-            delete poGeom;
             sqlite3_result_null(pContext);
             return;
         }
@@ -629,6 +626,51 @@ static void OGR2SQLITE_SetGeom_AndDestroy(sqlite3_context *pContext,
     delete poGeom;
 }
 
+/************************************************************************/
+/*                     OGR2SQLITE_ST_GeodesicArea()                     */
+/************************************************************************/
+
+static void OGR2SQLITE_ST_GeodesicArea(sqlite3_context *pContext, int argc,
+                                       sqlite3_value **argv)
+{
+    if (sqlite3_value_int(argv[1]) != 1)
+    {
+        CPLError(CE_Warning, CPLE_NotSupported,
+                 "ST_Area(geom, use_ellipsoid) is only supported for "
+                 "use_ellipsoid = 1");
+    }
+
+    int nSRSId = -1;
+    auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
+    if (poGeom != nullptr)
+    {
+        OGRSpatialReference oSRS;
+        oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+        if (nSRSId > 0)
+        {
+            if (oSRS.importFromEPSG(nSRSId) != OGRERR_NONE)
+            {
+                sqlite3_result_null(pContext);
+                return;
+            }
+        }
+        else
+        {
+            CPLDebug("OGR_SQLITE",
+                     "Assuming EPSG:4326 for GeodesicArea() computation");
+            oSRS.importFromEPSG(4326);
+        }
+        poGeom->assignSpatialReference(&oSRS);
+        sqlite3_result_double(
+            pContext, OGR_G_GeodesicArea(OGRGeometry::ToHandle(poGeom.get())));
+        poGeom->assignSpatialReference(nullptr);
+    }
+    else
+    {
+        sqlite3_result_null(pContext);
+    }
+}
+
 #ifdef MINIMAL_SPATIAL_FUNCTIONS
 
 /************************************************************************/
@@ -638,7 +680,7 @@ static void OGR2SQLITE_SetGeom_AndDestroy(sqlite3_context *pContext,
 static void OGR2SQLITE_ST_AsText(sqlite3_context *pContext, int argc,
                                  sqlite3_value **argv)
 {
-    OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
+    auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
     if (poGeom != nullptr)
     {
         char *pszWKT = nullptr;
@@ -646,7 +688,6 @@ static void OGR2SQLITE_ST_AsText(sqlite3_context *pContext, int argc,
             sqlite3_result_text(pContext, pszWKT, -1, CPLFree);
         else
             sqlite3_result_null(pContext);
-        delete poGeom;
     }
     else
         sqlite3_result_null(pContext);
@@ -659,8 +700,7 @@ static void OGR2SQLITE_ST_AsText(sqlite3_context *pContext, int argc,
 static void OGR2SQLITE_ST_AsBinary(sqlite3_context *pContext, int argc,
                                    sqlite3_value **argv)
 {
-    OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
-    if (poGeom != nullptr)
+    if (auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr))
     {
         const size_t nBLOBLen = poGeom->WkbSize();
         if (nBLOBLen > static_cast<size_t>(std::numeric_limits<int>::max()))
@@ -683,7 +723,6 @@ static void OGR2SQLITE_ST_AsBinary(sqlite3_context *pContext, int argc,
         }
         else
             sqlite3_result_null(pContext);
-        delete poGeom;
     }
     else
         sqlite3_result_null(pContext);
@@ -752,31 +791,20 @@ static void OGR2SQLITE_ST_GeomFromWKB(sqlite3_context *pContext, int argc,
 /*                         CheckSTFunctions()                           */
 /************************************************************************/
 
-static int CheckSTFunctions(sqlite3_context *pContext, int argc,
-                            sqlite3_value **argv, OGRGeometry **ppoGeom1,
-                            OGRGeometry **ppoGeom2, int *pnSRSId)
+static bool CheckSTFunctions(sqlite3_context *pContext, int argc,
+                             sqlite3_value **argv,
+                             std::unique_ptr<OGRGeometry> &poGeom1,
+                             std::unique_ptr<OGRGeometry> &poGeom2,
+                             int *pnSRSId)
 {
-    *ppoGeom1 = nullptr;
-    *ppoGeom2 = nullptr;
-
     if (argc != 2)
     {
-        return FALSE;
+        return false;
     }
 
-    *ppoGeom1 = OGR2SQLITE_GetGeom(pContext, argc, argv, pnSRSId);
-    if (*ppoGeom1 == nullptr)
-        return FALSE;
-
-    *ppoGeom2 = OGR2SQLITE_GetGeom(pContext, argc - 1, argv + 1, nullptr);
-    if (*ppoGeom2 == nullptr)
-    {
-        delete *ppoGeom1;
-        *ppoGeom1 = nullptr;
-        return FALSE;
-    }
-
-    return TRUE;
+    poGeom1 = OGR2SQLITE_GetGeom(pContext, argc, argv, pnSRSId);
+    poGeom2 = OGR2SQLITE_GetGeom(pContext, argc - 1, argv + 1, nullptr);
+    return poGeom1 && poGeom2;
 }
 
 /************************************************************************/
@@ -787,27 +815,28 @@ static int CheckSTFunctions(sqlite3_context *pContext, int argc,
     static void OGR2SQLITE_ST_##op(sqlite3_context *pContext, int argc,        \
                                    sqlite3_value **argv)                       \
     {                                                                          \
-        OGRGeometry *poGeom1 = nullptr;                                        \
-        OGRGeometry *poGeom2 = nullptr;                                        \
-        if (!CheckSTFunctions(pContext, argc, argv, &poGeom1, &poGeom2,        \
+        std::unique_ptr<OGRGeometry> poGeom1;                                  \
+        std::unique_ptr<OGRGeometry> poGeom2;                                  \
+        if (!CheckSTFunctions(pContext, argc, argv, poGeom1, poGeom2,          \
                               nullptr))                                        \
         {                                                                      \
             sqlite3_result_int(pContext, 0);                                   \
             return;                                                            \
         }                                                                      \
                                                                                \
-        sqlite3_result_int(pContext, poGeom1->op(poGeom2));                    \
-                                                                               \
-        delete poGeom1;                                                        \
-        delete poGeom2;                                                        \
+        sqlite3_result_int(pContext, poGeom1->op(poGeom2.get()));              \
     }
 
-OGR2SQLITE_ST_int_geomgeom_op(Intersects) OGR2SQLITE_ST_int_geomgeom_op(Equals)
-    OGR2SQLITE_ST_int_geomgeom_op(Disjoint) OGR2SQLITE_ST_int_geomgeom_op(
-        Touches) OGR2SQLITE_ST_int_geomgeom_op(Crosses)
-        OGR2SQLITE_ST_int_geomgeom_op(Within)
-            OGR2SQLITE_ST_int_geomgeom_op(Contains)
-                OGR2SQLITE_ST_int_geomgeom_op(Overlaps)
+// clang-format off
+OGR2SQLITE_ST_int_geomgeom_op(Intersects)
+OGR2SQLITE_ST_int_geomgeom_op(Equals)
+OGR2SQLITE_ST_int_geomgeom_op(Disjoint)
+OGR2SQLITE_ST_int_geomgeom_op(Touches)
+OGR2SQLITE_ST_int_geomgeom_op(Crosses)
+OGR2SQLITE_ST_int_geomgeom_op(Within)
+OGR2SQLITE_ST_int_geomgeom_op(Contains)
+OGR2SQLITE_ST_int_geomgeom_op(Overlaps)
+// clang-format on
 
 /************************************************************************/
 /*                   OGR2SQLITE_ST_int_geom_op()                        */
@@ -817,19 +846,18 @@ OGR2SQLITE_ST_int_geomgeom_op(Intersects) OGR2SQLITE_ST_int_geomgeom_op(Equals)
     static void OGR2SQLITE_ST_##op(sqlite3_context *pContext, int argc,        \
                                    sqlite3_value **argv)                       \
     {                                                                          \
-        OGRGeometry *poGeom =                                                  \
-            OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);                 \
+        auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);       \
         if (poGeom != nullptr)                                                 \
             sqlite3_result_int(pContext, poGeom->op());                        \
         else                                                                   \
             sqlite3_result_int(pContext, 0);                                   \
-                                                                               \
-        delete poGeom;                                                         \
     }
 
-                    OGR2SQLITE_ST_int_geom_op(IsEmpty)
-                        OGR2SQLITE_ST_int_geom_op(IsSimple)
-                            OGR2SQLITE_ST_int_geom_op(IsValid)
+    // clang-format off
+OGR2SQLITE_ST_int_geom_op(IsEmpty)
+OGR2SQLITE_ST_int_geom_op(IsSimple)
+OGR2SQLITE_ST_int_geom_op(IsValid)
+// clang-format on
 
 /************************************************************************/
 /*                  OGR2SQLITE_ST_geom_geomgeom_op()                    */
@@ -839,27 +867,26 @@ OGR2SQLITE_ST_int_geomgeom_op(Intersects) OGR2SQLITE_ST_int_geomgeom_op(Equals)
     static void OGR2SQLITE_ST_##op(sqlite3_context *pContext, int argc,        \
                                    sqlite3_value **argv)                       \
     {                                                                          \
-        OGRGeometry *poGeom1 = nullptr;                                        \
-        OGRGeometry *poGeom2 = nullptr;                                        \
+        std::unique_ptr<OGRGeometry> poGeom1;                                  \
+        std::unique_ptr<OGRGeometry> poGeom2;                                  \
         int nSRSId = -1;                                                       \
-        if (!CheckSTFunctions(pContext, argc, argv, &poGeom1, &poGeom2,        \
+        if (!CheckSTFunctions(pContext, argc, argv, poGeom1, poGeom2,          \
                               &nSRSId))                                        \
         {                                                                      \
             sqlite3_result_null(pContext);                                     \
             return;                                                            \
         }                                                                      \
                                                                                \
-        OGR2SQLITE_SetGeom_AndDestroy(pContext, poGeom1->op(poGeom2), nSRSId); \
-                                                                               \
-        delete poGeom1;                                                        \
-        delete poGeom2;                                                        \
+        OGR2SQLITE_SetGeom_AndDestroy(pContext, poGeom1->op(poGeom2.get()),    \
+                                      nSRSId);                                 \
     }
 
-                                OGR2SQLITE_ST_geom_geomgeom_op(Intersection)
-                                    OGR2SQLITE_ST_geom_geomgeom_op(Difference)
-                                        OGR2SQLITE_ST_geom_geomgeom_op(Union)
-                                            OGR2SQLITE_ST_geom_geomgeom_op(
-                                                SymDifference)
+    // clang-format off
+OGR2SQLITE_ST_geom_geomgeom_op(Intersection)
+OGR2SQLITE_ST_geom_geomgeom_op(Difference)
+OGR2SQLITE_ST_geom_geomgeom_op(Union)
+OGR2SQLITE_ST_geom_geomgeom_op(SymDifference)
+    // clang-format on
 
     /************************************************************************/
     /*                      OGR2SQLITE_ST_SRID()                            */
@@ -869,7 +896,7 @@ OGR2SQLITE_ST_int_geomgeom_op(Intersects) OGR2SQLITE_ST_int_geomgeom_op(Equals)
                                    sqlite3_value **argv)
 {
     int nSRSId = -1;
-    OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
+    auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
     if (poGeom != nullptr)
     {
         CPLPushErrorHandler(CPLQuietErrorHandler);
@@ -878,7 +905,6 @@ OGR2SQLITE_ST_int_geomgeom_op(Intersects) OGR2SQLITE_ST_int_geomgeom_op(Equals)
     }
     else
         sqlite3_result_null(pContext);
-    delete poGeom;
 }
 
 /************************************************************************/
@@ -888,16 +914,16 @@ OGR2SQLITE_ST_int_geomgeom_op(Intersects) OGR2SQLITE_ST_int_geomgeom_op(Equals)
 static void OGR2SQLITE_ST_Area(sqlite3_context *pContext, int argc,
                                sqlite3_value **argv)
 {
-    OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
+    auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, nullptr);
     if (poGeom != nullptr)
     {
         CPLPushErrorHandler(CPLQuietErrorHandler);
-        sqlite3_result_double(pContext, OGR_G_Area((OGRGeometryH)poGeom));
+        sqlite3_result_double(pContext,
+                              OGR_G_Area(OGRGeometry::ToHandle(poGeom.get())));
         CPLPopErrorHandler();
     }
     else
         sqlite3_result_null(pContext);
-    delete poGeom;
 }
 
 /************************************************************************/
@@ -908,14 +934,13 @@ static void OGR2SQLITE_ST_Buffer(sqlite3_context *pContext, int argc,
                                  sqlite3_value **argv)
 {
     int nSRSId = -1;
-    OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
+    auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
     int bGotVal = FALSE;
     double dfDist = OGR2SQLITE_GetValAsDouble(argv[1], &bGotVal);
     if (poGeom != nullptr && bGotVal)
         OGR2SQLITE_SetGeom_AndDestroy(pContext, poGeom->Buffer(dfDist), nSRSId);
     else
         sqlite3_result_null(pContext);
-    delete poGeom;
 }
 
 /************************************************************************/
@@ -966,12 +991,11 @@ static void OGR2SQLITE_ST_MakeValid(sqlite3_context *pContext, int argc,
                                     sqlite3_value **argv)
 {
     int nSRSId = -1;
-    OGRGeometry *poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
+    auto poGeom = OGR2SQLITE_GetGeom(pContext, argc, argv, &nSRSId);
     if (poGeom != nullptr)
         OGR2SQLITE_SetGeom_AndDestroy(pContext, poGeom->MakeValid(), nSRSId);
     else
         sqlite3_result_null(pContext);
-    delete poGeom;
 }
 
 /************************************************************************/
@@ -1010,7 +1034,9 @@ static void OGRSQLITE_hstore_get_value(sqlite3_context *pContext,
 #define SQLITE_INNOCUOUS 0
 #endif
 
+#ifndef UTF8_INNOCUOUS
 #define UTF8_INNOCUOUS (SQLITE_UTF8 | SQLITE_DETERMINISTIC | SQLITE_INNOCUOUS)
+#endif
 
 static void *OGRSQLiteRegisterSQLFunctions(sqlite3 *hDB)
 {
@@ -1130,6 +1156,13 @@ static void *OGRSQLiteRegisterSQLFunctions(sqlite3 *hDB)
 
     if (bAllowOGRSQLiteSpatialFunctions)
     {
+        // We add a ST_Area() method with 2 arguments even when Spatialite
+        // is there to indicate we want to use the ellipsoid version
+        sqlite3_create_function(hDB, "Area", 2, UTF8_INNOCUOUS, nullptr,
+                                OGR2SQLITE_ST_GeodesicArea, nullptr, nullptr);
+        sqlite3_create_function(hDB, "ST_Area", 2, UTF8_INNOCUOUS, nullptr,
+                                OGR2SQLITE_ST_GeodesicArea, nullptr, nullptr);
+
         static bool gbRegisterMakeValid = [bSpatialiteAvailable, hDB]()
         {
             bool bRegisterMakeValid = false;
@@ -1154,11 +1187,7 @@ static void *OGRSQLiteRegisterSQLFunctions(sqlite3 *hDB)
             }
             if (bRegisterMakeValid)
             {
-                OGRPoint p(0, 0);
-                CPLErrorStateBackuper oBackuper;
-                CPLErrorHandlerPusher oPusher(CPLQuietErrorHandler);
-                auto validGeom = std::unique_ptr<OGRGeometry>(p.MakeValid());
-                return validGeom != nullptr;
+                return OGRGeometryFactory::haveGEOS();
             }
             return false;
         }();
