@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: ogrsqlitevirtualogr.h 978b70fdbf1a464955d8999e0cbecb3826c9c1ab 2017-03-01 16:50:58Z Even Rouault $
+ * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  SQLite Virtual Table module using OGR layers
- * Author:   Even Rouault, even dot rouault at mines dash paris dot org
+ * Author:   Even Rouault, even dot rouault at spatialys.com
  *
  ******************************************************************************
- * Copyright (c) 2012, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2012, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,13 +34,15 @@
 
 class OGR2SQLITEModule;
 
-OGR2SQLITEModule* OGR2SQLITE_Setup(GDALDataset* poDS,
-                                   OGRSQLiteDataSource* poSQLiteDS);
+OGR2SQLITEModule *OGR2SQLITE_Setup(GDALDataset *poDS,
+                                   OGRSQLiteDataSource *poSQLiteDS);
 
-int OGR2SQLITE_AddExtraDS(OGR2SQLITEModule* poModule, OGRDataSource* poDS);
+void OGR2SQLITE_SetCaseSensitiveLike(OGR2SQLITEModule *poModule, bool b);
+
+int OGR2SQLITE_AddExtraDS(OGR2SQLITEModule *poModule, OGRDataSource *poDS);
 
 void OGR2SQLITE_Register();
 
-CPLString OGR2SQLITE_GetNameForGeometryColumn(OGRLayer* poLayer);
+CPLString OGR2SQLITE_GetNameForGeometryColumn(OGRLayer *poLayer);
 
-#endif // OGR_SQLITE_VIRTUAL_OGR_H_INCLUDED
+#endif  // OGR_SQLITE_VIRTUAL_OGR_H_INCLUDED
