@@ -49,7 +49,7 @@
 #include "cpl_error.h"
 #include "cpl_vsi_virtual.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <io.h>
 #include <fcntl.h>
 #endif
@@ -72,7 +72,7 @@ static void VSIStdinInit()
 {
     if (gpabyBuffer == nullptr)
     {
-#ifdef WIN32
+#ifdef _WIN32
         setmode(fileno(stdin), O_BINARY);
 #endif
         constexpr size_t MAX_INITIAL_ALLOC = 1024 * 1024;
@@ -100,11 +100,13 @@ class VSIStdinFilesystemHandler final : public VSIFilesystemHandler
                            CSLConstList /* papszOptions */) override;
     int Stat(const char *pszFilename, VSIStatBufL *pStatBuf,
              int nFlags) override;
+
     bool SupportsSequentialWrite(const char * /* pszPath */,
                                  bool /* bAllowLocalTempFile */) override
     {
         return false;
     }
+
     bool SupportsRandomWrite(const char * /* pszPath */,
                              bool /* bAllowLocalTempFile */) override
     {
@@ -129,6 +131,7 @@ class VSIStdinHandle final : public VSIVirtualHandle
 
   public:
     VSIStdinHandle() = default;
+
     ~VSIStdinHandle() override
     {
         VSIStdinHandle::Close();
